{---------------------------------------------------------------------

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/01/2006 version

----------------------------------------------------------------------}

unit Dialogs0;

interface

uses
  Classes, Forms;

type
  TCommonDialogSimple = class(TComponent)
  private
    FHelpContext: THelpContext;
    procedure MkFormHelp;
  protected
    F: TForm;
    procedure MkForm; virtual; abstract;
    procedure PostShowModal (Res: Integer); virtual;
  public
    function  ShowModal: Integer;
  published
    property HelpContext: THelpContext read FHelpContext write FHelpContext default 0;
  end;

  TCommonDialogSimpleWS = class(TCommonDialogSimple)
  private
    procedure GetF;
  protected
    function GetFormClass: TFormClass; virtual; abstract;
  public
    procedure Show;
    procedure PerhapsClose;
  end;

  TPersComDialSimple = class(TPersistent)
  private
    FHelpContext: THelpContext;
    procedure MkFormHelp;
  protected
    F: TForm;
    procedure MkForm; virtual; abstract;
    procedure PostShowModal (Res: Integer); virtual;
  public
    function  ShowModal: Integer;
  published
    property HelpContext: THelpContext read FHelpContext write FHelpContext default 0;
  end;

  TPersComDialSimpleSizeable = class(TPersComDialSimple)
  private
    W, H: Integer;
  protected
    procedure MkForm; override;
    procedure PostShowModal (Res: Integer); override;
  end;

  TPersComDialSimpleWS = class(TPersComDialSimple)
  private
    procedure GetF;
  protected
    function GetFormClass: TFormClass; virtual; abstract;
  public
    function  CurrForm: TForm;
    procedure Show;
    procedure PerhapsClose;
  end;

implementation

{--------------------------------------}
{-PRIVATE----------}

procedure TCommonDialogSimple.MkFormHelp;
begin
  MkForm;
  F.HelpContext := HelpContext;
end {MkFormHelp};

{-PROTECTED--------}

procedure TCommonDialogSimple.PostShowModal (Res: Integer);
begin
  F.Free;
  F := nil;
end {PostShowModal};

{-PUBLIC-----------}

function TCommonDialogSimple.ShowModal: Integer;
var
  Res: Integer;
begin
  MkFormHelp;
  Res := F.ShowModal;
  PostShowModal (Res);
  Result := Res;
end {ShowModal};

{--------------------------------------}
{-PRIVATE----------}

procedure TCommonDialogSimpleWS.GetF;
begin
  F := Application.FindComponent (Copy (GetFormClass.ClassName,2,255)) as TForm;
end;

{-PUBLIC-----------}

procedure TCommonDialogSimpleWS.Show;
var
  CreateHere: Boolean;
begin
  GetF;
  CreateHere := not Assigned(F);
  if CreateHere
  then  MkFormHelp;
  if  (not CreateHere) and (F.WindowState = wsMinimized)
  then  F.WindowState := wsNormal;
  F.Show;
end {Show};

procedure TCommonDialogSimpleWS.PerhapsClose;
begin
  GetF;
  if Assigned(F)
  then F.Close;
end;

{--------------------------------------}
{-PRIVATE----------}

procedure TPersComDialSimple.MkFormHelp;
begin
  MkForm;
  F.HelpContext := HelpContext;
end {MkFormHelp};

{-PROTECTED--------}

procedure TPersComDialSimple.PostShowModal (Res: Integer);
begin
  F.Free;
  F := nil;
end {PostShowModal};

{-PUBLIC-----------}

function TPersComDialSimple.ShowModal: Integer;
var
  Res: Integer;
begin
  MkFormHelp;
  Res := F.ShowModal;
  PostShowModal (Res);
  Result := Res;
end {ShowModal};

{--------------------------------------}
{-PROTECTED--------}

procedure TPersComDialSimpleSizeable.MkForm;
begin
  if  W <> 0  then  begin
    F.Width := W;
    F.Height := H;
  end;
end;

procedure TPersComDialSimpleSizeable.PostShowModal(Res: Integer);
begin
  W := F.Width;
  H := F.Height;
end;

{--------------------------------------}
{-PRIVATE----------}

procedure TPersComDialSimpleWS.GetF;
begin
  F := Application.FindComponent (Copy (GetFormClass.ClassName,2,255)) as TForm;
end;

{-PUBLIC-----------}

function TPersComDialSimpleWS.CurrForm: TForm;
begin
  GetF;
  Result := F;
end;

procedure TPersComDialSimpleWS.Show;
var
  CreateHere: Boolean;
begin
  GetF;
  CreateHere := not Assigned(F);
  if CreateHere
  then  MkFormHelp;
  if  (not CreateHere) and (F.WindowState = wsMinimized)
  then  F.WindowState := wsNormal;
  F.Show;
end {Show};

procedure TPersComDialSimpleWS.PerhapsClose;
begin
  GetF;
  if Assigned(F)
  then F.Close;
end;

{--------------------------------------}

end.
