{---------------------------------------------------------------------

 Several General Types and Functions

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/07/2006 version

----------------------------------------------------------------------}

unit StrAnal0;   

interface

uses
  Classes;

function PosChForward (const S: string;  Delim: Char): Integer;
function PosChBackward (const S: string;  Delim: Char): Integer;
function StrUntilDelimiter (var InStr: string;  const Delim: string): string;
function StrBkUntilDelimiter (var InStr: string;  Delim: Char): string;
function StrBkUntilSDelimiter (var InStr: string;  const Delim: string): string;
function StrUntilDelimiters (var InStr: string;  const DelStr: string): string;
function StrUntilDelimitersAndDelimiter (var InStr: string;
  const DelStr: string;  out Delimiter: Char): string;
function StrUntilNoDelimiters (var InStr: string;
  const DelStr: string): string;
function StrDelimitersXAtBegin (const InStr, DelStr: string): Boolean;
function StrUntilDelimitersX (var InStr: string;  const DelStr: string): string;
function StrUntilDelimitersXAndDelimiter (var InStr: string;
  const DelStr: string;  out Delimiter: Char): string;
function StrReplaceSimple (InStr: string;
  const SearchStr, ReplaceStr: string): string;
function StrReplacedSimple (var S: string;
  const SearchStr, ReplaceStr: string): Boolean;
function StrReplaceText (InStr: string;  const SearchStr, ReplaceStr: string;
  var Changed: Boolean;  CaseSensitive: Boolean): string;
function StrReplaceBetweenDelim (InStr: string;
  const SearchStr, ReplaceStr, DelStr: string): string;
function StrIsAbbreviated (const S: string): Boolean;
function NotFoundCharsInParent (const Test, Parent: string): string;
function ContainsAnyChar (const Substr, S: string): Boolean;
function OnlySubstrChar (const Substr, S: string): Boolean;
function StrFoundInMask (Test: string;  const Mask: string): Boolean;
procedure DivideStr
 (const SourceStr, Delimiter: string;
  out LeftStr, RightStr: string;  out Found: Boolean);
function StrCharCount (const S: string;  C: Char): Integer;
function StrStrCount (const S, SubS: string): Integer;
function StrCommon (const Str1, Str2: string): string;
function StrCapitalizedSL (const S: string;
  ExcludedWords: TStringList): string;

implementation

uses
  SysUtils, Math,
  StrUtil00;

function StrFromPosToYesOrNoDelimiter
 (const InStr: string;  const DelStr: string;  Span: Boolean;
  var PosB: Integer;  out PosE: Integer): string;
{ Returns string with all characters from InStr PosB(egin) position
  until anyone char equal (Span=False) or different (Span=True)
     to anyone char from DelStr }
var
  Done: Boolean;
begin
  Result := '';
  PosE := 0;
  if  PosB > 0  then  begin
    Done := False;
    while  (PosB <= Length(InStr)) and not Done  do  begin
      PosE := Pos (InStr[PosB], DelStr);
      if  Span xor (PosE = 0)  then  begin
        Result := Result + InStr[PosB];
        Inc (PosB);
      end
      else
        Done := True;
    end;
    if  (Span xor (PosE = 0)) or (PosB > Length(InStr))  then  PosB := 0;
  end;
end {StrFromPosToYesOrNoDelimiter};

function PosChForward (const S: string;  Delim: Char): Integer;
var
  I, N: Integer;
begin
  N := 0;
  for  I := 1 to Length (S) do
    if  S[I] = Delim
    then  Inc (N);
  Result := N;
end {PosChForward};

function PosChBackward (const S: string;  Delim: Char): Integer;
var
  I, J: Integer;
begin
  J := Length (S);
  Result := 0;
  for  I := J downto 1  do
    if  S[I] = Delim  then  begin
      Result := I;
      Exit;
    end;
end {PosChBackward};

function StrUntilDelimiter (var InStr: string;  const Delim: string): string;
{ Returns string with all characters from InStr beginning
  until anyone char equal to anyone char from Delim }
var
  I: Integer;
begin
  I := Pos (Delim, InStr);
  if  I = 0  then  begin
    Result := InStr;
    InStr := '';
  end
  else  begin
    Result := Copy (InStr, 1, I-1);
    Delete (InStr, 1, Length(Delim)+I-1);
  end;
end {StrUntilDelimiter};

function StrBkUntilDelimiter (var InStr: string;  Delim: Char): string;
{ Returns string with all characters from InStr ending
  until anyone char equal to anyone char from Delim }
var
  J, Pos: Integer;
begin
  J := Length (InStr);
  Pos := PosChBackward (InStr, Delim);
  if  Pos = 0  then  begin
    Result := InStr;
    InStr := '';
  end
  else  begin
    Result := Copy (InStr, Pos+1, J-Pos);
    Delete (InStr, Pos, J-Pos+1);
  end;
end {StrBkUntilDelimiter};

function StrBkUntilSDelimiter (var InStr: string;  const Delim: string): string;
var
  I: Integer;
  TransStr, ResStr: string;
  Count: Integer;
begin
  I := Pos (Delim, InStr);
  if  I = 0  then  begin
    Result := InStr;
    InStr := '';
  end
  else  begin
    TransStr := InStr;
    Count := 0;
    repeat
      ResStr := StrUntilDelimiter (TransStr, Delim);
      Inc (Count, Length (ResStr)+Length(Delim));
    until  Pos (Delim, TransStr) = 0;
    Result := TransStr;
    InStr := Copy (InStr, 1, Count-Length(Delim));
  end;
end {StrBkUntilSDelimiter};

function StrUntilDelimiters (var InStr: string;  const DelStr: string): string;
{ Returns string with all characters from InStr beginning
  until anyone char equal to anyone char from Delim }
var
  Cp, Dp: Integer;
begin
  Cp := 1;
  Result := StrFromPosToYesOrNoDelimiter (InStr, DelStr, False, Cp, Dp);
  if  Cp = 0  then
    InStr := ''
  else
    Delete (InStr, 1, Cp-1);
end {StrUntilDelimiters};

function StrUntilDelimitersAndDelimiter (var InStr: string;
  const DelStr: string;  out Delimiter: Char): string;
{ Returns string with all characters from InStr beginning
  until anyone char equal to anyone char from DelStr
  Sample:
  InStr := 'hello world';
  W := StrUntilDelimitersAndDelimiter (S, ' ', Delim);
  then  W = 'hello', InStr = 'world' and Delimiter = ' '
  }
var
  Cp, Dp: Integer;
begin
  Cp := 1;
  Delimiter := #0;
  Result := StrFromPosToYesOrNoDelimiter (InStr, DelStr, False, Cp, Dp);
  if  Cp = 0  then
    InStr := ''
  else  begin
    Delimiter := InStr[Cp];
    Delete (InStr, 1, Cp);
  end;
end {StrUntilDelimitersAndDelimiter};

function StrUntilNoDelimiters (var InStr: string;
  const DelStr: string): string;
{ Returns string with all characters from InStr beginning
  until anyone char different to anyone char from DelStr }
var
  Cp, Dp: Integer;
begin
  Cp := 1;
  Result := StrFromPosToYesOrNoDelimiter (InStr, DelStr, True, Cp, Dp);
  if  Cp = 0  then
    InStr := ''
  else
    Delete (InStr, 1, Cp-1);
end {StrUntilNoDelimiters};

function StrDelimitersXAtBegin (const InStr, DelStr: string): Boolean;
var
  J: Word;
begin
  Result := False;
  if  InStr = ''
  then  Exit;
  for  J := 1 to Length (DelStr)  do
    if  DelStr[J] = InStr[1]  then  begin
      Result := True;
      Exit;
    end;
end {StrDelimitersXAtBegin};

function StrUntilDelimitersX (var InStr: string;  const DelStr: string): string;
{ Returns string with all characters from InStr beginning
 (or past anyone char equal to anyone char from Delim)
  until past anyone char equal to anyone char from Delim }
begin
  Result := '';
  if  InStr = ''
  then  Exit;
  if  StrDelimitersXAtBegin (InStr, DelStr)
  then  StrUntilNoDelimiters (InStr, DelStr);
  Result := StrUntilDelimiters (InStr, DelStr);
  StrUntilNoDelimiters (InStr, DelStr);
end {StrUntilDelimitersX};

function StrUntilDelimitersXAndDelimiter (var InStr: string;
  const DelStr: string;  out Delimiter: Char): string;
{ Returns string with all characters from InStr beginning
 (or past anyone char equal to anyone char from Delim)
  until past anyone char equal to anyone char from Delim }
begin
  Result := '';
  Delimiter := #0;
  if  StrDelimitersXAtBegin (InStr, DelStr)
  then  StrUntilNoDelimiters (InStr, DelStr);
  Result := StrUntilDelimitersAndDelimiter (InStr, DelStr, Delimiter);
  StrUntilNoDelimiters (InStr, DelStr);
end {StrUntilDelimitersXAndDelimiter};

function StrReplaceSimple (InStr: string;
  const SearchStr, ReplaceStr: string): string;
var
  I: Integer;
begin
  if  SearchStr = ''  then  begin
    Result := InStr;
    Exit;
  end
  else
    Result := '';
  while  InStr <> ''  do  begin
    I := Pos (SearchStr, InStr);
    if  I > 0  then  begin
      Result := Concat (Result, Copy (InStr, 1, I-1), ReplaceStr);
      InStr := Copy (InStr, I+Length(SearchStr), MaxInt);
    end
    else  begin
      Result := Concat (Result, InStr);
      InStr := '';
    end;
  end;
end {StrReplaceSimple};

function StrReplacedSimple (var S: string;
  const SearchStr, ReplaceStr: string): Boolean;
var
  SRes: string;
begin
  SRes := StrReplaceSimple (S, SearchStr, ReplaceStr);
  Result := SRes <> S;
  S := SRes;
end {StrReplacedSimple};

function StrReplaceText (InStr: string;  const SearchStr, ReplaceStr: string;
  var Changed: Boolean;  CaseSensitive: Boolean): string;
var
  I: Integer;
begin
  Changed := False;
  if  SearchStr = ''  then  begin
    Result := InStr;
    Exit;
  end
  else  begin
    Result := '';
    while  InStr <> ''  do  begin
      if CaseSensitive
      then  I := Pos (SearchStr, InStr)
      else  I := Pos (AnsiUpperCase(SearchStr), AnsiUpperCase(InStr));
      if  I > 0  then  begin
        Result := Concat (Result, Copy (InStr, 1, I-1), ReplaceStr);
        Changed := True;
        InStr := Copy (InStr, I+Length(SearchStr), MaxInt);
      end
      else  begin
        Result := Concat (Result, InStr);
        InStr := '';
      end;
    end;
  end;
end {StrReplaceText};

function StrReplaceBetweenDelim (InStr: string;
  const SearchStr, ReplaceStr, DelStr: string): string;
var
  C, ResultStr: string;
begin
  if  SearchStr = ''  then  begin
    Result := InStr;
    Exit;
  end
  else
    ResultStr := '';
  while  InStr <> ''  do  begin
    C := StrUntilNoDelimiters (InStr, DelStr);
    ResultStr := Concat (ResultStr, C);
    C := StrUntilDelimiters (InStr, DelStr);
    if  C = SearchStr  then  C := ReplaceStr;
    ResultStr := Concat (ResultStr, C);
  end;
  Result := ResultStr;
end {StrReplaceBetweenDelim};

function StrIsAbbreviated (const S: string): Boolean;
var
  S1, S2: string;
begin
  S1 := S;
  Result := False;
  while  S1 <> ''  do  begin
    S2 := StrUntilDelimiter (S1, ' ');
    if  (Length (S2) = 1) or
        ((Length (S2) = 2) and (S2[2]='.'))  then  begin
      Result := True;
      Exit;
    end;
  end;
end {StrIsAbbreviated};

function NotFoundCharsInParent (const Test, Parent: string): string;
var
  I: Integer;
begin
  Result := '';
  for  I := 1 to Length(Test) do
    if  Pos (Test[I], Parent) = 0
    then  Result := Concat (Result, Test[I]);
end {NotFoundCharsInParent};

function ContainsAnyChar (const Substr, S: string): Boolean;
var
  I: Integer;
begin
  Result := False;
  if  (Substr = '') or (S = '')
  then  Exit;
  for  I := 1 to Length(Substr) do
    if  Pos (Substr[I], S) <> 0  then begin
      Result := True;
      Exit;
    end;
end {ContainsAnyChar};

function OnlySubstrChar (const Substr, S: string): Boolean;
var
  I: Integer;
begin
  Result := True;
  if  (Substr = '') or (S = '')
  then  Exit;
  for  I := 1 to Length(S) do
    if  Pos (S[I], Substr) = 0  then begin
      Result := False;
      Exit;
    end;
end {OnlySubstrChar};

function StrFoundInMask (Test: string;  const Mask: string): Boolean;
{Is insensitive case and Mask yet must un AnsiUpperCase}
function OkJust (C: Char): Boolean;
var
  I: Byte;
  Ok: Boolean;
begin
  Ok := True;
  for  I := 1 to Length (Mask)  do
    if  Mask[I] = C
    then  Break
    else if  Mask[I] <> Test[I]  then  begin
      Ok := False;
      Break;
    end;
  Result := Ok;
end {OkJust};
begin
  Test := AnsiUpperCase (Test);
  if  (Test = Mask) or (Mask = '*')
  then  Result := True
  else if  ((Pos ('*', Mask) = 0) and (Pos ('?', Mask) = 0)) or
          ((Pos ('*', Mask) <> 0) and (Pos ('?', Mask) <> 0)) {invalid}
  then  Result := False
  else if  Pos ('*', Mask) <> 0
  then  Result := OkJust ('*')
  else if  (Pos ('?', Mask) <> 0) and (Length (Test) = Length (Mask))
  then  Result := OkJust ('?')
  else  Result := False;
end {StrFoundInMask};

procedure DivideStr (const SourceStr, Delimiter: string;
  out LeftStr, RightStr: string;  out Found: Boolean);
{ Divide SourceStr }
var
  LongSource, Position: Integer;
  LongDelim: Byte;
begin
  LongSource := Length (SourceStr);
  LongDelim  := Length (Delimiter);
  LeftStr := '';
  RightStr := '';
  Found := False;
  if  (LongSource = 0) or (LongDelim = 0)  then
    Exit;
  Position := Pos (Delimiter, SourceStr);
  if  Position > 0  then  begin
    LeftStr := Copy (SourceStr, 1, Position - 1);
    RightStr := Copy (SourceStr, Position + LongDelim,
                    LongSource - Position - LongDelim + 1);
    Found := True;
  end;
end {DivideStr};

function StrSearch (const Substr, S: string;  const Index: Integer): Integer;
var
  Res: Integer;
begin
  Res := Pos (Substr, Copy (S, Index, MaxInt));
  if  Res <> 0
  then  Res := Res + (Index-1);
  Result := Res;
end {StrSearch};

function StrCharCount (const S: string;  C: Char): Integer;
{from JclStrings.pas}
var
  I: Integer;
begin
  Result := 0;
  for I := 1 to Length(S) do
    if S[I] = C then
      Inc(Result);
end {StrCharCount};

function StrStrCount (const S, SubS: string): Integer;
{from JclStrings.pas}
var
  I: Integer;
begin
  Result := 0;
  if (Length(SubS) > Length(S)) or (Length(SubS) = 0) or (Length(S) = 0) then
    Exit;
  if Length(SubS) = 1 then
  begin
    Result := StrCharCount(S, SubS[1]);
    Exit;
  end;
  I := StrSearch(SubS, S, 1);
  if I > 0 then
    Inc(Result);
  while (I > 0) and (Length(S) > I+Length(SubS)) do
  begin
    I := StrSearch(SubS, S, I+1);
    if I > 0 then
      Inc(Result);
  end
end {StrStrCount};

function StrCommon (const Str1, Str2: string): string;
var
  MinL, I: Integer;
begin
  Result := '';
  if  (Str1 = '') or (Str2 = '')
  then  Exit;
  MinL := Min (Length (Str1), Length (Str2));
  for  I := 1 to MinL  do  begin
    if  Str1[I] = Str2[I]
    then  Result := Concat (Result, Str1[I])
    else  Exit;
  end;
end {StrCommon};

function StrCapitalizedSL (const S: string;
  ExcludedWords: TStringList): string;
var
  S2, W: string;
begin
  Result := S;
  if  S <> ''  then  begin
    S2 := AnsiLowerCase (S);
    Result := '';
    while  S2 <> ''  do  begin
      W := StrUntilDelimitersX (S2, ' ');
      if  ExcludedWords.IndexOf (W) = -1
      then  W[1] := AnsiUpCase (W[1]);
      Result := ConcatSpaced (Result, W);
    end;
  end;
end {StrCapitalizedSL};

end.
