{---------------------------------------------------------------------

 Functions that uses numbers,  and usually returns strings 

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/01/2006 version

----------------------------------------------------------------------}

unit StrNumU;

interface

uses
  GrTypFun;

function IntegerFormat (WithThousandSeparator: Boolean): string;
function FloatFormat (WithThousandSeparator: Boolean;
  DecimalsCount: Byte): string;

{P: Places; DP: DecimalPlaces}
function IntTo2Str (Value: LongInt;  P: Byte): string;
function IntToCStr (Value: LongInt;  P: Byte;  C: Char): string;
function IntToZStr (Value: LongInt;  P: Byte): string;
{with ThousandPlaces:}
function IntToTSStr (Value: Int64): string; {1234 -> 1,234}
function IntToTPRXStr (Value: Int64;  Places: Byte): string;  {1234 -> 1,230, if Places = 1}
function IntToTPR1Str (Value: Int64): string; { = IntToTPRXStr(Value,1) }
function IntToTPR2Str (Value: Int64): string; { = IntToTPRXStr(Value,2) }

function Real2ToStr (Value: Real;  DP: Byte): string;  {without ThousandPlaces}
function Real3ToStr (Value: Real;  P, DP: Byte): string;
{with ThousandPlaces:}
function RealToTSStr (Value: Real;  DP: Byte): string; {1234.78 -> 1,234.8, if DP = 1}
function RealToTPRXStr (Value: Real;  Places: Byte): string;  {1234.78 -> 1,230, if Places = 1}
function RealToTPR1Str (Value: Real): string; { = RealToTPRXStr(Value,1) }
function RealToTPR2Str (Value: Real): string; { = RealToTPRXStr(Value,2) }

function CharIsCipher (Ch: Char): Boolean;
function ByteToStr1 (B: Byte): Str1;
function Str1ToByte (S: Str1): Byte;
function StrIsInt (const S: string): Boolean;
function StrIsFloat (const S: string): Boolean;

implementation

uses
  SysUtils;

function IntegerFormat (WithThousandSeparator: Boolean): string;
begin
  if  WithThousandSeparator
  then  Result := Concat ('#,##0')
  else  Result := '0';
end {IntegerFormat};

function FloatFormat (WithThousandSeparator: Boolean;
  DecimalsCount: Byte): string;
var
  S: string;
  I: Byte;
begin
  S := IntegerFormat (WithThousandSeparator);
  if  DecimalsCount <> 0  then  begin
    S := Concat (S, '.');
    for  I := 1 to DecimalsCount  do
      S := Concat (S, '0');
  end;
  Result := S;
end {FloatFormat};

{ Obtain a string from a longint number }

function IntTo2Str (Value: LongInt;  P: Byte): string;
begin
  Str (Value:P, Result);
end;

function IntToCStr (Value: LongInt;  P: Byte;  C: Char): string;
{ if Value=123, P=4, C='' returns '123' }
var
  I: Byte;
  Temp: string;
begin
  Temp := IntTo2Str (Value, P);
  I := 1;
  while  I <= P  do  begin
    case  Temp [I]  of
    ' ': Temp [I] := C;
    '-': if  C = '0'  then  begin
            Temp [1] := '-';
            if  I <> 1  then  Temp [I] := '0';
          end;
    else  begin
      Result := Temp;
      Exit;
    end;
    end;
    Inc (I);
  end;
end {IntToCStr};

function IntToZStr (Value: LongInt;  P: Byte): string;
begin
  Result := IntToCStr (Value, P, '0');
end;

function IntToTSStr (Value: Int64): string;
var
  R: Real;
begin
  R := Value;
  Result := Format ('%1.0n', [R]);
end {IntToTSStr};

function IntToTPRXStr (Value: Int64;  Places: Byte): string;
var
  I, J: Integer;
begin
  J := 1;
  for  I := 1 to Places  do
    J := J * 10;
  Result := FormatFloat (IntegerFormat (True), Round(Value/J)*J);
end {IntToTPRXStr};

function IntToTPR1Str (Value: Int64): string;
begin
  Result := IntToTPRXStr (Value, 1);
end;

function IntToTPR2Str (Value: Int64): string;
begin
  Result := IntToTPRXStr (Value, 2);
end;


{ Obtain a string from a real number }

function Real2ToStr (Value: Real;  DP: Byte): string;
begin
  Result := FormatFloat (FloatFormat (False, DP), Value);
end ;

function Real3ToStr (Value: Real;  P, DP: Byte): string;
begin
  Str (Value:P:DP, Result);
end;

function RealToTSStr (Value: Real;  DP: Byte): string;
begin
  Result := FormatFloat (FloatFormat (True, DP), Value);
end;

function RealToTPRXStr (Value: Real;  Places: Byte): string;
var
  I, J: Integer;
begin
  J := 1;
  for  I := 1 to Places  do
    J := J * 10;
  Result := FormatFloat (IntegerFormat (True), Round(Value/J)*J);
end {RealToTPRXStr};

function RealToTPR1Str (Value: Real): string;
begin
  Result := RealToTPRXStr (Value, 1);
end;

function RealToTPR2Str (Value: Real): string;
begin
  Result := RealToTPRXStr (Value, 2);
end;


function ByteToStr1 (B: Byte): Str1;
var
  C: Str1;
begin
  C := ' ';
  C[1] := Chr(B + 64);
  Result := C;
end {ByteToStr1};

function Str1ToByte (S: Str1): Byte;
begin
  Result := Ord(S[1]) - 64;
end;

function CharIsCipher (Ch: Char): Boolean;
begin
  Result := Pos (Ch, '1234567890') <> 0;
end;

function StrIsInt (const S: string): Boolean;
var
  Res: Int64;
begin
  Result := False;
  if  S = ''
  then  Exit;
  try
    Res := StrToInt (S);
    Result := True;
  except
    Result := False;
  end;
end {StrIsInt};

function StrIsFloat (const S: string): Boolean;
var
  Res: Extended;
begin
  Result := False;
  if  S = ''
  then  Exit;
  try
    Res := StrToFloat (S);
    Result := True;
  except
    Result := False;
  end;
end {StrIsFloat};

end.
