{---------------------------------------------------------------------

 Several General Types and Functions

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/07/2006 version

----------------------------------------------------------------------}

unit StrUtil00;

interface

uses
  GrTypFun;

function CharToStr1 (Ch: Char): Str1;
function AnsiUpCase (Ch: Char): Char;
function AnsiLoCase (Ch: Char): Char;

{Rx\JvJCLUtils.MakeStr: make string with Len of Char}
function MakeStrSpaces (Len: Word): string;
{Rx\JvJCLUtils.DelBSpace: Delete begin spaces}
function DelBChar (const S: string;  Ch: Char): string;
function DelEChar (const S: string;  Ch: Char): string;
{Rx\JvJCLUtils.DelESpace: Delete end spaces}
function DelAmpersand (const S: string): string;
function StrPad (AtBegin: Boolean;  Ch: Char;
  const S: string;  Len: Word): string;
function StrPadRight(const S: string;  Len: Word;  Ch: Char = ' '): string;
function StrOEMToChar (const InStr: string): string;
function StrCharToOEM (const InStr: string): string;
function StrReplaceChar (const S: string;
  const Source, Replace: Char): string; {from JCL strings}
function StrReplaceChars (const S: string;  const Source, Replace: string): string;
function StrReplaceSpaceToUnderscoreChar (const S: string): string;
function StrReplaceUnderscoreToSpaceChar (const S: string): string;
function StrCapitalized (const S: string;
  AllWords, ReplaceUnderscore: Boolean): string;
function StrQuoted (S: string): string;
function ConcatWithStr (const SBegin, SEnd, Sep: string): string;
function ConcatWithStrX (const SBegin, SEnd, Sep: string): string;
function ConcatSpaced (const SBegin, SEnd: string): string;
function RepeatedChrNumInStr (const S: string;  Chr: Char): Word;
function RemoveChars (const S, InvalidChars: string): string;
function RemoveDuplicatedChars (const S: string): string;

implementation

uses
  SysUtils, Windows,
  JvJCLUtils;

function CharToStr1 (Ch: Char): Str1;
begin
  SetLength (Result, 1);
  Result[1] := Ch;
end {CharToStr1};

function AnsiUpCase (Ch: Char): Char;
{ converts a character to uppercase }
var
  S: Str1;
begin
  S := CharToStr1 (Ch);
  Result := AnsiUpperCase(S)[1];
end {AnsiUpCase};

function AnsiLoCase (Ch: Char): Char;
{ converts a character to lowercase }
var
  S: Str1;
begin
  S := CharToStr1 (Ch);
  Result := AnsiLowerCase(S)[1];
end {AnsiLoCase};

function MakeStrSpaces (Len: Word): string;
begin
  Result := MakeStr (' ', Len);
end {MakeStrSpaces};

function DelBChar (const S: string;  Ch: Char): string;
{ Deletes begin char Ch from string }
var
  I, L: Word;
begin
  L := Length(S);
  I := 1;
  while (I <= L) and (S[I] = Ch)
  do  Inc(I);
  Result := Copy(S, I, MaxInt);
end {DelBChar};

function DelEChar (const S: string;  Ch: Char): string;
{ Deletes end char Ch from string }
var
  I: Word;
begin
  I := Length (S);
  while  (I >= 1) and (S[I] = Ch)
  do  Dec (I);
  Result := Copy (S, 1, I);
end {DelEChar};

function DelAmpersand (const S: string): string;
begin
  Result := DelChars (S, '&');
end {DelAmpersand};

function StrPad (AtBegin: Boolean;  Ch: Char;
  const S: string;  Len: Word): string;
begin
  {if  Len >= SizeOf (S)  then
    Len := SizeOf (S) - 1;}
  if  Length (S) > Len
  then  Result := Copy (S, 1, Len)
  else if  AtBegin
  then  Result := MakeStr (Ch, Len - Length (S)) + S
  else  Result := S + MakeStr (Ch, Len - Length (S));
end {StrPad};

function StrPadRight(const S: string;  Len: Word;  Ch: Char = ' '): string;
{ Trims Str to Len:
  Ex: S='hello' and Len=3 -> S='hel'
  Ex: S='hello' ans Len=6 -> S='hello '}
begin
  Result := StrPad (False, Ch, S, Len);
end {StrPadRight};

function StrOEMToChar (const InStr: string): string;
var
  Source, Target: array[0..1024] of Char;
begin
  StrPCopy (Source, InStr);
  OEMToChar (Source, Target);
  Result := StrPas (Target);
end {StrOEMToChar};

function StrCharToOEM (const InStr: string): string;
var
  Source, Target: array[0..1024] of Char;
begin
  StrPCopy (Source, InStr);
  CharToOEM (Source, Target);
  Result := StrPas (Target);
end {StrCharToOEM};

function StrReplaceChar (const S: string;
  const Source, Replace: Char): string;
var
  I: Integer;
begin
  Result := S;
  for  I := 1 to Length(S)  do
    if  Result[I] = Source
    then  Result[I] := Replace;
end {StrReplaceChar};

function StrReplaceChars (const S: string;  const Source, Replace: string): string;
var
  I: Integer;
  P: Integer;
begin
  Result := S;
  for  I := 1 to Length(S)  do  begin
    P := Pos (Result[I], Source);
    if  P <> 0
    then  Result[I] := Replace[P];
  end;
end {StrReplaceChars};

function StrReplaceSpaceToUnderscoreChar (const S: string): string;
begin
  Result := StrReplaceChar (S, ' ', '_');
end {StrReplaceSpaceToUnderscoreChar};

function StrReplaceUnderscoreToSpaceChar (const S: string): string;
begin
  Result := StrReplaceChar (S, '_', ' ');
end {StrReplaceUnderscoreToSpaceChar};

function StrCapitalized (const S: string;
  AllWords, ReplaceUnderscore: Boolean): string;
var
  I: Word;
begin
  Result := S;
  if  S <> ''  then  begin
    Result := AnsiLowerCase (S);
    Result[1] := AnsiUpCase (Result[1]);
    if AllWords and (Length(Result)>1) then  begin
      for  I := 2 to Length(Result)  do
        if  Result[I-1] in ['_',' ']
        then  Result[I] := AnsiUpCase (Result[I]);
    end;
    if ReplaceUnderscore
    then  Result := StrReplaceUnderscoreToSpaceChar (Result);
  end;
end {StrCapitalized};

function StrQuoted (S: string): string;
// Replaces " for ' at string begin and end
const
  Quote = '''';
procedure ChangeH (W: Integer);
begin
  if  S[W] = '"'
  then  S[W] := Quote;
end {ChangeH};
begin
  if  S <> ''  then  begin
    ChangeH (1);
    ChangeH (Length(S));
    if  S[1] <> Quote
    then  S := Concat (Quote, S, Quote);
  end;
  Result := S;
end {StrQuoted};

function ConcatWithStr (const SBegin, SEnd, Sep: string): string;
begin
  if  SBegin <> ''
  then  Result := Concat (SBegin, Sep, SEnd)
  else  Result := SEnd;
end {ConcatWithStr};

function ConcatWithStrX (const SBegin, SEnd, Sep: string): string;
begin
  if  SBegin <> '' then  begin
    if  SEnd <> ''
    then  Result := Concat (SBegin, Sep, SEnd)
    else  Result := SBegin;
  end
  else
    Result := SEnd;
end {ConcatWithStrX};

function ConcatSpaced (const SBegin, SEnd: string): string;
begin
  Result := ConcatWithStr (SBegin, SEnd, ' ');
end {ConcatSpaced};

function RepeatedChrNumInStr (const S: string;  Chr: Char): Word;
var
  I, Len: Word;
begin
  Len := Length (S);
  Result := 0;
  for  I := 1 to Len  do
    if  S[I] = Chr  then  Inc (Result);
end {RepeatedChrNumInStr};

function RemoveChars (const S, InvalidChars: string): string;
var
  I: Word;
begin
  Result := '';
  for  I := 1 to Length(S)  do
    if  Pos (S[I],InvalidChars) = 0
    then  Result := Concat (Result, S[I]);
end {RemoveChars};

function RemoveDuplicatedChars (const S: string): string;
var
  I: Word;
begin
  Result := '';
  for  I := 1 to Length (S)  do  begin
    if  Pos (S[I], Result) = 0
    then  Result := Concat (Result, S[I]);
  end;
end {RemoveDuplicatedChars};

end.
