{---------------------------------------------------------------------

 modified from Ian Marteens's ELog.pas unit, 
 Chapter 11 in "La cara oculta de Delphi4" book/CD 
 
 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/01/2006 version

 03/11/2006
 bug: now supports WideString Filename
 new: procedure WriteErrorToFile (const Mess: WideString);
     There are different modalities to open the Log file

----------------------------------------------------------------------}

unit TntELog;

interface

uses
  Classes, SysUtils, TntSysUtils, Forms;

resourcestring
  RS_ELog_NoLogFile = '%s'#13#10'not found'#13#10'(any critical error should not have been produced)';
  RS_ELog_NoEditor = 'An editor that allows to open ''%s'' directly doesn''t exist';

const
  DefaultEditor = 'Notepad.exe';
  ExtLog = 'log';
  DotExtLog = '.'+ ExtLog;

type
  TExceptionLogW = class
  private
    FFileName: WideString;
    FCriticalError: Boolean;
    FDefEditor: WideString;
    FUseDefEditor: Boolean;
    procedure SetDefEditor(const Value: WideString);
  protected
    SavedHandler: TExceptionEvent;
  public
    Enabled: Boolean;
    {sometimes is'nt necessary to write to log file:
     perhaps an user defined runtime SQL}
    constructor Create (const AFileName: WideString);
    destructor Destroy; override;
    procedure ExceptionHandler (Sender: TObject;  E: Exception);
    procedure ShowLogFile;
    procedure WriteErrorToFile (const Mess: WideString);
    property CriticalError: Boolean read FCriticalError write FCriticalError; {Program break down}
    property DefEditor: WideString read FDefEditor write SetDefEditor;
    property FileName: WideString read FFileName; {used for runtime read logfile}
    property UseDefEditor: Boolean read FUseDefEditor write FUseDefEditor;
  end;

var
  Log: TExceptionLogW = nil;

procedure StartLog;
procedure StopLog;

implementation

uses
  ShellApi, Windows,
  TntClasses, TntSystem, TntMessUsual;

{--------------------------------------}
{-PRIVATE----------}

procedure TExceptionLogW.SetDefEditor (const Value: WideString);
begin
  FDefEditor := Value;
  FUseDefEditor := FDefEditor <> '';
end;

{-PROTECTED--------}

procedure TExceptionLogW.ExceptionHandler (Sender: TObject;  E: Exception);
var
  S: WideString;
begin
  if  (Sender is TComponent) and Enabled
  then  S := Concat (TComponent(Sender).Name, ':')
  else  S := '';
  WideMessageBoxError (Concat (S, sLineBreak, E.ClassName, sLineBreak, E.Message));
  if Enabled
  then  WriteErrorToFile (Concat (S, ' ', E.ClassName, ': ', E.Message));
end {ExceptionHandler};

{-PUBLIC-----------}

constructor TExceptionLogW.Create (const AFileName: WideString);
begin
  inherited Create;
  FFileName := AFileName;
  DefEditor := DefaultEditor;
  SavedHandler := Application.OnException;
  Application.OnException := ExceptionHandler;
  Enabled := True;
end {Create};

destructor TExceptionLogW.Destroy;
begin
  Application.OnException := SavedHandler;
  inherited;
end;

procedure TExceptionLogW.ShowLogFile;
begin
  if  WideFileExists (Log.FileName)  then  begin
    if FUseDefEditor then  begin
      UseDefEditor := ShellExecuteW (Application.Handle, nil, PWideChar(FDefEditor),
        PWideChar(FileName), '', SW_SHOWNORMAL) > 32;
    end;
    if  not FUseDefEditor  then
      if  ShellExecuteW (Application.Handle, 'open', PWideChar(FileName),
        '', '', SW_SHOWNORMAL) <= 32
      then  WideMessageBoxError (Format (RS_ELog_NoEditor, [FileName]));
  end
  else
    WideMessageBoxInfo (Format (RS_ELog_NoLogFile, [FileName]));
end {ShowLogFile};

procedure TExceptionLogW.WriteErrorToFile (const Mess: WideString);
var
  Ok: Boolean;
  SL: TTntStringList;
procedure CloseAll;
begin
  SL.Free;
  if FCriticalError
  then  Application.Terminate;
end {CloseAll};
begin
  try
    SL := TTntStringList.Create;
    try
      if  WideFileExists (FFileName)
      then  SL.LoadFromFile (FFileName);
      Ok := True;
    finally
    end;
    if Ok then  begin
      SL.Add (Concat (
        FormatDateTime (ShortDateFormat+' '+LongTimeFormat, Now),
        ' ', Mess));
      try
        SL.SaveToFile (FFileName);
      finally
        CloseAll;
      end;
    end
    else begin
      CloseAll;
    end;
  except
  end;
end {WriteErrorToFile};

{--------------------------------------}

procedure StartLog;
begin
  if  not Assigned (Log)  then
    Log := TExceptionLogW.Create (WideChangeFileExt(WideParamStr(0), DotExtLog));
end;

procedure StopLog;
begin
  if  Assigned (Log)  then  begin
    Log.Free;
    Log := nil;
  end;
end {StopLog};

{--------------------------------------}

initialization
  StartLog;
finalization
  StopLog;
end.
