{---------------------------------------------------------------------

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es

 Controls with validation (that they contain any value) and
   some modifications of chars when typing (for edit inputs)
 Several functions for indirect validation of some controls

 05/19/2006 version
   New:
     InvalidFileNameChars is public
     use of Backspace char
 10/30/2006 version
   Add: property CharsWithoutModifiers for text inputs
   Add: Several functions for indirect validation of some controls
   Add: Name edition components (TTntEditName and TTntDBEditName),
     used for name and surname inputs.
   TntLabel (using the Caption) can be used for a message when validation is called

----------------------------------------------------------------------}

unit TntEdiTAmp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  StdCtrls, TntStdCtrls, TntStdCtrls2, DBCtrls, TntDBCtrls, TntDBCtrls2,    
  ExtCtrls, TntExtCtrls, TntJvToolEdit, TntJvDBLookup, TntVer;

  {
  FCheckOnExit:
    if is true, checks value when it try exits control
  Required (Value or Field):
    if is true, value is required
  cmProperCase:
    if is true, changes first letter of Text to upcase on type,
      and the 2nd letter prompt for locase
  Abbreviation:
    if is false, it not allow one letter words or
      words with one letter and a point
  WithBeginOrEndSpaces:
    if is false, space at beginning or end of Text is not allowed,
  CharsWithoutModifiers:
    if is true, if you type an accent letter changes it to unaccent letter
}

resourcestring
  RS_EdiTAmp_EditWithAbbrev = 'On ''%s'' abbreviations are not admitted';
  RS_EdiTAmp_EmptyDefinedEdit = '''%s'' must have something written';
  RS_EdiTAmp_EmptyEdit = '%s%s needs something written';
  RS_EdiTAmp_InvalidEdit = '%s%s needs a valid value';
  RS_EdiTAmp_EmptyEditL = 'Input ''%s'' needs something written';
  RS_EdiTAmp_InvalidEditL = 'Input ''%s'' needs a valid value';
  RS_EdiTAmp_RewriteOrCancelDBEdit = ', rewrite the field or cancel the record edition';
  RS_EdiTAmp_RGNoItemSelected = 'Group ''%s'' must have a chosen element';
  RS_EdiTAmp_SelectUpcaseToLocase = 'You have just pressed a key that would have to be a lower-case, to convert it? ';
  RS_EdiTAmp_WhereCursorWillSituate = ' (where cursor will situate)';
  RS_EdiTAmp_Input = 'Input';
  RS_EdiTAmp_Input_ByList = 'Input with list';
  RS_EdiTAmp_Input_Date = 'Date input';
  RS_EdiTAmp_Input_Dir = 'Directory input';
  RS_EdiTAmp_Input_File = 'File input';
  RS_EdiTAmp_Input_Filename = 'Filename input';
  RS_EdiTAmp_Input_Name = 'Name input';
  RS_EdiTAmp_Input_Text = 'Text input';

type
  TTntDBRadioGroupR = class (TTntDBRadioGroup)
  private
    FAbout: TAboutInfo;
  protected
    procedure KeyPress(var Key: Char); override;
  public
    function Valid: Boolean; virtual;
    function ValidNoMess: Boolean; virtual;
  published
    property About: TAboutInfo read FAbout write FAbout stored False;
  end;

  TTntRadioGroupR = class (TTntRadioGroup)
  private
    FAbout: TAboutInfo;
    FRequired: Boolean;
  protected
    procedure KeyPress(var Key: Char); override;
  public
    constructor Create(AOwner : TComponent); override;
    function Valid: Boolean; virtual;
    function ValidNoMess: Boolean; virtual;
  published
    property About: TAboutInfo read FAbout write FAbout stored False;
    property Required: Boolean read FRequired write FRequired default True;
  end;

  TCaseMode = (cmNormal, cmProperCase, cmLowerCase, cmUpperCase);

  TTntDBEditR = class (TTntDBEdit2)
  public
    function Valid: Boolean; virtual;
  end;

  TTntDBEditTxt = class (TTntDBEditR)
  private
    FAbbreviation: Boolean;
    FCase: TEditCharCase;
    FCaseMode: TCaseMode;
    FCharsWithoutModifiers: Boolean;
    procedure CMExit (var message: TCMExit); message CM_EXIT;
    procedure WMPaste (var Message: TMessage); message WM_PASTE;
  protected
    FWithBeginOrEndSpaces: Boolean;
    procedure KeyPressW(var Key: WideChar); override;
  public
    constructor Create(AOwner : TComponent); override;
    function Valid: Boolean; override;
    property CharCase: TEditCharCase read FCase;
  published
    property Abbreviation: Boolean read FAbbreviation write FAbbreviation default False;
    property CaseMode: TCaseMode read FCaseMode write FCaseMode;
    property WithBeginOrEndSpaces: Boolean read FWithBeginOrEndSpaces write FWithBeginOrEndSpaces default False;
    property CharsWithoutModifiers: Boolean read FCharsWithoutModifiers write FCharsWithoutModifiers default False;
  end;

  TTntDBEditName = class (TTntDBEditTxt)
  private
    FExtraChars: WideString;
    function  ExtraCharsStored: Boolean;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure SetExtraChars(const Value: WideString);
  protected
    procedure KeyPressW(var Key: WideChar); override;
  public
    constructor Create(AOwner : TComponent); override;
  published
    property ExtraChars: WideString read FExtraChars write SetExtraChars stored ExtraCharsStored;
    property WithBeginOrEndSpaces: Boolean read FWithBeginOrEndSpaces write FWithBeginOrEndSpaces default False;
  end;


  TTntEditR = class (TTntEdit2)
  private
    FRequired: Boolean;
  protected
    function  InputName: WideString; virtual;
  public
    constructor Create(AOwner : TComponent); override;
    procedure CreateWindowHandle(const Params: TCreateParams); override;
    function  Valid: Boolean; virtual;
    function  ValidL (L: TTntLabel): Boolean; virtual;
  published
    property Required: Boolean read FRequired write FRequired default True;
  end;

  TTntEditNoBegSpace = class(TTntEditR)
  private
    procedure CMExit (var message: TCMExit); message CM_EXIT;
  protected
    function  OtherKeyNoValid (Key: Char): Boolean; virtual;
    procedure KeyPress(var Key: Char); override;
  end;

  TTntEditOnlyFileName = class(TTntEditNoBegSpace)
  private
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
  protected {for no compiler Hint message}
    function  InputName: WideString;  override;
    function  OtherKeyNoValid (Key: Char): Boolean; override;
  end;

  TTntEditTxt = class(TTntEditR)
  private
    FAbbreviation: Boolean;
    FCase: TEditCharCase;
    FCaseMode: TCaseMode;
    FCharsWithoutModifiers: Boolean;
    FWithBeginOrEndSpaces: Boolean;
    procedure CMExit (var message: TCMExit); message CM_EXIT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure SetCaseMode(const Value: TCaseMode);
    procedure SetCharsWithoutModifiers(const Value: Boolean);
    procedure SetWithBeginOrEndSpaces(const Value: Boolean);
  protected
    function  InputName: WideString;  override;
    procedure KeyPressW(var Key: WideChar); override;
  public
    constructor Create(AOwner: TComponent); override;
    function  Valid: Boolean; override;
    function  ValidL (L: TTntLabel): Boolean; override;
    property CharCase: TEditCharCase read FCase;
  published
    property Abbreviation: Boolean read FAbbreviation write FAbbreviation default False;
    property CaseMode: TCaseMode read FCaseMode write SetCaseMode;
    property WithBeginOrEndSpaces: Boolean read FWithBeginOrEndSpaces write SetWithBeginOrEndSpaces default True;
    property CharsWithoutModifiers: Boolean read FCharsWithoutModifiers write SetCharsWithoutModifiers default False;
  end;

  TTntEditName = class (TTntEditTxt)
  private
    FExtraChars: WideString;
    function  ExtraCharsStored: Boolean;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure SetExtraChars(const Value: WideString);
  protected
    function  InputName: WideString;  override;
    procedure KeyPressW(var Key: WideChar); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property ExtraChars: WideString read FExtraChars write SetExtraChars stored ExtraCharsStored;
    property WithBeginOrEndSpaces default False;
  end;

  TTntComboBoxR = class (TTntComboBoxA)
  private
    FRequired: Boolean;
  public
    constructor Create(AOwner : TComponent); override;
    procedure CreateWindowHandle(const Params: TCreateParams); override;
    function Valid: Boolean; virtual;
    function ValidL (L: TTntLabel): Boolean; virtual;
  published
    property Required: Boolean read FRequired write FRequired default True;
  end;

  TTntComboBoxRWLI = class (TTntComboBoxR)
  private
    FMaxItemsCount: Integer;
  public
    procedure AddLastInput;
  published
    property MaxItemsCount: Integer read FMaxItemsCount write FMaxItemsCount;
  end;

const
  InvalidFileNameChars = PathDelim + DriveDelim + './|?<>"';

function TntComboBoxValid (CB: TTntComboBox): Boolean;
function TntComboBoxValidL (L: TTntLabel;  CB: TTntComboBox): Boolean;
function TntDateEditValid (DE: TTntJvDateEdit): Boolean;
function TntDateEditValidL (L: TTntLabel;  DE: TTntJvDateEdit): Boolean;
function TntDBComboBoxValid (DBCB: TTntDBComboBox): Boolean;
function TntDBComboBoxValidL (L: TTntLabel;  DBCB: TTntDBComboBox): Boolean;
function TntDBEditValid (DBE: TTntDBEdit): Boolean;
function TntDBEditValidL (L: TTntLabel;  DBE: TTntDBEdit): Boolean;
function TntDBLookupComboValid (DBLCB: TTntJvDBLookupCombo): Boolean;
function TntDBLookupEditValid (DBLE: TTntJvDBLookupEdit): Boolean;
function TntDBLookupEditValidL (L: TTntLabel;  DBLE: TTntJvDBLookupEdit): Boolean;
function TntDBRadioGroupValid (DBRGV: TTntDBRadioGroup): Boolean;
function TntDirEditValid (DE: TTntJvDirectoryEdit): Boolean;
function TntDirEditValidL (L: TTntLabel;  DE: TTntJvDirectoryEdit): Boolean;
function TntEditValid (E: TTntCustomEdit): Boolean;
function TntEditValidL (L: TTntLabel;  E: TTntCustomEdit): Boolean;
function TntFileEditValid (FE: TTntJvFilenameEdit): Boolean;
function TntFileEditValidL (L: TTntLabel;  FE: TTntJvFilenameEdit): Boolean;
function TntRadioGroupValid (RGV: TTntRadioGroup): Boolean;

implementation

uses
  Db,
  TntDialogs, TntClipbrd,
  TntJvJCLUtils, JvJCLUtils,
  JvConsts,
  GrTypFun, TntStrUtil0, TntStrAnal, TntMessUsual;

const
  DefaultExtraNameChars = ''' -';

{------------------}

function CaptFromLabel (L: TTntLabel): WideString;
begin
  Result := DelAmpersandW (L.Caption);
end;

procedure EditMessErrorAndTryFocus (const Mess: WideString;  WC: TWinControl);
begin
  try
    WC.SetFocus;
    WideMessageBoxError (WideFormat (RS_EdiTAmp_EmptyEdit, [Mess, RS_EdiTAmp_WhereCursorWillSituate]));
  except
    WideMessageBoxError (WideFormat (RS_EdiTAmp_EmptyEdit, [Mess, '']));
  end;
end {EditMessErrorAndTryFocus};

procedure EditMessErrorAndTryFocusL (L: TTntLabel;  WC: TWinControl);
procedure ShowRequiredMess;
begin
  WideMessageBoxError (WideFormat (RS_EdiTAmp_EmptyEditL, [CaptFromLabel(L)]));
end;
begin
  try
    WC.SetFocus;
    ShowRequiredMess;
  except
    ShowRequiredMess;
  end;
end {EditMessErrorAndTryFocusL};

procedure DBEditMessErrorAndTryFocus0 (const Mess, S: string;  WC: TWinControl);
begin
  try
    WC.SetFocus;
    WideMessageBoxError (WideFormat (RS_EdiTAmp_EmptyEdit, [Mess, Concat (S, ' ', RS_EdiTAmp_WhereCursorWillSituate)]));
  except
    WideMessageBoxError (WideFormat (RS_EdiTAmp_EmptyEdit, [Mess, S]));
  end;
end {DBEditMessErrorAndTryFocus0};

procedure DBEditMessErrorAndTryFocus (const Mess: WideString;  F: TField;  WC: TWinControl);
var
  S: WideString;
begin
  if  Assigned (F)
  then  S := Concat (' ''', F.FieldName, '''')
  else  S := '';
  DBEditMessErrorAndTryFocus0 (Mess, S, WC);
end {DBEditMessErrorAndTryFocus};

procedure DBEditMessErrorAndTryFocusL (const Mess: WideString;  L: TTntLabel;  WC: TWinControl);
begin
  DBEditMessErrorAndTryFocus0 (Mess, CaptFromLabel(L), WC);
end;

procedure MaskEditMessErrorAndTryFocus (const Mess: WideString;  WC: TWinControl);
begin
  try
    WC.SetFocus;
    WideMessageBoxError (WideFormat (RS_EdiTAmp_InvalidEdit, [Mess, RS_EdiTAmp_WhereCursorWillSituate]));
  except
    WideMessageBoxError (WideFormat (RS_EdiTAmp_InvalidEdit, [Mess, '']));
  end;
end {MaskEditMessErrorAndTryFocus};

procedure MaskEditMessErrorAndTryFocusL (L: TTntLabel;  WC: TWinControl);
procedure ShowRequiredMess;
begin
  WideMessageBoxError (WideFormat (RS_EdiTAmp_InvalidEditL, [CaptFromLabel(L)]));
end;
begin
  try
    WC.SetFocus;
    ShowRequiredMess;
  except
    ShowRequiredMess;
  end;
end {MaskEditMessErrorAndTryFocusL};

function RadioGroupRValidNoMess (FRequired: Boolean;  ItemIndex: Integer): Boolean;
begin
  Result := True;
  if FRequired and (ItemIndex = -1) then  begin
    Result := False;
  end;
end {RadioGroupRValidNoMess};


function TntComboBoxValid (CB: TTntComboBox): Boolean;
begin
  Result := CB.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocus (RS_EdiTAmp_Input_ByList, CB);
end {TntComboBoxValid};

function TntComboBoxValidL (L: TTntLabel;  CB: TTntComboBox): Boolean;
begin
  Result := CB.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocusL (L, CB);
end {TntComboBoxValidL};

function TntDateEditValid (DE: TTntJvDateEdit): Boolean;
begin
  Result := DE.Date <> NullDate;
  if  not Result
  then  MaskEditMessErrorAndTryFocus (RS_EdiTAmp_Input_Date, DE);
end {TntDateEditValid};

function TntDateEditValidL (L: TTntLabel;  DE: TTntJvDateEdit): Boolean;
begin
  Result := DE.Date <> NullDate;
  if  not Result
  then  EditMessErrorAndTryFocusL (L, DE);
end {TntDateEditValidL};

function TntDBComboBoxValid (DBCB: TTntDBComboBox): Boolean;
begin
  Result := DBCB.Text <> '';
  if  not Result
  then  DBEditMessErrorAndTryFocus (RS_EdiTAmp_Input_ByList, DBCB.Field, DBCB);
end {TntDBComboBoxValid};

function TntDBComboBoxValidL (L: TTntLabel;  DBCB: TTntDBComboBox): Boolean;
begin
  Result := DBCB.Text <> '';
  if  not Result
  then  DBEditMessErrorAndTryFocusL (RS_EdiTAmp_Input_ByList, L, DBCB);
end {TntDBComboBoxValidL};

function TntDBEditValid (DBE: TTntDBEdit): Boolean;
begin
  Result := DBE.Text <> '';
  if  not Result
  then  DBEditMessErrorAndTryFocus (RS_EdiTAmp_Input, DBE.Field, DBE);
end {TntDBEditValid};

function TntDBEditValidL (L: TTntLabel;  DBE: TTntDBEdit): Boolean;
begin
  Result := DBE.Text <> '';
  if  not Result
  then  DBEditMessErrorAndTryFocusL (RS_EdiTAmp_Input, L, DBE);
end {TntDBEditValidL};

function TntDBLookupComboValid (DBLCB: TTntJvDBLookupCombo): Boolean;
begin
  Result := Assigned (DBLCB.Field) and (DBLCB.Field.AsString <> '');
  if  not Result
  then  DBEditMessErrorAndTryFocus (RS_EdiTAmp_Input_ByList, DBLCB.Field, DBLCB);
end {TntDBLookupComboBoxValid};

function TntDBLookupEditValid (DBLE: TTntJvDBLookupEdit): Boolean;
begin
  Result := DBLE.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocus (RS_EdiTAmp_Input_Text, DBLE);
end {TntDBLookupEditValid};

function TntDBLookupEditValidL (L: TTntLabel;  DBLE: TTntJvDBLookupEdit): Boolean;
begin
  Result := DBLE.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocusL (L, DBLE);
end {TntDBLookupEditValidL};

function TntDBRadioGroupValid (DBRGV: TTntDBRadioGroup): Boolean;
begin
  Result := Assigned (DBRGV.Field) and RadioGroupRValidNoMess (DBRGV.Field.Required, DBRGV.ItemIndex);
  if  not Result  then  begin
    WideMessageBoxError (WideFormat (RS_EdiTAmp_RGNoItemSelected,
      [DelAmpersandW (DBRGV.Caption)]));
  end;
end {TntDBRadioGroupValid};

function TntDirEditValid (DE: TTntJvDirectoryEdit): Boolean;
begin
  Result := DE.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocus (RS_EdiTAmp_Input_Dir, DE);
end {TntDirEditValid};

function TntDirEditValidL (L: TTntLabel;  DE: TTntJvDirectoryEdit): Boolean;
begin
  Result := DE.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocusL (L, DE);
end {TntDirEditValidL};

function TntEditValid (E: TTntCustomEdit): Boolean;
begin
  Result := E.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocus (RS_EdiTAmp_Input, E);
end {TntEditValid};

function TntEditValidL (L: TTntLabel;  E: TTntCustomEdit): Boolean;
begin
  Result := E.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocusL (L, E);
end {TntEditValidL};

function TntFileEditValid (FE: TTntJvFilenameEdit): Boolean;
begin
  Result := FE.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocus (RS_EdiTAmp_Input_File, FE);
end {TntFileEditValid};

function TntFileEditValidL (L: TTntLabel;  FE: TTntJvFilenameEdit): Boolean;
begin
  Result := FE.Text <> '';
  if  not Result
  then  EditMessErrorAndTryFocusL (L, FE);
end {TntFileEditValidL};

function TntRadioGroupValid (RGV: TTntRadioGroup): Boolean;
begin
  Result := RadioGroupRValidNoMess (True, RGV.ItemIndex);
  if  not Result  then  begin
    WideMessageBoxError (WideFormat (RS_EdiTAmp_RGNoItemSelected,
      [DelAmpersandW (RGV.Caption)]));
  end;
end {TntDBRadioGroupValid};

{------------------}

procedure ChangeCapsKeyStatus (var Key: WideChar;
  CaseMode: TCaseMode;  SelStart: Integer;  const Text: WideString;
  FCharsWithoutModifiers: Boolean);
begin
  case CaseMode of
  cmProperCase : begin
      if  SelStart = 0
      then  Key := WideUpperCase(Key)[1]
      else if  (Text[SelStart] <> ' ') and WCharIsLetterUppercase(Key) and
        WideUserSayYesWarning (RS_EdiTAmp_SelectUpcaseToLocase)
      then  Key := WideLowerCase(Key)[1];
    end;
  cmUpperCase  : Key := WideUpperCase(Key)[1];
  cmLowerCase  : Key := WideLowerCase(Key)[1];
  end;
  if FCharsWithoutModifiers then  Key := WCharToWithoutModifierWChar (Key);
end {ChangeCapsKeyStatus};

function IsNameChar (Key: WideChar;  const ExtraChars: WideString): Boolean;
begin
  Result := WCharIsLetter (Key) or (Pos(Key, ExtraChars) <> 0) or (Key < #32);
end {IsNameChar};

function ChangedCaseModeStr (var S: WideString;  CaseMode: TCaseMode): Boolean;
var
  R: WideString;
begin
  if  (S = '') or (CaseMode = cmNormal)  then  begin
    ChangedCaseModeStr := False;
  end
  else  begin
    case CaseMode of
    cmProperCase: begin
        R := WideLowerCase (S);
        R[1] := WideUpCase (R[1]);
      end;
    cmLowerCase: R := WideLowerCase (S);
    cmUpperCase: R := WideUpperCase (S);
    end;
    ChangedCaseModeStr := S <> R;
    S := R;
  end;
end {ChangedCaseModeStr};

procedure PasteChars (WithBeginOrEndSpaces: Boolean;  CaseMode: TCaseMode;
  SelStart: Integer;  Text: WideString);
var
  R: WideString;
  L: Integer;
begin
  R := TntClipboard.AsText;
  L := Length (R);
  if  L > 0  then  begin
    if  (not WithBeginOrEndSpaces) and (SelStart = 0)
    then  R := WideDelBSpace (R);
    L := Length (R);
    if  L > 0  then  begin
      if  (not WithBeginOrEndSpaces) and (SelStart = Length(Text))
      then  R := WideDelESpace (R);
    end;
    case CaseMode of
    cmProperCase: begin
        R := WideLowerCase (R);
        if  (SelStart = 0) and (Length(R) > 0)
        then  R[1] := WideUpCase (R[1]);
      end;
    cmLowerCase: R := WideLowerCase (R);
    cmUpperCase: R := WideUpperCase (R);
    end;
    TntClipboard.AsText := R;
  end;
end {PasteChars};

procedure PasteNameChars (const ExtraChars: WideString);
var
  S, R: WideString;
  L, I: Integer;
begin
  S := TntClipboard.AsText;
  L := Length (S);
  if  L > 0  then  begin
    R := '';
    for  I := 1 to L  do
      if  IsNameChar (S[I], ExtraChars)
      then  R := Concat (R, S[I]);
    TntClipboard.AsText := R;
  end;
end {PasteNameChars};

function RadioGroupRValid (FRequired: Boolean;  const WinCtrlTitle: WideString;
  ItemIndex: Integer;  WC: TWinControl): Boolean;
var
  Ok: Boolean;
begin
  Ok := RadioGroupRValidNoMess (FRequired, ItemIndex);
  if  not Ok  then  begin
    WideMessageBoxError (WideFormat (RS_EdiTAmp_RGNoItemSelected, [WinCtrlTitle]));
    WC.SetFocus;
  end;
  Result := Ok;
end {RadioGroupRValid};

function EditRValid (FRequired: Boolean;  const WinCtrlTitle, Text: WideString;
  WC: TWinControl;  FromDBEdit: Boolean): Boolean;
var

  FEM, S, S1: WideString;
  C1: WideString;
  I: Byte;
  Literal: Boolean;
begin
  Result := True;
  S := '';
  if FRequired then  begin
    if  FromDBEdit  then  begin
      FEM := TTntDBEditR(WC).Field.EditMask;
      if  FEM <> '' then  begin
        S1 := Copy (FEM, 1, Pos(';',FEM)-1);
        C1 := ' ';
        Literal := False;
        for  I := 1 to length (S1)  do  begin
          if Literal then  begin
            C1[1] := S1[I];
            S := Concat (S, C1);
            Literal := False;
          end
          else begin
            if  Pos ('AaCc09#LI', S1[I]) <> 0  then  begin
              C1[1] := FEM[Length(FEM)];
              S := Concat (S, C1);
            end
            else if  Pos (':\', S1[I]) <> 0  then  begin
              C1[1] := S1[I];
              S := Concat (S, C1);
            end
            else if  S1[I] = '\'
            then  Literal := True
            else if  Pos ('!><', S1[I]) <> 0
            then;
          end;
        end;
      end;
    end;
    if (Text = '') or (Text = S) then  begin
      if  WinCtrlTitle <> '' then  begin
        S := WideFormat (RS_EdiTAmp_EmptyDefinedEdit, [WinCtrlTitle]);
        if FromDBEdit
        then  S := Concat (S, RS_EdiTAmp_RewriteOrCancelDBEdit);
        WideMessageBoxError (S);
      end
      else
        WideMessageBoxError (RS_EdiTAmp_EmptyEdit);
      WC.SetFocus;
      Result := False;
    end;
  end;
end {EditRValid};

function EditTxtValid (FRequired, FAbbreviation: Boolean;  const WinCtrlTitle, Text: WideString;
  WC: TWinControl;  FromDBEdit: Boolean): Boolean;
var
  IsValid: Boolean;
begin
  IsValid := EditRValid (FRequired, WinCtrlTitle, Text, WC, FromDBEdit);
  if IsValid and not FAbbreviation and StrIsAbbreviatedW (Text)
  then  begin
    WideMessageBoxError (WideFormat (RS_EdiTAmp_EditWithAbbrev, [WinCtrlTitle]));
    WC.SetFocus;
    IsValid := False;
  end;
  Result := IsValid;
end {EditTxtValid};

{--------------------------------------}
{-PROTECTED--------}

procedure TTntDBRadioGroupR.KeyPress (var Key: Char);
begin
  if  (Key = Backspace) and Assigned(Field) and (not Field.Required)
  then  begin
    Key := #0;
    ItemIndex := -1;
    Exit;
  end;
  inherited KeyPress(Key);
end {KeyPress};

function DeleteLetters (const S: WideString): WideString;
var
  I: Integer;
begin
  Result := '';
  if  S = ''
  then  Exit;
  for I := 1 to Length (S)  do
    if  not WCharIsLetter (S[I])
    then  Result := Concat (Result, S[I]);
end {DeleteLetters};

{-PUBLIC-----------}

function TTntDBRadioGroupR.Valid: Boolean;
begin
  Result := TntDBRadioGroupValid (Self);
end;

function TTntDBRadioGroupR.ValidNoMess: Boolean;
begin
  Result := Assigned(Field) and RadioGroupRValidNoMess (Field.Required, ItemIndex);
end;

{--------------------------------------}
{-PROTECTED--------}

procedure TTntRadioGroupR.KeyPress (var Key: Char);
begin
  if  (Key = Backspace) and (not FRequired)
  then  begin
    Key := #0;
    ItemIndex := -1;
    Exit;
  end;
  inherited KeyPress(Key);
end {KeyPress};

{-PUBLIC-----------}

constructor TTntRadioGroupR.Create (AOwner : TComponent);
begin
  inherited Create(AOwner);
  FRequired := True;
end {Create};

function TTntRadioGroupR.Valid: Boolean;
begin
  Result := RadioGroupRValid (FRequired, DelAmpersandW (Caption), ItemIndex, Self);
end;

function TTntRadioGroupR.ValidNoMess: Boolean;
begin
  Result := RadioGroupRValidNoMess (FRequired, ItemIndex);
end;

{--------------------------------------}
{-PUBLIC-----------}

function TTntDBEditR.Valid: Boolean;
begin
  Result := Assigned (Field) and ((not (Field.Required)) or TntDBEditValid (Self));
end {Valid};

{--------------------------------------}
{-PRIVATE----------}

procedure TTntDBEditTxt.CMExit (var message: TCMExit);
begin
  if not FWithBeginOrEndSpaces
  then  Text := WideDelESpace (Text);
end {CMExit};

procedure TTntDBEditTxt.WMPaste(var Message: TMessage);
begin
  if ReadOnly then
    inherited
  else
  begin
    PasteChars (FWithBeginOrEndSpaces, CaseMode, SelStart, Text);
    inherited
  end;
end {WMPaste};

{-PROTECTED--------}

procedure TTntDBEditTxt.KeyPressW(var Key: WideChar);

begin
  if  (Key = ' ') and not FWithBeginOrEndSpaces and ((SelStart = 0) or (Text[SelStart] = ' '))
  then  begin
    Key := #0;
    Exit;
  end;
  ChangeCapsKeyStatus (Key, CaseMode, SelStart, Text, FCharsWithoutModifiers);
end {KeyPressW};

{-PUBLIC-----------}

constructor TTntDBEditTxt.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FCase := ecNormal;
  CaseMode := cmProperCase;
end {Create};

function TTntDBEditTxt.Valid: Boolean;
begin
  Valid := EditTxtValid (Field.Required, FAbbreviation, DataField, Text, Self, True);
end {Valid};

{--------------------------------------}
{-PRIVATE----------}

function TTntDBEditName.ExtraCharsStored: Boolean;
begin
  Result := ExtraChars <> DefaultExtraNameChars;
end;

procedure TTntDBEditName.WMPaste(var Message: TMessage);
begin
  if ReadOnly then
    inherited
  else
  begin
    PasteNameChars (ExtraChars);
    inherited
  end;
end {WMPaste};

procedure TTntDBEditName.SetExtraChars(const Value: WideString);
begin
  FExtraChars := DeleteLetters (Value);
end;

{-PROTECTED--------}

procedure TTntDBEditName.KeyPressW (var Key: WideChar);
begin
  if  not IsNameChar (Key, ExtraChars)  then  begin
    Key := #0;
    Exit;
  end;
  inherited KeyPressW(Key);
end {KeyPressW};

{-PUBLIC-----------}

constructor TTntDBEditName.Create(AOwner: TComponent);
begin
  inherited;
  ExtraChars := DefaultExtraNameChars;
end;

{--------------------------------------}
{-PROTECTED--------}

function TTntEditR.InputName: WideString;
begin
  Result := RS_EdiTAmp_Input;
end;

{-PUBLIC-----------}

constructor TTntEditR.Create (AOwner : TComponent);
begin
  inherited Create(AOwner);
  FRequired := True;
end {Create};

procedure TTntEditR.CreateWindowHandle(const Params: TCreateParams);
begin
  inherited;
  if (csDesigning in ComponentState)
  then  Text := '';
end {CreateWindowHandle};

function TTntEditR.Valid: Boolean;
begin
  Result := (not FRequired) or (Text <> '');
  if  not Result
  then  EditMessErrorAndTryFocus (InputName, Self);
end;

function TTntEditR.ValidL (L: TTntLabel): Boolean;
begin
  Result := (not FRequired) or (Text <> '');
  if  not Result
  then  EditMessErrorAndTryFocusL (L, Self);
end;

{--------------------------------------}
{-PRIVATE----------}

procedure TTntEditNoBegSpace.CMExit (var message: TCMExit);
begin
  Text := WideDelESpace (Text);
end {CMExit};

{-PROTECTED--------}

function TTntEditNoBegSpace.OtherKeyNoValid (Key: Char): Boolean;
begin
  Result := False;
end;

procedure TTntEditNoBegSpace.KeyPress (var Key: Char);
begin
  if  ((Key = ' ') and (SelStart = 0)) or (OtherKeyNoValid (Key))  then  begin
    Key := #0;
    Exit;
  end;
  inherited KeyPress(Key);
end {KeyPress};

{--------------------------------------}
{-PRIVATE----------}

procedure TTntEditOnlyFileName.WMPaste(var Message: TMessage);
var
  R, S: WideString;
  L, I: Integer;
begin
  if ReadOnly then
    inherited
  else
  begin
    R := TntClipboard.AsText;
    L := Length (R);
    if  L > 0  then  begin
      S := '';
      for  I := 1 to L  do
        if  Pos (R[I], InvalidFileNameChars) = 0
        then  S := Concat (S, R[I]);
      TntClipboard.AsText := S;
    end;
    inherited
  end;
end {WMPaste};

{-PROTECTED--------}

function TTntEditOnlyFileName.InputName: WideString;
begin
  Result := RS_EdiTAmp_Input_Filename;
end;

function TTntEditOnlyFileName.OtherKeyNoValid (Key: Char): Boolean;
begin
  Result := Pos (Key, InvalidFileNameChars) <> 0;
end;

{--------------------------------------}
{-PRIVATE----------}

procedure TTntEditTxt.CMExit (var message: TCMExit);
begin
  if not FWithBeginOrEndSpaces
  then  Text := WideDelESpace (Text);
end {CMExit};

procedure TTntEditTxt.WMPaste(var Message: TMessage);
begin
  if ReadOnly then
    inherited
  else
  begin
    PasteChars (FWithBeginOrEndSpaces, CaseMode, SelStart, Text);
    inherited
  end;
end {WMPaste};

procedure TTntEditTxt.SetCaseMode (const Value: TCaseMode);
var
  S: WideString;
begin
  FCaseMode := Value;
  S := Text;
  if  ChangedCaseModeStr (S, Value)
  then  Text := S;
end {SetCaseMode};

procedure TTntEditTxt.SetCharsWithoutModifiers (const Value: Boolean);
var
  S: WideString;
begin
  FCharsWithoutModifiers := Value;
  if Value then  begin
    S := WStrToWithoutModifierWStr (Text);
    if  S <> Text
    then  Text := S;
  end;
end {SetCharsWithoutModifiers};

procedure TTntEditTxt.SetWithBeginOrEndSpaces (const Value: Boolean);     
var
  S: WideString;
begin
  FWithBeginOrEndSpaces := Value;
  if  not Value  then  begin
    S := WideDelRSpace (Text);
    if  S <> Text
    then  Text := S;
  end;
end {SetWithBeginOrEndSpaces};

{-PROTECTED--------}

function TTntEditTxt.InputName: WideString;
begin
  Result := RS_EdiTAmp_Input_Text;
end;

procedure TTntEditTxt.KeyPressW(var Key: WideChar);
begin
  if  (Key = ' ') and not FWithBeginOrEndSpaces and ((SelStart = 0) or (Text[SelStart] = ' '))
  then  begin
    Key := #0;
    Exit;
  end;
  ChangeCapsKeyStatus (Key, CaseMode, SelStart, Text, FCharsWithoutModifiers);
end {KeyPressW};

{-PUBLIC-----------}

constructor TTntEditTxt.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FCase := ecNormal;
  CaseMode := cmProperCase;
  WithBeginOrEndSpaces := True;
end {Create};

function TTntEditTxt.Valid: Boolean;
begin
  Valid := EditTxtValid (FRequired, FAbbreviation, InputName, Text, Self, False);
end;

function TTntEditTxt.ValidL (L: TTntLabel): Boolean;
begin
  Result := EditTxtValid (FRequired, FAbbreviation, CaptFromLabel(L), Text, Self, False);
end;

{--------------------------------------}
{-PRIVATE----------}

function TTntEditName.ExtraCharsStored: Boolean;
begin
  Result := ExtraChars <> DefaultExtraNameChars;
end;

procedure TTntEditName.WMPaste(var Message: TMessage);
begin
  if ReadOnly then
    inherited
  else
  begin
    PasteNameChars (ExtraChars);
    inherited
  end;
end {WMPaste};

procedure TTntEditName.SetExtraChars(const Value: WideString);
begin
  FExtraChars := DeleteLetters (Value);
end;

{-PROTECTED--------}

function TTntEditName.InputName: WideString;
begin
  Result := RS_EdiTAmp_Input_Name;
end;

procedure TTntEditName.KeyPressW (var Key: WideChar);
begin
  if  not IsNameChar (Key, ExtraChars)  then  begin
    Key := #0;
    Exit;
  end;
  inherited KeyPressW(Key);
end {KeyPressW};

{-PUBLIC-----------}

constructor TTntEditName.Create(AOwner: TComponent);
begin
  inherited;
  WithBeginOrEndSpaces := False;
  ExtraChars := DefaultExtraNameChars;
end;

{--------------------------------------}
{-PUBLIC-----------}

constructor TTntComboBoxR.Create (AOwner : TComponent);
begin
  inherited Create(AOwner);
  FRequired := True;
end {Create};

procedure TTntComboBoxR.CreateWindowHandle(const Params: TCreateParams);
begin
  inherited;
  if (csDesigning in ComponentState)
  then  Text := '';
end {CreateWindowHandle};

function TTntComboBoxR.Valid: Boolean;
begin
  Result := (not FRequired) or TntComboBoxValid (Self);
end;

function TTntComboBoxR.ValidL (L: TTntLabel): Boolean;
begin
  Result := (not FRequired) or TntComboBoxValidL (L, Self);
end;

{--------------------------------------}
{-PUBLIC-----------}

procedure TTntComboBoxRWLI.AddLastInput;
var
  P: Integer;
  S: WideString;
begin
  S := Text;
  if  S = ''
  then  Exit;
  P := Items.IndexOf (S);
  if  P <> -1
  then  Items.Delete (P);
  Items.Insert (0, S);
  if (FMaxItemsCount > 0) and (Items.Count > FMaxItemsCount)
  then  Items.Delete (Items.Count-1);
  Text := S;
end {AddLastInput};

{--------------------------------------}

end.
