{---------------------------------------------------------------------

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/01/2006 version

----------------------------------------------------------------------}

unit TntFileUtil;

interface

uses
  Classes;

function GetFileSizeW (const FileName: WideString): Int64;
function FileDateTimeW (const FileName: WideString): TDateTime;

type
  TFileOperation = (foCopy, foDelete, foMove, foRename);
  TFileOperFlag = (flAllowUndo, flConfirmMouse, flFilesOnly, flMultiDest,
    flNoConfirmation, flNoConfirmMkDir, flRenameOnCollision, flSilent,
    flSimpleProgress, flNoErrorUI);
  TFileOperFlags = set of TFileOperFlag;

  TTntJvFileOperator = class(TComponent)
  private
    FAborted: Boolean;
    FOperation: TFileOperation;
    FOptions: TFileOperFlags;
    FProgressTitle: WideString;
    FSource: WideString;
    FDestination: WideString;
    function TaskModalDialog(DialogFunc: Pointer; var DialogData): Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    function Execute: Boolean; virtual;
    property Aborted: Boolean read FAborted;
  published
    property Destination: WideString read FDestination write FDestination;
    property Operation: TFileOperation read FOperation write FOperation
      default foCopy;
    property Options: TFileOperFlags read FOptions write FOptions
      default [flAllowUndo, flNoConfirmMkDir];
    property ProgressTitle: WideString read FProgressTitle write FProgressTitle;
    property Source: WideString read FSource write FSource;
  end;

implementation

uses
  Forms, Windows, ShellApi, TntWindows, SysUtils, TntSysUtils, TntWideStrUtils;

function GetFileSizeW (const FileName: WideString): Int64;
var
  SearchRec: TSearchRecW;
begin
  if WideFindFirst (WideExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
    Result := SearchRec.Size
  else Result := -1;
  WideFindClose(SearchRec);
end {GetFileSizeW};

function FileDateTimeW (const FileName: WideString): TDateTime;
var
  Age: Longint;
begin
  Age := WideFileAge (FileName);
  if Age = -1 then
    Result := 0
  else
    Result := FileDateToDateTime (Age);
end {FileDateTimeW};

{--------------------------------------}

constructor TTntJvFileOperator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOptions := [flAllowUndo, flNoConfirmMkDir];
end;

function TTntJvFileOperator.TaskModalDialog(DialogFunc: Pointer; var DialogData): Boolean;
type
  TDialogFunc = function(var DialogData): Integer stdcall;
var
  ActiveWindow: HWnd;
  WindowList: Pointer;
begin
  ActiveWindow := GetActiveWindow;
  WindowList := DisableTaskWindows(0);
  try
    Result := TDialogFunc(DialogFunc)(DialogData) = 0;
  finally
    EnableTaskWindows(WindowList);
    SetActiveWindow(ActiveWindow);
  end;
end;

function TTntJvFileOperator.Execute: Boolean;
const
  OperTypes: array[TFileOperation] of UINT = (
    FO_COPY, FO_DELETE, FO_MOVE, FO_RENAME);
  OperOptions: array[TFileOperFlag] of FILEOP_FLAGS = (
    FOF_ALLOWUNDO, FOF_CONFIRMMOUSE, FOF_FILESONLY, FOF_MULTIDESTFILES,
    FOF_NOCONFIRMATION, FOF_NOCONFIRMMKDIR, FOF_RENAMEONCOLLISION,
    FOF_SILENT, FOF_SIMPLEPROGRESS, FOF_NOERRORUI);
var
  OpStruct: TSHFileOpStructW;
  Flag: TFileOperFlag;

  function AllocFileStr(const S: WideString): PWideChar;
  var
    P: PWideChar;
  begin
    Result := nil;
    if S <> '' then begin
      Result := WStrCopy(WStrAlloc(Length(S) + 2), PWideChar(S));
      P := Result;
      while P^ <> #0 do begin
        if (P^ = ';') or (P^ = '|') then P^ := #0;
        Inc(P);
      end;
      Inc(P);
      P^ := #0;
    end;
  end;

begin
  FAborted := False;
  FillChar(OpStruct, SizeOf(OpStruct), 0);
  with OpStruct do
  try
    if (Application.MainForm <> nil) and
      Application.MainForm.HandleAllocated then
      Wnd := Application.MainForm.Handle
    else Wnd := Application.Handle;
    wFunc := OperTypes[Operation];
    pFrom := AllocFileStr(FSource);
    pTo := AllocFileStr(FDestination);
    fFlags := 0;
    for Flag := Low(Flag) to High(Flag) do
      if Flag in FOptions then fFlags := fFlags or OperOptions[Flag];
    lpszProgressTitle := PWideChar(FProgressTitle);
    Result := TaskModalDialog(@SHFileOperation, OpStruct);
    FAborted := fAnyOperationsAborted;
  finally
    if pFrom <> nil then WStrDispose(pFrom);
    if pTo <> nil then WStrDispose(pTo);
  end;
end;

{--------------------------------------}

end.
