unit TntIACheckControlJ;

{---------------------------------------------------------------------

  TTntCheckBoxMLines and TTntRadioButtonMLines are Controls
  similar to TCheckBox and TRadioButton,
  with Multilines Caption
  modified from Igor Andronov's IACheckControl, by:
  
 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/01/2006 version

----------------------------------------------------------------------}

{$I jvcl.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TntClasses, TntControls,
  TntClasses2, TntJvExControls, JvConsts;

type
  TIACheckControlType = (iacctCheck, iacctSelect);
  TIACheckControlVAlign = (iaccvaTop, iaccvaBottom, iaccvaCenter);
  TIACheckControlChangeStateProcedure = procedure (ASender: TPersistent; ANewStateSelected: Boolean) of object;

  TTntCheckBoxMLines = class (TTntJvExCustomControl)
  private
    FAlignment: TAlignment;
    FAutomaticWrap: Boolean;
    FCaptionEditor: TUnicodeLinesEditor;
    FControlType: TIACheckControlType;
    FGroupIndex: Cardinal;
    FChecked: Boolean;
    FCheckVerticalAlign: TIACheckControlVAlign;
    FGapTextToImage: Byte;
    FPaintAllBackground: Boolean;
    FTransparent: Boolean;
    FShowFocus: Boolean;
    FOnChangeState: TIACheckControlChangeStateProcedure;
    function  IsFontStored: Boolean;
    function  GetCaptionEditor: TUnicodeLinesEditor;
    procedure SetCaptionEditor(const Value: TUnicodeLinesEditor);
    function  GetFont: TFont;
    procedure SetFont (Value: TFont);
    function  GetHeight: Integer;
    procedure SetHeight (Value: Integer);
    function  GetWidth: Integer;
    procedure SetWidth (Value: Integer);

    procedure WMKeyUp (var Message: TWMKeyUp); message WM_KEYUP;
    procedure WMLButtonDown (var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonDblClk (var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure WMSIZE (var Message: TMessage); message WM_SIZE;
    procedure WMSETFOCUS (var Message: TMessage); message WM_SETFOCUS;
    procedure WMKILLFOCUS (var Message: TMessage); message WM_KILLFOCUS;
    function  Alignment2: TAlignment;
    procedure SetControlType (Value: TIACheckControlType);
    procedure FontIsChange (Sender: TObject);
    procedure SetAlignment (Value: TAlignment);
    procedure SetAutomaticWrap (Value: Boolean);
    procedure SetChecked (Value: Boolean);
    procedure SetGroupIndex (Value: Cardinal);
    procedure SetPaintAllBackground (Value: Boolean);
    procedure SetCheckVerticalAlign (Value: TIACheckControlVAlign);
    function  GetTextSize: TSize;
    function  GetFullSize: TSize;
    function  GetBasis: TSize;
    function  GetOffset: TSize;
    function  GetBMPBasis: TSize;
    function  GetEnabled: Boolean; reintroduce;
    procedure SetEnabled (Value: Boolean); reintroduce;
    procedure DrawFocus;
    procedure SetGapTextToImage (Value: Byte);
    procedure SetTransparent (Value: Boolean);
    procedure SetProperlyRGN;
    procedure SetShowFocus (Value: Boolean);
    procedure SetOnChangeState (Value: TIACheckControlChangeStateProcedure);
    function  GetVisible: Boolean;
    procedure SetVisible (Value: Boolean);
  protected
    procedure AdjustSize; override;
    procedure CreateHandle; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DoDrawText (var Rect: TRect; Flags: Longint); dynamic;
    procedure DrawCheckBMP; virtual;
    function  GetBMPSize: TSize; virtual;
    procedure Paint; override;
    procedure SetText (const Value: TWideCaption); override;
  public
    procedure Assign (Source: TPersistent); override;
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    function  GetControlsAlignment: TAlignment; override;
    procedure SetBounds (ALeft, ATop, AWidth, AHeight: Integer); override;
    property FullSize: TSize read GetFullSize;
    property TextSize: TSize read GetTextSize;
  published
    //Opened properties from TControl
    property Action;
    property Align;
    property Anchors;
    property BiDiMode;
    property Caption write SetText;
    property Color;
    property Constraints;
    property Enabled: Boolean read GetEnabled write SetEnabled stored False;
    property Font: TFont read GetFont write SetFont stored IsFontStored;
    property Height: Integer read GetHeight write SetHeight;
    property HintColor;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Visible: Boolean read GetVisible write SetVisible stored False;
    property Width: Integer read GetWidth write SetWidth;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    {$IFDEF VCL}
    property OnMouseEnter;
    property OnMouseLeave;
    {$ENDIF VCL}
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;

    //Opened properties from TWinControl
    property TabOrder;
    property TabStop;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    //New properties
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property AutomaticWrap: Boolean read FAutomaticWrap write SetAutomaticWrap;
    property CaptionEditor: TUnicodeLinesEditor read GetCaptionEditor write SetCaptionEditor stored False;
    property CheckVerticalAlign: TIACheckControlVAlign read FCheckVerticalAlign write SetCheckVerticalAlign default iaccvaTop;
    property Checked: Boolean read FChecked write SetChecked stored True;
    property ControlType: TIACheckControlType read FControlType write SetControlType default iacctCheck;
    property GapTextToImage: Byte read FGapTextToImage write SetGapTextToImage default 4;
    property GroupIndex: Cardinal read FGroupIndex write SetGroupIndex default 0;
    property OnChangeState: TIACheckControlChangeStateProcedure read FOnChangeState write SetOnChangeState;
    property PaintAllBackground: Boolean read FPaintAllBackground write SetPaintAllBackground; 
    property ShowFocus: Boolean read FShowFocus write SetShowFocus default True;
    property Transparent: Boolean read FTransparent write SetTransparent default True;
  end;

  TTntRadioButtonMLines = class (TTntJvExCustomControl{TTntJvExGraphicControl})
  private
    FAlignment: TLeftRight;
    FCaptionEditor: TUnicodeLinesEditor;
    FChecked: Boolean;
    FGapTextToImage: Byte;
    procedure SetAlignment(Value: TLeftRight);
    function  GetCaptionEditor: TUnicodeLinesEditor;
    procedure SetCaptionEditor(const Value: TUnicodeLinesEditor);
    procedure DrawCheckBMP;
    procedure FontIsChange (Sender: TObject);
    function  GetBMPBasis: TSize;
    function  GetBMPSize: TSize;
    function  GetBasis: TSize;
    function  GetFullSize: TSize;
    function  GetOffset: TSize;
    function  GetTextSize: TSize;
    procedure WMLButtonDown (var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure SetGapTextToImage (Value: Byte);
    procedure SetChecked (Value: Boolean);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DoDrawText (var Rect: TRect; Flags: Longint); dynamic;
    procedure Paint; override;
    procedure SetText (const Value: TWideCaption); override;
  public
    procedure Assign (Source: TPersistent); override;
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    property FullSize: TSize read GetFullSize;
    property TextSize: TSize read GetTextSize;
  published
    property Action;
    property Alignment: TLeftRight read FAlignment write SetAlignment default taRightJustify;
    property Caption write SetText;
    property CaptionEditor: TUnicodeLinesEditor read GetCaptionEditor write SetCaptionEditor stored False;
    property Color;
    property Font;
    property Height;
    property HintColor;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    {$IFDEF VCL}
    property OnMouseEnter;
    property OnMouseLeave;
    {$ENDIF VCL}
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property Checked: Boolean read FChecked write SetChecked stored True;
    property GapTextToImage: Byte read FGapTextToImage write SetGapTextToImage default 4;
  end;

implementation

{$R *.RES}

uses
  Math,
  TntGraphics, TntSysUtils, TntWideStrUtils;

{--------------------------------------}

function WStrReplaceSimple (InStr: WideString;
  const SearchStr, ReplaceStr: WideString): WideString;
var
  I: Integer;
begin
  if  SearchStr = ''  then  begin
    Result := InStr;
    Exit;
  end
  else
    Result := '';
  while  InStr <> ''  do  begin
    I := Pos (SearchStr, InStr);
    if  I > 0  then  begin
      Result := Concat (Result, Copy (InStr, 1, I-1), ReplaceStr);
      InStr := Copy (InStr, I+Length(SearchStr), MaxInt);
    end
    else  begin
      Result := Concat (Result, InStr);
      InStr := '';
    end;
  end;
end {WStrReplaceSimple};

function WStrUntilDelimiter (var InStr: WideString;
  const Delim: WideString): WideString;
var
  I: Integer;
begin
  I := Pos (Delim, InStr);
  if  I = 0  then  begin
    Result := InStr;
    InStr := '';
  end
  else  begin
    Result := Copy (InStr, 1, I-1);
    Delete (InStr, 1, Length(Delim)+I-1);
  end;
end {WStrUntilDelimiter};

{--------------------------------------}

function GetCColor (Color01, Color02: TColor;  R, I: Integer): TColor;
var
  C1, C2: TColor;
  R1, G1, B1, R2, G2, B2: Byte;
  Nr, Ng, Nb: Integer;
begin
  c1 := ColorTorgb (Color01);
  c2 := ColorTorgb (Color02);
  R1 := Pbyte (@C1)^;
  G1 := Pbyte (Integer(@C1)+1)^;
  B1 := Pbyte (Integer(@C1)+2)^;
  R2 := Pbyte (@C2)^;
  G2 := Pbyte (Integer(@C2)+1)^;
  B2 := Pbyte (Integer(@C2)+2)^;
  if R <> 0 then
    begin
      Nr := (R1+(R2-R1)*I div R);
      Ng := (G1+(G2-G1)*I div R);
      Nb := (B1+(B2-B1)*I div R);
      if  Nr < 0    then Nr := 0;
      if  Nr > 255  then Nr := 255;
      if  Ng < 0    then Ng := 0;
      if  Ng > 255  then Ng := 255;
      if  Nb < 0    then Nb := 0;
      if  Nb > 255  then Nb := 255;
      Result := RGB (Nr, Ng, Nb);
    end else Result := Color01;
end {GetCColor};

{--------------------------------------}
{-PRIVATE----------}

procedure TTntCheckBoxMLines.CreateHandle;
begin
  inherited CreateHandle;
  FShowFocus := True;
  SetProperlyRGN;
end {CreateHandle};

procedure TTntCheckBoxMLines.CreateParams(var Params: TCreateParams);
const
  Alignments: array[Boolean, TAlignment] of DWORD =
    ((SS_LEFT, SS_RIGHT, SS_CENTER), (SS_RIGHT, SS_LEFT, SS_CENTER));
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'BUTTON');
  with Params do
  begin
    Style := Style or BS_3STATE or
      Alignments[UseRightToLeftAlignment, FAlignment];
    WindowClass.style := WindowClass.style and not (CS_HREDRAW or CS_VREDRAW);
  end;
end;

procedure TTntCheckBoxMLines.DrawFocus;
var
  s1, s5: TSize;
  RGN2: HRGN;
  P1: array [0..3] of TPoint;
  Ac: TColor;
begin
  if HandleAllocated then  begin
    s1 := GetBasis;
    s5 := TextSize;
    p1[0].x := s1.Cx;
    p1[0].y := s1.Cy;
    p1[1].x := s1.Cx+s5.Cx;
    p1[1].y := s1.Cy;
    p1[2].x := p1[1].x;
    p1[2].y := p1[1].y+s5.Cy;
    p1[3].x := s1.Cx;
    p1[3].y := s1.Cy+s5.Cy;
    case Alignment2 of
    taLeftJustify :
      begin
      end;
    taRightJustify :
      begin
        Inc (p1[0].x, Width-s1.cx);
        Inc (p1[1].x, Width-s1.cx);
        Inc (p1[2].x, Width-s1.cx);
        Inc (p1[3].x, Width-s1.cx);
      end;
    taCenter :
      begin
        Inc (p1[0].x, (Width-s1.cx) div 2);
        Inc (p1[1].x, (Width-s1.cx) div 2);
        Inc (p1[2].x, (Width-s1.cx) div 2);
        Inc (p1[3].x, (Width-s1.cx) div 2);
      end;
    end;
    RGN2 := CreatePolygonRGN (P1, 4, Alternate);
    AC := GetCColor (Color, clBlack, 100, 30);
    Canvas.Brush.Color := Ac;
    FrameRgn (Canvas.Handle, RGN2, Canvas.Brush.Handle, 1, 1);
    Deleteobject (RGN2);
  end;
end {DrawFocus};

procedure TTntCheckBoxMLines.FontIsChange (Sender: TObject);
begin
  SetProperlyRGN;
  Invalidate;
end;

function TTntCheckBoxMLines.GetBasis: TSize;
var
  s1, s2, s3: TSize;
begin
  s1 := GetBmpSize;
  s2 := TextSize;
  s3.Cx := Abs (s2.Cx);
  s3.Cy := Abs (s2.Cy);
  Result.Cx := 0;
  Result.Cy := 0;
  case FCheckVerticalAlign of
    iaccvaTop :
      begin
        case Alignment2 of
          taLeftJustify :
            begin
              Result.Cx := s1.Cx+FGapTextToImage;
              Result.Cy := 0;
            end;
          taRightJustify :
            begin
              Result.Cx := 0;
              Result.Cy := 0;
            end;
          taCenter :
            begin
              Result.Cx := Max (0, (s1.Cx-s3.Cx) div 2);
              Result.Cy := s1.Cy+FGapTextToImage;
            end;
        end;
      end;
    iaccvaBottom :
      begin
        case Alignment2 of
          taLeftJustify :
            begin
              Result.Cx := s1.Cx+FGapTextToImage;
              Result.Cy := Max (0, s1.Cy-s3.Cy);
            end;
          taRightJustify :
            begin
              Result.Cx := 0;
              Result.Cy := Max (0, s1.Cy-s3.Cy);
            end;
          taCenter :
            begin
              Result.Cx := Max (0, (s1.Cx-s3.Cx) div 2);
              Result.Cy := 0;
            end;
        end;
      end;
    iaccvaCenter :
      begin
        case Alignment2 of
          taLeftJustify :
            begin
              Result.Cx := s1.Cx+FGapTextToImage;
              Result.Cy := Max (0, (s1.Cy-s3.Cy) div 2);
            end;
          taRightJustify :
            begin
              Result.Cx := 0;
              Result.Cy := Max (0, (s1.Cy-s3.Cy) div 2);
            end;
          taCenter :
            begin
              //No Action cause unavaible...
            end;
         end;
      end;
  end;
end {GetBasis};

function TTntCheckBoxMLines.GetBMPBasis: TSize;
var
  s1, s2, s3: TSize;
begin
  s1 := GetBmpSize;
  s2 := TextSize;
  s3.Cx := Abs (s2.Cx);
  s3.Cy := Abs (s2.Cy);
  Result.Cx := 0;
  Result.Cy := 0;
  case FCheckVerticalAlign of
    iaccvaTop :
      begin
        case Alignment2 of
          taLeftJustify :
            begin
              Result.Cx := 0;
              Result.Cy := 0;
            end;
          taRightJustify :
            begin
              Result.Cx := s3.Cx+FGapTextToImage;
              Result.Cy := 0;
            end;
          taCenter :
            begin
              Result.Cx := Max (0, -(s1.Cx-s3.Cx) div 2);
              Result.Cy := 0;
            end;
        end;
      end;
    iaccvaBottom :
      begin
        case Alignment2 of
          taLeftJustify :
            begin
              Result.Cx := 0;
              Result.Cy := Max (0, -s1.Cy+s3.Cy);
            end;
          taRightJustify :
            begin
              Result.Cx := s3.Cx+FGapTextToImage;
              Result.Cy := Max (0, -s1.Cy+s3.Cy);
            end;
          taCenter :
            begin
              Result.Cx := Max (0, -(s1.Cx-s3.Cx) div 2);
              Result.Cy := s3.Cy+FGapTextToImage;
            end;
        end;
      end;
    iaccvaCenter :
      begin
        case Alignment2 of
          taLeftJustify :
            begin
              Result.Cx := 0;
              Result.Cy := Max (0, -(s1.Cy-s3.Cy) div 2);
            end;
          taRightJustify :
            begin
              Result.Cx := s3.Cx+FGapTextToImage;
              Result.Cy := Max (0, -(s1.Cy-s3.Cy) div 2);
            end;
          taCenter :
            begin
              //No Action cause unavaible...
            end;
         end;
      end;
  end;
end {GetBMPBasis};

function TTntCheckBoxMLines.IsFontStored: Boolean;
begin
  Result := not ParentFont and not DesktopFont;
end;

function TTntCheckBoxMLines.GetCaptionEditor: TUnicodeLinesEditor;
begin
  Result := FCaptionEditor;
  FCaptionEditor.SetText (Caption);
end;

procedure TTntCheckBoxMLines.SetCaptionEditor(
  const Value: TUnicodeLinesEditor);
begin
  FCaptionEditor.Assign (Value);
  Caption := FCaptionEditor.GetText;
end;

function TTntCheckBoxMLines.GetEnabled: Boolean;
begin
  Result := inherited Enabled;
end;

function TTntCheckBoxMLines.GetFont: TFont;
begin
  Result := inherited Font;
end;

function TTntCheckBoxMLines.GetFullSize: TSize;
var
  s1, s2, s3: TSize;
begin
  s1 := GetBmpSize;
  s2 := TextSize;
  s3.Cx := Abs (s2.Cx);
  s3.Cy := Abs (s2.Cy);
  Result.Cx := 0;
  Result.Cy := 0;
  case FCheckVerticalAlign of
    iaccvaTop :
      begin
        case Alignment2 of
          taLeftJustify :
            begin
              Result.Cx := s1.Cx+s3.Cx+FGapTextToImage;
              Result.Cy := Max (s1.Cy, s3.Cy);
            end;
          taRightJustify :
            begin
              Result.Cx := s1.Cx+s3.Cx+FGapTextToImage;
              Result.Cy := Max (s1.Cy, s3.Cy);
            end;
          taCenter :
            begin
              Result.Cx := Max (s1.Cx, s3.Cx);
              Result.Cy := s1.Cy+s3.Cy+FGapTextToImage;
            end;
        end;
      end;
    iaccvaBottom :
      begin
        case Alignment2 of
          taLeftJustify :
            begin
              Result.Cx := s1.Cx+s3.Cx+FGapTextToImage;
              Result.Cy := Max (s1.Cy, s3.Cy);
            end;
          taRightJustify :
            begin
              Result.Cx := s1.Cx+s3.Cx+FGapTextToImage;
              Result.Cy := Max (s1.Cy, s3.Cy);
            end;
          taCenter :
            begin
              Result.Cx := Max (s1.Cx, s3.Cx);
              Result.Cy := s1.Cy+s3.Cy+FGapTextToImage;
            end;
        end;
      end;
    iaccvaCenter :
      begin
        case Alignment2 of
          taLeftJustify :
            begin
              Result.Cx := s1.Cx+s3.Cx+FGapTextToImage;
              Result.Cy := Max (s1.Cy, s3.Cy);
            end;
          taRightJustify :
            begin
              Result.Cx := s1.Cx+s3.Cx+FGapTextToImage;
              Result.Cy := Max (s1.Cy, s3.Cy);
            end;
          taCenter :
            begin
              //No Action cause unavaible...
            end;
         end;
      end;
  end;
end {GetFullSize};

function TTntCheckBoxMLines.GetHeight: Integer;
begin
  Result := inherited Height;
end;

function TTntCheckBoxMLines.GetOffset: TSize;
var
  s2, s3: TSize;
begin
  s2 := TextSize;
  s3.Cx := Abs (s2.Cx);
  s3.Cy := Abs (s2.Cy);
  Result.Cx := s3.Cx-s2.Cx;
  Result.Cy := 0;
end {GetOffset};

function TTntCheckBoxMLines.GetTextSize: TSize;
var
  S, S0: WideString;
  S1, S2: TSize;
begin
  if FAutomaticWrap then  begin
    Result.Cx := Width;
    Result.Cy := Height;
    S2 := GetBmpSize;
    if  ((FCheckVerticalAlign = iaccvaTop) or
         (FCheckVerticalAlign = iaccvaBottom)) and
        (FAlignment = taCenter)
    then  Result.Cy := Height -(S2.Cy + FGapTextToImage);
    if  ((FAlignment = taLeftJustify) or
         (FAlignment = taRightJustify))
    then  Result.Cx := Width -(S2.Cx + FGapTextToImage);
  end
  else  begin
    Result.Cx := 0;
    Result.Cy := 0;
    Canvas.Font.Assign (Font);
    if  Caption <> ''  then  begin
      S := Caption;
      while  S <> ''  do  begin
        S0 := WStrUntilDelimiter (S, sLineBreak);
        GetTextExtentPoint32W (Canvas.Handle, PWideChar(@S0[1]), Length(S0), s1);
        if  Result.Cx < s1.Cx
        then  Result.Cx := s1.Cx;
        Result.Cy := Result.Cy+s1.Cy;
      end;
    end;
    Result.Cx := Result.Cx + 2;
    Result.Cy := Result.Cy + 2;
  end;
end {GetTextSize};

function TTntCheckBoxMLines.GetVisible: Boolean;
begin
  Result := inherited Visible;
end;

function TTntCheckBoxMLines.GetWidth: Integer;
begin
  Result := inherited Width;
end;

procedure TTntCheckBoxMLines.SetAutomaticWrap (Value: Boolean); 
begin
  if Value <> FAutomaticWrap then  begin
    FAutomaticWrap := Value;
    SetProperlyRGN;
    Invalidate;
  end;
end {SetAutomaticWrap};

procedure TTntCheckBoxMLines.SetChecked (Value: Boolean);
var
  I: Integer;
  Sibling: TControl;
begin
  if Value <> FChecked then  begin
    FChecked := Value;
    if FChecked and (FControlType=iacctSelect) then  begin
      if assigned (Parent) and Parent.HandleAllocated then  begin
        for I := 0 to Parent.ControlCount-1 do  begin
          Sibling := Parent.Controls[I];
          if (Sibling is TTntCheckBoxMLines) and
             ((Sibling as TTntCheckBoxMLines) <> self) and
             ((Sibling as TTntCheckBoxMLines).FControlType=iacctSelect) and
             ((Sibling as TTntCheckBoxMLines).FGroupIndex=FGroupIndex) then
             (Sibling as TTntCheckBoxMLines).Checked := False;
        end;
      end;
    end;
    if assigned (FOnChangeState) then FOnChangeState (self, FChecked);
    SetProperlyRGN;
    Invalidate;
  end;
end {SetChecked};

function TTntCheckBoxMLines.Alignment2: TAlignment;
begin
  if not UseRightToLeftAlignment then
    Result := FAlignment
  else
    if FAlignment = taLeftJustify then
      Result := taRightJustify
    else
      Result := taLeftJustify;
end {Alignment2};

procedure TTntCheckBoxMLines.SetControlType (Value: TIACheckControlType);
var
  I: Integer;
  Sibling: TControl;
begin
  if Value <> FControlType then  begin
    FControlType := Value;
    if FChecked and (FControlType=iacctSelect) then  begin
      if assigned (Parent) and Parent.HandleAllocated then  begin
        for I := 0 to Parent.ControlCount-1 do  begin
          Sibling := Parent.Controls[I];
          if (Sibling is TTntCheckBoxMLines) and
             ((Sibling as TTntCheckBoxMLines) <> self) and
             ((Sibling as TTntCheckBoxMLines).FGroupIndex=FGroupIndex) and
             ((Sibling as TTntCheckBoxMLines).FControlType=iacctSelect) then
             (Sibling as TTntCheckBoxMLines).Checked := False;
        end;
      end;
    end;
    SetProperlyRGN;
    Invalidate;
  end;
end {SetControlType};

procedure TTntCheckBoxMLines.SetEnabled (Value: Boolean);
begin
  inherited Enabled := Value;
  SetProperlyRGN;
  Invalidate;
end {SetEnabled};

procedure TTntCheckBoxMLines.SetFont (Value: TFont);
begin
  inherited Font := Value;
  SetProperlyRGN;
end;

procedure TTntCheckBoxMLines.SetGapTextToImage (Value: Byte);
begin
  if  Value <> FGapTextToImage  then  begin
    FGapTextToImage := Value;
    SetProperlyRGN;
    Invalidate;
  end;
end {SetGapTextToImage};

procedure TTntCheckBoxMLines.SetGroupIndex (Value: Cardinal);
begin
  if Value <> FGroupIndex then  begin
    FGroupIndex := Value;
    SetProperlyRGN;
    Invalidate;
  end;
end {SetGroupIndex};

procedure TTntCheckBoxMLines.SetHeight (Value: Integer);
begin
  inherited Height := Value;
  SetProperlyRGN;
end;

procedure TTntCheckBoxMLines.SetOnChangeState (Value: TIACheckControlChangeStateProcedure);
begin
  FOnChangeState := Value;
end;

procedure TTntCheckBoxMLines.SetPaintAllBackground (Value: Boolean);
begin
  if Value <> FPaintAllBackground then  begin
    FPaintAllBackground := Value;
    SetProperlyRGN;
    Invalidate;
  end;
end {SetPaintAllBackground};

procedure TTntCheckBoxMLines.SetProperlyRGN;
var
  s1, s2, s3, s4, s5, fs: TSize;
  RGN1, RGN2: HRGN;
  P1: array [0..3] of TPoint;
begin
  if  HandleAllocated and not FPaintAllBackground  then  begin
    if FTransparent then  begin
      s1 := GetBasis;
      s2 := GetOffset;
      s5 := TextSize;
      s3 := GetBmpBasis;
      s4 := GetBmpSize;
      fs := FullSize;
      case Alignment2 of
      taLeftJustify :
        begin
        end;
      taRightJustify :
        begin
          Inc (s1.cx, Width-fs.cx);
          Inc (s3.cx, Width-fs.cx);
        end;
      taCenter :
        begin
          Inc (s1.cx, (Width-fs.cx) div 2);
          Inc (s3.cx, (Width-fs.cx) div 2);
        end;
      end;
      RGN1 := CreateRectRGN (s3.Cx, s3.Cy, s3.Cx+s4.Cx, s3.Cy+s4.Cy);
      p1[0].x := s1.Cx+s2.Cx;
      p1[0].y := s1.Cy+s2.Cy;
      p1[1].x := s1.Cx+s2.Cx+s5.Cx;
      p1[1].y := s1.Cy+s2.Cy;
      p1[2].x := p1[1].x;
      p1[2].y := p1[1].y+s5.Cy;
      p1[3].x := s1.Cx+s2.Cx;
      p1[3].y := s1.Cy+s2.Cy+s5.Cy;
      RGN2 := CreatePolygonRGN (P1, 4, Alternate);
      CombineRGN (RGN1, RGN1, RGN2, RGN_OR);
      DeleteObject (RGN2);
      SetWindowRgn (Handle, RGN1, True);
    end
    else  begin
      SetWindowRgn (Handle, 0, True);
    end;
  end;
end {SetProperlyRGN};

procedure TTntCheckBoxMLines.SetShowFocus (Value: Boolean);
begin
  if  Value <> FShowFocus  then  begin
    FShowFocus := Value;
    Invalidate;
  end;
end {SetShowFocus};

procedure TTntCheckBoxMLines.SetAlignment (Value: TAlignment);
begin
  if  Value <> FAlignment  then  begin
    FAlignment := Value;
    SetProperlyRGN;
    Invalidate;
  end;
end {SetAlignment};

procedure TTntCheckBoxMLines.SetCheckVerticalAlign (Value: TIACheckControlVAlign);
begin
  if  Value <> FCheckVerticalAlign  then  begin
    FCheckVerticalAlign := Value;
    SetProperlyRGN;
    Invalidate;
  end;
end {SetCheckVerticalAlign};

procedure TTntCheckBoxMLines.SetTransparent (Value: Boolean);
begin
  if  Value <> FTransparent  then  begin
    FTransparent := Value;
    SetProperlyRGN;
    Invalidate;
  end;
end {SetTransparent};

procedure TTntCheckBoxMLines.SetVisible (Value: Boolean);
begin
  inherited Visible := Value;
  Invalidate;
end;

procedure TTntCheckBoxMLines.SetWidth (Value: Integer);
begin
  inherited Width := Value;
  SetProperlyRGN;
end;

procedure TTntCheckBoxMLines.WMKeyUp (var Message: TWMKeyUp);
begin
  inherited;
  if Enabled and (Message.CharCode=32) then  begin
    case FControlType of
      iacctCheck :Checked := not Checked;
      iacctSelect: Checked := True;
    end;
  end;
end {WMKeyUp};

procedure TTntCheckBoxMLines.WMKILLFOCUS (var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TTntCheckBoxMLines.WMLButtonDblClk (var Message: TWMLButtonDblClk);
begin
  perform (WM_LButtonDown, 0, 0);
end;

procedure TTntCheckBoxMLines.WMLButtonDown (var Message: TWMLButtonDown);
begin
  inherited;
  if Enabled then  begin
    case FControlType of
      iacctCheck: Checked := not Checked;
      iacctSelect: Checked := True;
    end;
    self.SetFocus;
  end;
end {WMLButtonDown};

procedure TTntCheckBoxMLines.WMSETFOCUS (var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TTntCheckBoxMLines.WMSIZE (var Message: TMessage);
begin
  inherited;
  SetProperlyRGN;
end;

{-PROTECTED--------}

procedure TTntCheckBoxMLines.DoDrawText (var Rect: TRect; Flags: Longint);
var
  S: WideString;
const
  FShowAccelChar = True;
begin
  S := Caption;
  if FAutomaticWrap
  then  S := WStrReplaceSimple (S, sLineBreak, ' ');
  if (Flags and DT_CALCRECT <> 0) and ((S = '') or FShowAccelChar and
    (S[1] = '&') and (S[2] = #0))
  then S := S + ' ';
  if not FShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Flags := DrawTextBiDiModeFlags (Flags);
  Canvas.Font := Font;
  if not Enabled then
  begin
    OffsetRect (Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawTextW (Canvas.Handle, PWideChar(S), Length(S), Rect, Flags);
    OffsetRect (Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawTextW (Canvas.Handle, PWideChar(S), Length(S), Rect, Flags);
  end
  else
    DrawTextW (Canvas.Handle, PWideChar(S), Length(S), Rect, Flags);
end {DoDrawText};

procedure TTntCheckBoxMLines.DrawCheckBMP;
var
  s1: TSize;
  Bm: TBitmap;
  S: string;
begin
  s1 := GetBMPBasis;
  Bm := TBitmap.Create;
  if Enabled then  begin
    if Checked
    then  S := 'CHECKENAB'
    else  S := 'UNCHECKENAB';
  end
  else  begin
    if Checked
    then  S := 'CHECKDIS'
    else  S := 'UNCHECKDIS';
  end;
  if FControlType = iacctSelect
  then  S := Concat ('R', S);
  try
    Bm.LoadFromResourceName (HInstance, S);
    Bm.Transparent := True;
    case Alignment2 of
    taLeftJustify :
      begin
      end;
    taRightJustify :
      begin
        s1.cx := Width-GetBMPSize.cx;
      end;
    taCenter :
      begin
        s1.cx := (Width-GetBMPSize.cx) div 2;
      end;
    end;
    Canvas.Draw (s1.Cx, s1.Cy, Bm);
  finally
    Bm.Free;
  end;
end {DrawCheckBMP};

function TTntCheckBoxMLines.GetBMPSize: TSize;
begin
  Result.Cx := 15;
  Result.Cy := 15;
end {GetBMPSize};

procedure TTntCheckBoxMLines.AdjustSize;
begin
  //NoAction;
end;

procedure TTntCheckBoxMLines.Paint;
var
  s1, s2, ABase, AOffset: TSize;
  R: TRect;
  DrawStyle: Longint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
begin
  if Visible or (csDesigning in ComponentState) then  begin
    s1 := FullSize;
    s2 := TextSize;
    Canvas.Brush.Color := Color;
    Canvas.FillRect (Rect (0, 0, Width, Height));
    ABase := GetBasis;
    AOffset := GetOffset;
    Canvas.Brush.Style := bsClear;
    R := ClientRect;
    R.Left := ABase.Cx + AOffset.Cx + 1;
    R.Top := ABase.Cy + AOffset.Cy + 1;
    R.Right := R.Left + s2.cx - 2;
    R.Bottom := R.Top + s2.cy - 2;
    case Alignment2 of
    taLeftJustify :
      begin
      end;
    taRightJustify :
      begin
        R.Left := R.Left + (Width-s1.cx);
        R.Right := R.Right + (Width-s1.cx);
      end;
    taCenter :
      begin
        R.Left := R.Left + ((Width-s1.cx) div 2);
        R.Right := R.Right + ((Width-s1.cx) div 2);
      end;
    end;
    DrawStyle := DT_EXPANDTABS or WordWraps[FAutomaticWrap] or Alignments[FAlignment];
    DoDrawText (R, DrawStyle);
    Canvas.Brush.Style := bsSolid;
    if FShowFocus and Focused and Enabled
    then  DrawFocus;
    DrawCheckBMP;
  end;
end {Paint};

procedure TTntCheckBoxMLines.SetText(const Value: TWideCaption);
begin
  inherited;
  SetProperlyRGN;
  Invalidate;
end {SetText};

{-PUBLIC-----------}

procedure TTntCheckBoxMLines.Assign (Source: TPersistent);
begin
  if Source is TTntCheckBoxMLines then  begin
    SetBounds (Left, Top, (Source as TTntCheckBoxMLines).Width, (Source as TTntCheckBoxMLines).Height);
    inherited Color := (Source as TTntCheckBoxMLines).Color;
    inherited Font.Assign ((Source as TTntCheckBoxMLines).Font);
    inherited Text := (Source as TTntCheckBoxMLines).Text;
    FControlType := (Source as TTntCheckBoxMLines).FControlType;
    FGroupIndex := (Source as TTntCheckBoxMLines).FGroupIndex;
    Checked := (Source as TTntCheckBoxMLines).FChecked;
    FAlignment := (Source as TTntCheckBoxMLines).FAlignment;
    FCheckVerticalAlign := (Source as TTntCheckBoxMLines).FCheckVerticalAlign;
    FGapTextToImage := (Source as TTntCheckBoxMLines).FGapTextToImage;
    FTransparent := (Source as TTntCheckBoxMLines).FTransparent;
    TabStop := (Source as TTntCheckBoxMLines).TabStop;
    FShowFocus := (Source as TTntCheckBoxMLines).FShowFocus;
    FAutomaticWrap := (Source as TTntCheckBoxMLines).AutomaticWrap;
    FPaintAllBackground := (Source as TTntCheckBoxMLines).PaintAllBackground;
  end else
  begin
    inherited Assign (Source);
  end;
end {Assign};

constructor TTntCheckBoxMLines.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  if  csDesigning in ComponentState
  then  FCaptionEditor := TUnicodeLinesEditor.Create;
  ControlStyle := ControlStyle - [csOpaque];
  inherited SetBounds (Left, Top, 80, 20);
  inherited Font.OnChange := FontIsChange;
  Caption := 'CheckText';
  FAlignment := taLeftJustify;
  FControlType := iacctCheck;
  FGroupIndex := 0;
  FCheckVerticalAlign := iaccvaTop;
  FGapTextToImage := 4;
  FTransparent := True;
  TabStop := True;
  FOnChangeState := nil;
  inherited AutoSize := False;
end {Create};

destructor TTntCheckBoxMLines.Destroy;
begin
  if  csDesigning in ComponentState
  then  FCaptionEditor.Free;
  inherited;
end {Destroy};

function TTntCheckBoxMLines.GetControlsAlignment: TAlignment;
begin
  if not UseRightToLeftAlignment then
    Result := taLeftJustify
  else
    if FAlignment = taLeftJustify then
      Result := taRightJustify
    else
      Result := taLeftJustify;
end;

procedure TTntCheckBoxMLines.SetBounds (ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds (ALeft, ATop, AWidth, AHeight);
end;

{--------------------------------------}
{-PRIVATE----------}

procedure TTntRadioButtonMLines.SetAlignment(Value: TLeftRight);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

function TTntRadioButtonMLines.GetCaptionEditor: TUnicodeLinesEditor;
begin
  Result := FCaptionEditor;
  FCaptionEditor.SetText (Caption);
end;

procedure TTntRadioButtonMLines.SetCaptionEditor(
  const Value: TUnicodeLinesEditor);
begin
  FCaptionEditor.Assign (Value);
  Caption := FCaptionEditor.GetText;
end;

procedure TTntRadioButtonMLines.DrawCheckBMP;
var
  s1: TSize;
  Bm: TBitmap;
  S: string;
begin
  s1 := GetBMPBasis;
  Bm := TBitmap.Create;
  if Checked
  then  S := 'RCHECKENAB'
  else  S := 'RUNCHECKENAB';
  try
    Bm.LoadFromResourceName (HInstance, S);
    Bm.Transparent := True;
    Canvas.Draw (s1.Cx, s1.Cy, Bm);
  finally
    Bm.Free;
  end;
end {DrawCheckBMP};

procedure TTntRadioButtonMLines.FontIsChange (Sender: TObject);
begin
  Invalidate;
end;

function TTntRadioButtonMLines.GetBasis: TSize;
var
  s1: TSize;
begin
  s1 := GetBmpSize;
  Result.Cx := s1.Cx + FGapTextToImage;
  Result.Cy := 0;
end {GetBasis};

function TTntRadioButtonMLines.GetBMPBasis: TSize;
var
  s1, s2, s3: TSize;
begin
  s1 := GetBmpSize;
  s2 := TextSize;
  s3.Cx := Abs (s2.Cx);
  s3.Cy := Abs (s2.Cy);
  Result.Cx := 0;
  Result.Cy := Max (0, -(s1.Cy-s3.Cy) div 2);
end {GetBMPBasis};

function TTntRadioButtonMLines.GetBMPSize: TSize;
begin
  Result.Cx := 15;
  Result.Cy := 15;
end {GetBMPSize};

function TTntRadioButtonMLines.GetFullSize: TSize;
var
  s1, s2, s3: TSize;
begin
  s1 := GetBmpSize;
  s2 := TextSize;
  s3.Cx := Abs (s2.Cx);
  s3.Cy := Abs (s2.Cy);
  Result.Cx := s1.Cx + s3.Cx + FGapTextToImage;
  Result.Cy := Max (s1.Cy, s3.Cy);
end {GetFullSize};

function TTntRadioButtonMLines.GetOffset: TSize;
var
  s2, s3: TSize;
begin
  s2 := TextSize;
  s3.Cx := Abs (s2.Cx);
  s3.Cy := Abs (s2.Cy);
  Result.Cx := s3.Cx-s2.Cx;
  Result.Cy := 0;
end {GetOffset};

function TTntRadioButtonMLines.GetTextSize: TSize;
var
  S2: TSize;
begin
  Result.Cx := Width;
  Result.Cy := Height;
  S2 := GetBmpSize;
  Result.Cx := Width - (S2.Cx + FGapTextToImage);
end {GetTextSize};

procedure TTntRadioButtonMLines.WMLButtonDown (var Message: TWMLButtonDown);
begin
  inherited;
  Checked := True;
end;

procedure TTntRadioButtonMLines.SetChecked (Value: Boolean);
var
  I: Integer;
  Sibling: TControl;
begin
  if  Value <> FChecked  then  begin
    FChecked := Value;
    if FChecked then  begin
      if Assigned (Parent) and Parent.HandleAllocated then  begin
        for  I  := 0 to Parent.ControlCount-1  do  begin
          Sibling := Parent.Controls[I];
          if (Sibling is TTntRadioButtonMLines) and
             ((Sibling as TTntRadioButtonMLines) <> self)
          then
             (Sibling as TTntRadioButtonMLines).Checked := False;
        end;
      end;
    end;
    //if assigned (FOnChangeState) then FOnChangeState (self, FChecked);
    Invalidate;
  end;
end {SetChecked};

procedure TTntRadioButtonMLines.SetGapTextToImage (Value: Byte);
begin
  if  Value <> FGapTextToImage  then  begin
    FGapTextToImage := Value;
    Invalidate;
  end;
end;

{-PROTECTED--------}

procedure TTntRadioButtonMLines.CreateParams(var Params: TCreateParams);
const
  Alignments: array[Boolean, TLeftRight] of DWORD =
    ((BS_LEFTTEXT, 0), (0, BS_LEFTTEXT));
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'BUTTON');
  with Params do
    Style := Style or BS_RADIOBUTTON or
      Alignments[UseRightToLeftAlignment, FAlignment];
end;

procedure TTntRadioButtonMLines.DoDrawText (var Rect: TRect;  Flags: Longint);
var
  S: WideString;
const
  FShowAccelChar = True;
begin
  S := Caption;
  if (Flags and DT_CALCRECT <> 0) and ((S = '') or FShowAccelChar and
    (S[1] = '&') and (S[2] = #0))
  then S := S + ' ';
  if not FShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Flags := DrawTextBiDiModeFlags (Flags);
  Canvas.Font := Font;
  if not Enabled then
  begin
    OffsetRect (Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawTextW (Canvas.Handle, PWideChar(S), Length(S), Rect, Flags);
    OffsetRect (Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawTextW (Canvas.Handle, PWideChar(S), Length(S), Rect, Flags);
  end
  else
    DrawTextW (Canvas.Handle, PWideChar(S), Length(S), Rect, Flags);
end {DoDrawText};

procedure TTntRadioButtonMLines.Paint;
var
  s1, s2, ABase, AOffset: TSize;
  R: TRect;
begin
  if Visible or (csDesigning in ComponentState) then  begin
    s1 := FullSize;
    s2 := TextSize;
    Canvas.Brush.Color := Color;
    Canvas.FillRect (Rect (0, 0, Width, Height));
    ABase := GetBasis;
    AOffset := GetOffset;
    Canvas.Brush.Style := bsClear;
    R := ClientRect;
    R.Left := ABase.Cx + AOffset.Cx + 1;
    R.Top := ABase.Cy + AOffset.Cy + 1;
    R.Right := R.Left + s2.cx - 2;
    R.Bottom := R.Top + s2.cy - 2;
    DoDrawText (R, DT_EXPANDTABS or DT_WORDBREAK or DT_LEFT);
    Canvas.Brush.Style := bsSolid;
    DrawCheckBMP;
  end;
end {Paint};

procedure TTntRadioButtonMLines.SetText(const Value: TWideCaption);
begin
  inherited;
  Invalidate;
end;

{-PUBLIC-----------}

procedure TTntRadioButtonMLines.Assign (Source: TPersistent);
begin
  if Source is TTntRadioButtonMLines then  begin
    SetBounds (Left, Top, (Source as TTntRadioButtonMLines).Width, (Source as TTntRadioButtonMLines).Height);
    inherited Font.Assign ((Source as TTntRadioButtonMLines).Font);
    inherited Text := (Source as TTntRadioButtonMLines).Text;
    FAlignment := (Source as TTntRadioButtonMLines).FAlignment;
    Checked := (Source as TTntRadioButtonMLines).FChecked;
    FGapTextToImage := (Source as TTntRadioButtonMLines).FGapTextToImage;
  end
  else begin
    inherited Assign (Source);
  end;
end;

constructor TTntRadioButtonMLines.Create (AOwner: TComponent);
begin
  inherited;
  if  csDesigning in ComponentState
  then  FCaptionEditor := TUnicodeLinesEditor.Create;
  inherited SetBounds (Left, Top, 80, 20);
  inherited Font.OnChange := FontIsChange;
  Caption := 'ButtonText';
  FAlignment := taRightJustify;
  FGapTextToImage := 4;
  ControlStyle := ControlStyle - [csOpaque];
end;

destructor TTntRadioButtonMLines.Destroy;
begin
  if  csDesigning in ComponentState
  then  FCaptionEditor.Free;
  inherited;
end;

{--------------------------------------}

end.
