{---------------------------------------------------------------------

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 05/19/2006 version

----------------------------------------------------------------------}

unit TntInst0;

interface

uses
  Windows;

resourcestring
  RS_TntInst0_ErrorProgramNotFound = 'Program not found: ';
  RS_TntInst0_ErrorLinkPathExtensionMustBeLNK = 'Link path extension must be .LNK';
  RS_TntInst0_LinkNotCreated = 'Link could not create'#13#10'(%s)'#13#10#13#10+
   'It must be created with Windows Explorer.'#13#10'Remember: the program (%s) is in'#13#10'%s';

const
  ExtLnk = '.lnk';

type
  TTntInstApp0 = class (TObject)
  protected
    procedure WhenMkLnkIncorrectExt; virtual; abstract;
    procedure WhenMkLnkNotCreated (const PgmPath, LinkPath: WideString); virtual; abstract;
    procedure WhenPgmToLinkToNotExists (const PgmToLinkTo: WideString); virtual; abstract;
  public
    function  GetSpecialFolder (aFolder: Integer;  var Location: WideString): LongWord;
    function  MakeAssociation (Ext: WideString;
      const PgmToLinkTo: WideString): Boolean;
    function  MakeLink (const PgmPath, PgmArgs, LinkPath, Descr: WideString): Boolean;
    function  RegDeleteKey (RKey: HKey;  const KeyPath: WideString): Boolean;
    function  RemoveAssociation (Ext: WideString): Boolean;
    function  SetValInReg (RKey: HKey;
      const KeyPath, ValName, NewVal: WideString): Boolean;
  end;

implementation

uses
  Forms,
  Controls, ShlObj, ComObj, ActiveX, SysUtils,
  TntRegistry, TntSysUtils;

//---------------------------------------------------------------------------

function CreateShellLinkW (const LinkFileName, AssocFileName, Desc,
  WorkDir, Args, IconFileName: WideString; const IconIdx: Integer): Boolean;
var
  SL: IShellLinkW;
  PF: IPersistFile;
begin
  {Modified from http://www.latiumsoftware.com/es/pascal/0052.php}
  Result := False;
  CoInitialize(nil);
  try
    if Succeeded(
      CoCreateInstance(
        CLSID_ShellLink, nil, CLSCTX_INPROC_SERVER, IShellLinkW, SL
      )
    ) then
    begin
      SL.SetPath(PWideChar(AssocFileName));
      SL.SetDescription(PWideChar(Desc));
      SL.SetWorkingDirectory(PWideChar(WorkDir));
      SL.SetArguments(PWideChar(Args));
      if (IconFileName <> '') and (IconIdx >= 0) then
        SL.SetIconLocation(PWideChar(IconFileName), IconIdx);
      PF := SL as IPersistFile;
      Result := Succeeded(
        PF.Save(PWideChar(LinkFileName), True)
      );
    end;
  finally
    CoUninitialize;
  end;
end {CreateShellLinkW};

//---------------------------------------------------------------------------
{* Folder types are a.o.
 *	CSIDL_DESKTOPDIRECTORY, CSIDL_STARTMENU, CSIDL_SENDTO,
 * CSIDL_PROGRAMS, CSIDL_STARTUP etc.
 *}
function TTntInstApp0.GetSpecialFolder (aFolder: Integer;  var Location: WideString): LongWord;
var
  pidl:      PItemIDList;
  hRes:      HRESULT;
  RealPath:  array[0..MAX_PATH] of WideChar;
  Success:   Boolean;
begin
  Result := 0;
  hRes   := SHGetSpecialFolderLocation (Application.Handle, aFolder, pidl);
  if  hRes = NO_ERROR  then
  begin
    Success := SHGetPathFromIDListW (pidl, RealPath);
    if Success then
       Location := WideString (RealPath) + '\'
    else
       Result := LongWord (E_UNEXPECTED);
    GlobalFreePtr (pidl);
  end else
    Result := hRes;
end {GetSpecialFolder};

function TTntInstApp0.MakeAssociation (Ext: WideString;
  const PgmToLinkTo: WideString): Boolean;
begin
  { ALL extensions must be in lowercase to avoid trouble! }
  Ext := WideLowerCase (Ext);
  Result := False;
  if  WideFileExists (PgmToLinkTo)  then  begin
    SetValInReg (HKEY_CLASSES_ROOT,
           '.' + Ext,            { extension we want to define }
           '',                   { specify the default data item }
           Ext + '_auto_file'); { This is the value of the default data item -
                                   this referances our new type to be defined  }
    SetValInReg (HKEY_CLASSES_ROOT,
          Ext + '_auto_file',    { this is the type we want to define }
          '',                    { specify the default data item }
          Ext + ' Files');       { This is the value of the default data item -
                                   this is the English description of the file type }

    SetValInReg (HKEY_CLASSES_ROOT,
          Ext + '_auto_file\DefaultIcon', { Create a file...DefaultIcon.}
          '',	                            { Specify the default data item.}
          PgmToLinkTo + ',0');            { Executable where icon is in and it's Sequence number.}
    SHChangeNotify (SHCNE_ASSOCCHANGED, SHCNF_FLUSH, nil, nil);

// un-comment this if your file type can be viewed by Quick View
//    SetValInReg (HKEY_CLASSES_ROOT,
//          Ext + '_auto_file\QuickView', { create a key for QuickView compat. }
//          '',                    { specify the default data item }
//          '*');                 { flag to tell Explorer that QuickView is OK }

    SetValInReg (HKEY_CLASSES_ROOT,
          Ext + '_auto_file\shell\open\command', { create a file...open key }
          '',                    { specify the default data item }
          PgmToLinkTo + ' %1'); { command line to open file with }
    Result := True;
  end
  else
    WhenPgmToLinkToNotExists (RS_TntInst0_ErrorProgramNotFound + PgmToLinkTo); {New}
end {MakeAssociation};

{* Make a Shell Link, also called a "shortcut".
 * MakeLink - uses the shell's IShellLinkW and IPersistFile interfaces
 * to create and store a shortcut to the specified object.
 *
 * PgmPath  - address of a buffer containing the path of the object.
 * LinkPath - address of a buffer containing the path where the shell link is to be stored.
 * Descr    - address of a buffer containing the description of the shell link.
 * PgmArgs  - address of a buffer containing the arguments for the shell link.
 *}
function TTntInstApp0.MakeLink (const PgmPath, PgmArgs, LinkPath, Descr: WideString): Boolean;
begin
  Result := False;
  if  WideLowerCase (WideExtractFileExt (LinkPath)) <> ExtLnk  then
  begin
    WhenMkLnkIncorrectExt; {New}
    Exit;
  end;
  if  not
    CreateShellLinkW (LinkPath, PgmPath, Descr,
      WideExtractFilePath(PgmPath), '', '', -1)
  then
    WhenMkLnkNotCreated (PgmPath, LinkPath) {New}
  else
    Result := True;  
end {MakeLink};

function TTntInstApp0.RegDeleteKey (RKey: HKey;  const KeyPath: WideString): Boolean;
begin
  with TTntRegistry.Create do
  try
    RootKey := RKey;
    // Under Win95, all keys under this one are auto. deleted also.
    // But, under WinNT, the keys under this one will be left alone.
    DeleteKey (KeyPath);
    Result := True;
  finally
    Free;
  end;
end {RegDeleteKey};

function TTntInstApp0.RemoveAssociation (Ext: WideString): Boolean;
begin
  Ext := WideLowerCase (Ext);
  try
    RegDeleteKey (HKEY_CLASSES_ROOT,
                 '.' + Ext);     { extension we want to undefine }
    RegDeleteKey (HKEY_CLASSES_ROOT,
                 Ext + '_auto_file\DefaultIcon');
    RegDeleteKey (HKEY_CLASSES_ROOT,
                 Ext + '_auto_file\shell\open\command');
    RegDeleteKey (HKEY_CLASSES_ROOT,
                 Ext + '_auto_file');
    SHChangeNotify (SHCNE_ASSOCCHANGED, SHCNF_FLUSH, nil, nil);
    Result := True;
  finally
  end;
end {RemoveAssociation};

{ Set a value in the registry. This is NOT related to the .LNK code.
  This will create a new registry key if it doesn't already exist. }
function TTntInstApp0.SetValInReg (RKey: HKey;
  const KeyPath, ValName, NewVal: WideString): Boolean;
begin
  with TTntRegistry.Create do
  try
    RootKey := RKey;
    OpenKey (KeyPath, True);
    WriteString (ValName, NewVal);
    Result := True;
  finally
    Free;
  end;
end {SetValInReg};

end.
