{---------------------------------------------------------------------

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/01/2006 version
 05/08/2006 version:
   Read/Write Strings are renamed to Read/Write StringsW
   adds Read/Write Strings for TStrings

----------------------------------------------------------------------}

unit TntMabSave;

interface

uses
   SysUtils, WinTypes, WinProcs, Classes, TntClasses, IniFiles, TntIniFiles;

{$I Help.inc}

const
 {$IFDEF RtfHelp}
  ExtHelp = '.hlp';
 {$ELSE}
  ExtHelp = '.chm';
 {$ENDIF}
  ExtIni0 = 'ini';
  ExtIni = '.' + ExtIni0;
  IdentCount = 'Count';

type
  TTntNewIniFile = class(TTntMemIniFile)
  public
    function  ReadChar (const Section, Ident: string;  DefValue: Char): Char;
    procedure WriteChar (const Section, Ident: string;  Value: Char);
    function  ReadWideChar (const Section, Ident: string;
      DefValue: WideChar): WideChar;
    procedure WriteWideChar (const Section, Ident: string;  Value: WideChar);
    function  ReadIntegerLims (const Section, Ident: string;
      Default, LimInf, LimSup: Longint): Longint;
    procedure ReadStrings (const Section, Ident: string;
      Strings: TStrings);
    procedure WriteStrings (const Section, Ident: string;
      Strings: TStrings);
    procedure ReadStringsW (const Section, Ident: string;
      Strings: TTntStrings);
    procedure WriteStringsW (const Section, Ident: string;
      Strings: TTntStrings);
    procedure UpdateFile; override;
  end;

  TTntHlpIniFLoadSave = class(TComponent)
  private
    FApplicationPath: WideString;
    FApplicationName: WideString;
    FIniFileName: WideString;
  protected
    procedure LoadSpecified; virtual;
    procedure SaveSpecified; virtual;
  public
    IniFile: TTntNewIniFile;
    constructor Create (AOwner: TComponent); override;
    procedure ApplicationHelpFileInSubDir (const SubDir: WideString);
    procedure CreateIniFile;
    procedure FreeIniFile;
    procedure Load;
    procedure Save;
    procedure SetDefaultIniFileName;
    {if is necessari load and save in another ini file, uses:}
    procedure SetIniFileName (const Name: WideString);
    property  ApplicationName: WideString read FApplicationName;
    property  ApplicationPath: WideString read FApplicationPath;
    property  IniFileName: WideString read FIniFileName;
  end;

implementation

uses
  Forms,
  GrTypFun, TntFileUtil2,
  StrUtil00, TntStrUtil0;

{--------------------------------------}
{-PUBLIC-----------}

function TTntNewIniFile.ReadChar (const Section, Ident: string;
  DefValue: Char): Char;
var
  S: Str1;
begin
  S := ReadString (Section, Ident, CharToStr1(DefValue));
  Result := S[1];
end {ReadChar};

procedure TTntNewIniFile.WriteChar (const Section, Ident: string;  Value: Char);
var
  S: Str1;
begin
  S := CharToStr1 (Value);
  WriteString (Section, Ident, S);
end {WriteChar};

function TTntNewIniFile.ReadWideChar (const Section, Ident: string;
  DefValue: WideChar): WideChar;
var
  S: WideString;
begin
  S := ReadString (Section, Ident, WCharToStr1(DefValue));
  Result := S[1];
end {ReadWideChar};

procedure TTntNewIniFile.WriteWideChar (const Section, Ident: string;
  Value: WideChar);
var
  S: Str1;
begin
  S := WCharToStr1 (Value);
  WriteString (Section, Ident, S);
end {WriteWideChar};

function TTntNewIniFile.ReadIntegerLims (const Section, Ident: string;
  Default, LimInf, LimSup: Integer): Longint;
begin
  Result := ReadInteger (Section, Ident, Default);
  if  Result < LimInf
  then  Result := LimInf
  else if  Result > LimSup
  then  Result := LimSup;
end {ReadIntegerLims};

procedure TTntNewIniFile.ReadStrings (const Section, Ident: string;
  Strings: TStrings);
var
  I: Integer;
  S: string;
begin
  Strings.Clear;
  I := ReadInteger (Section, Concat (Ident, IdentCount), 0);
  for  I := 0 to I-1  do  begin
    S := ReadString (Section, Concat (Ident, IntToStr(I)), '');
    Strings.Add (S);
  end;
end {ReadStrings};

procedure TTntNewIniFile.WriteStrings (const Section, Ident: string;
  Strings: TStrings);
var
  I: Integer;
begin
  I := ReadInteger (Section, Concat (Ident, IdentCount), 0);
  for  I := 0 to I-1  do
    DeleteKey (Section, Concat (Ident, IntToStr(I)));
  I := Strings.Count;
  WriteInteger (Section, Concat (Ident, IdentCount), I);
  for  I := 0 to I-1  do
    WriteString (Section, Concat (Ident, IntToStr(I)), Strings[I]);
end {WriteStrings};

procedure TTntNewIniFile.ReadStringsW (const Section, Ident: string;
  Strings: TTntStrings);
var
  I: Integer;
  S: WideString;
begin
  Strings.Clear;
  I := ReadInteger (Section, Concat (Ident, IdentCount), 0);
  for  I := 0 to I-1  do  begin
    S := ReadString (Section, Concat (Ident, IntToStr(I)), '');
    Strings.Add (S);
  end;
end {ReadStringsW};

procedure TTntNewIniFile.WriteStringsW (const Section, Ident: string;
  Strings: TTntStrings);
var
  I: Integer;
begin
  I := ReadInteger (Section, Concat (Ident, IdentCount), 0);
  for  I := 0 to I-1  do
    DeleteKey (Section, Concat (Ident, IntToStr(I)));
  I := Strings.Count;
  WriteInteger (Section, Concat (Ident, IdentCount), I);
  for  I := 0 to I-1  do
    WriteString (Section, Concat (Ident, IntToStr(I)), Strings[I]);
end {WriteStringsW};

procedure TTntNewIniFile.UpdateFile;
begin
  if  FileExistsSetNotReadOnlyMessW (FileName)
  then  inherited;
end {UpdateFile};

{--------------------------------------}
{-PUBLIC-----------}

constructor TTntHlpIniFLoadSave.Create (AOwner: TComponent);
begin
  inherited;
  ApplicationPathAndNameW (FApplicationPath, FApplicationName);
  SetDefaultIniFileName;
  ApplicationHelpFileInSubDir ('');
end {Create};

procedure TTntHlpIniFLoadSave.ApplicationHelpFileInSubDir
  (const SubDir: WideString);
begin
  Application.HelpFile := Concat (FApplicationPath, SubDir, FApplicationName, ExtHelp);
end;

procedure TTntHlpIniFLoadSave.CreateIniFile;
begin
  IniFile := TTntNewIniFile.Create(FIniFileName);
end;

procedure TTntHlpIniFLoadSave.FreeIniFile;
begin
  IniFile.Free;
end;

procedure TTntHlpIniFLoadSave.LoadSpecified;
begin
  {nothing}
end;

procedure TTntHlpIniFLoadSave.SaveSpecified;
begin
  {nothing}
end;

procedure TTntHlpIniFLoadSave.Load;
begin
  CreateIniFile;
  LoadSpecified;
  FreeIniFile;
end {Load};

procedure TTntHlpIniFLoadSave.Save;
begin
  CreateIniFile;
  SaveSpecified;
  IniFile.UpdateFile;
  FreeIniFile;
end {Save};

procedure TTntHlpIniFLoadSave.SetDefaultIniFileName;
begin
  FIniFileName := Concat (FApplicationPath, FApplicationName, ExtIni);
end;

procedure TTntHlpIniFLoadSave.SetIniFileName (const Name: WideString);
begin
  FIniFileName := Concat (FApplicationPath, Name, ExtIni);
end;

{------------------}

end.
