{---------------------------------------------------------------------

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/01/2006 version

 Portions created by Borland Software Corporation are
 Copyright (c) 1995-2005 Borland Software Corporation

----------------------------------------------------------------------}

unit TntMessUsual; {Usual Messages}

interface

uses
  Classes, Windows, Messages, Dialogs;

procedure WideMessageBoxMDT (const Msg: WideString;  MDT: TMsgDlgType);
procedure WideMessageBoxWarning (const Msg: WideString);
procedure WideMessageBoxError (const Msg: WideString);
procedure WideMessageBoxInfo (const Msg: WideString);
function  WideMessageCompletPrompt (const Msg: WideString): Integer;
function  WideUserSayYes (const Msg: WideString): Boolean;
function  WideUserSayYesWarning (const Msg: WideString): Boolean;

{ Modified from ExtCtrls.TTimer }

type
  TTimerH = class(TComponent)
  private
    FInterval: Cardinal;
    FWindowHandle: HWND;
    FEnabled: Boolean;
    procedure UpdateTimer;
    procedure SetEnabled(Value: Boolean);
    procedure SetInterval(Value: Cardinal);
    procedure WndProc(var Msg: TMessage);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Interval: Cardinal read FInterval write SetInterval default 1000;
  end;

var
  TimeForChar: Cardinal = 70;
procedure WideMessageBoxMDTTimer (const Msg: WideString;  MDT: TMsgDlgType);
procedure WideMessageBoxWarningTimer (const Msg: WideString);
procedure WideMessageBoxErrorTimer (const Msg: WideString);
procedure WideMessageBoxInfoTimer (const Msg: WideString);
function  WideMessageCompletPromptTimer (const Msg: WideString): Integer;

implementation

uses
  Consts, Controls, TntDialogs, Forms, TntForms;

{--------------------------------------}

procedure WideMessageBoxMDT (const Msg: WideString;  MDT: TMsgDlgType);
begin
  WideMessageDlg (Msg, MDT, [mbOk], 0);
end;

procedure WideMessageBoxWarning (const Msg: WideString);
begin
  WideMessageBoxMDT (Msg, mtWarning);
end;

procedure WideMessageBoxError (const Msg: WideString);
begin
  WideMessageBoxMDT (Msg, mtError);
end;

procedure WideMessageBoxInfo (const Msg: WideString);
begin
  WideMessageBoxMDT (Msg, mtInformation);
end;

function WideMessageCompletPrompt (const Msg: WideString): Integer;
begin
  Result := WideMessageDlg (Msg, mtConfirmation, [mbYes, mbNo, mbCancel], 0);
end;

function WideUserSayYes (const Msg: WideString): Boolean;
begin
  Result := WideMessageDlg (Msg, mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end;

function WideUserSayYesWarning (const Msg: WideString): Boolean;
begin
  Result := WideMessageDlg (Msg, mtWarning, [mbYes, mbNo], 0) = mrYes;
end;

{--------------------------------------}

constructor TTimerH.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FInterval := 1000;
  FWindowHandle := AllocateHWnd(WndProc);
end;

destructor TTimerH.Destroy;
begin
  FEnabled := False;
  UpdateTimer;
  DeallocateHWnd(FWindowHandle);
  inherited Destroy;
end;

procedure TTimerH.WndProc(var Msg: TMessage);
begin
  with Msg do
    if Msg = WM_TIMER then
      try
        TTntForm(Owner).Close;
      except
        Application.HandleException(Self);
      end
    else
      Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
end;

procedure TTimerH.UpdateTimer;
begin
  KillTimer(FWindowHandle, 1);
  if (FInterval <> 0) and FEnabled  then
    if SetTimer(FWindowHandle, 1, FInterval, nil) = 0 then
      raise EOutOfResources.Create(SNoTimers);
end;

procedure TTimerH.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    UpdateTimer;
  end;
end;

procedure TTimerH.SetInterval(Value: Cardinal);
begin
  if Value <> FInterval then
  begin
    FInterval := Value;
    UpdateTimer;
  end;
end;

{------------------}

function WideCreateMessageDialogTimer (const Msg: WideString;  DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons): TTntForm;
var
  Form: TTntForm;
  T: TTimerH;
begin
  Form := WideCreateMessageDialog (Msg, DlgType, Buttons);
  T := TTimerH.Create (Form);
  T.Interval := (Length (Msg) * TimeForChar) + 800;
  T.Enabled := True;
  Result := Form;
end {WideCreateMessageDialogTimer};

function WideMessageDlgPosHelpTimer(const Msg: WideString;  DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons;  HelpCtx: Longint;  X, Y: Integer;
  const HelpFileName: WideString): Integer;
begin
  with WideCreateMessageDialogTimer (Msg, DlgType, Buttons) do
    try
      HelpContext := HelpCtx;
      HelpFile := HelpFileName;
      if X >= 0 then Left := X;
      if Y >= 0 then Top := Y;
      if (Y < 0) and (X < 0) then Position := poScreenCenter;
      Result := ShowModal;
    finally
      Free;
    end;
end {WideMessageDlgPosHelpTimer};

function MessageDlgTimer(const Msg: WideString; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer;
begin
  Result := WideMessageDlgPosHelpTimer(Msg, DlgType, Buttons, HelpCtx, -1, -1, '');
end;

procedure WideMessageBoxMDTTimer (const Msg: WideString;  MDT: TMsgDlgType);
begin
  MessageDlgTimer (Msg, MDT, [mbOk], 0);
end;

procedure WideMessageBoxWarningTimer (const Msg: WideString);
begin
  WideMessageBoxMDTTimer (Msg, mtWarning);
end;

procedure WideMessageBoxErrorTimer (const Msg: WideString);
begin
  WideMessageBoxMDTTimer (Msg, mtError);
end;

procedure WideMessageBoxInfoTimer (const Msg: WideString);
begin
  WideMessageBoxMDTTimer (Msg, mtInformation);
end;

function WideMessageCompletPromptTimer (const Msg: WideString): Integer;
begin
  Result := MessageDlgTimer (Msg, mtConfirmation, [mbYes, mbNo, mbCancel], 0);
end;

{--------------------------------------}

end.
