unit TntdnPxMask;

(*****************************************************************************
Copyright  1997-99 Dieter Nth
version    : 2.1
date       : 10.08.1999
description:

  This unit implements Paradox-style Inputmasks:
  most of the validation-code of Inputmasks is based upon Borland's VALIDATE.PAS
  Unit ( 1992 Borland International) from Borland Pascal 7,
  I don't know why they dropped it from VCL...

Bugs fixed:
1.03 :
  Borland'S masks have a 'feature', which caused errors:
  there may be empty choices in groups (without meaning), e.g. ,,
  or *[&[*?]*[{#, ,.,;,,;;,:,;[,;],(,),}]]
                                      ^ <-- Paradox works fine with this, but my
  PxMask crashed sometimes. I couldn't fix this, so I decided to remove all
  unnecessary commas, when setting the mask :-)
1.02 :
  Borland'S example input mask (from DBD)
  *[&[*?]*[{#, ,.,;,,;;,:,;[,;],(,),}]]  (capitalized words) allowed only
  ONE Word to be entered.
  Yes, I should have tried at least the example masks, but I did it once and
  then I found a bug in Borland's Iteration function causing infinite loops in
  iterations (without a number) containing groups....
  I thought, I fixed it, but this fix caused this new problem...
  There still may be some combinations of groups and iterations that will
  cause an infinite loop. So, if your running your program from within Delphi
  and it hangs while typing in a TPxMaskEdit, set a breakpoint in
  function Iteration and stop your program. If this happens within the
  PxMask-property editor you'll have to kill Delphi ;-).
  Before you send me an email complaining about this, please try to follow
  all those recursive calls with debugger (and remember, that part of code
  wasn't written by Borland, not me), maybe YOU'll fix it. If you don't,
  I'll try my best.

Changes:
2.00:
  Put most of the PxMask-code in a class, so it's easier to maintain the code and
  Create components with a PxMask, BUT now the properties changed:
    PxMask is now an CLASS-property NOT a WideString: AutoFill, AutoValidate,
    RaiseInputError, PxMask, ErrorMessage are now properties of that class.
  So you'll have to change your project-source, sorry for the inconvenience...

1.03:
  Changed strings in english RES-file to be equivalent to those in DBD32:
    SPxMaskIllegalString, "The field Value fails picture validity check"
    SPxMaskIllegalChar, "Illegal character for picture validation"

  added new error WideString, if input WideString is valid, but not yet complete:
    SPxMaskInputIncomplete, "Input OK, but not complete"

  new property ErrorText: WideString;
    You may include the PxMask by inserting '%MASK%', which is replaced with
    the actual PxMask, e.g. 'Input does not confirm to picture: %MASK%'.
    This means, if you want to make this the standard behaviour, you may change
    the default-strings SPxMaskIllegalString/SPxMaskIllegalChar in the RES-file.
    You may also include %ERROR% in a customised ErrorMessage, this includes the
    original ErrorMessage.

1.02 :
  Testing an input mask in DBD32 I noticed, that there's a new special WideChar:
  ---> ~ <--- lower case WideChar
  and, of course, I implemented it.
  If you don't want to use this new feature, you may comment out that part in
  function Scan

******************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls, Forms, Menus,
  Consts, Mask,
  TntStdCtrls,
  TntStdCtrls2,
  TntdnPxCnst, TntdnGlobal;

const
  errPxMaskSyntax = 1;
  errPxMaskIllegalChar = 2;
  errPxMaskInputIncomplete = 3;
  errPxMaskIllegalString = 4;

type
  EPxMaskError = class(EDBEditError)
  public
    ErrorCode : Integer;
  end;

  TPxMaskState = set of ( pxReEnter, pxSending, pxValid, pxModified,
                          pxNoValidation, pxStored, pxMaskFromParadox );

{*** Base class for handling Paradox masks***}
  TTntPxMask = class(TPersistent)
  private
    FMask: WideString;
    FErrMsg: WideString;
    FErrPos : Integer;
    FAutoFill,
    FAutoValidate,
    FRaiseInputError : Boolean;

    procedure SetAutoValidate(Value: Boolean);
    procedure SetAutoFill(Value: Boolean);
    procedure SetErrMsg(const Value: WideString);
    procedure SetErrorPos(Value : Integer);
    procedure SetMask(const Value : WideString);
    procedure SetRaiseInputError(Value: Boolean);

    function MaskStored : Boolean;
    function PropsStored: Boolean;

    function CheckValid(S : WideString): WideString;
    function FormatErrorText(Ident: WideString): WideString;
  protected
    function MaskLoaded : Boolean; virtual;

    procedure ValidChanged; virtual;

    {*** raises EDBEditError, if Prop RaiseInputError = True,
         gets ErrorText via FormatErrorText(vePxMaskIllegalChar)}
    procedure InputError; dynamic;
    {*** raises EPxMaskError}
    procedure ValidateError(Incomplete : Boolean); dynamic;

  public
    FState : TPxMaskState;
    constructor Create(aOwner : TComponent);

    procedure Assign(Source: TPersistent); override;

    {*** Checks Value with func PxMaskValidate}
    procedure Validate(var Value : WideString);

    procedure LoadMask;

    procedure Clear;
    function Cleared: Boolean;

    {*** raises EPxMaskError, if aMask contains syntaxerrors}
    procedure CheckSyntax(var aMask : WideString);

    {*** True if PxMask <> ''}
    function isMasked : Boolean;
    function Valid(const S : WideString): Boolean; {New, used when AutoValidate = False}

    {*** Position of last Error after any validation}
    property ErrorPos : Integer read FErrPos write SetErrorPos;

  published
    {*** Custom error message}
    property ErrorMessage : WideString read FErrMsg
                                       write SetErrMsg
                                       stored PropsStored;

    property Mask : WideString         read FMask
                                       write SetMask
                                       stored MaskStored;

    property AutoValidate : Boolean    read FAutoValidate
                                       write SetAutoValidate
                                       stored PropsStored
                                       default AutoValidateDefault;

    property AutoFill : Boolean        read FAutoFill
                                       write SetAutoFill
                                       stored PropsStored
                                       default AutoFillDefault;

    property RaiseInputError : Boolean read FRaiseInputError
                                       write SetRaiseInputError
                                       stored PropsStored
                                       default RaiseInputErrorDefault;
  end;

  TControlPxMask = class;

  TValidChangedEvent = procedure( Sender: TControlPxMask;
                                  EditControl: TWinControl;
                                  TextValid: Boolean) of object;

  TControlPxMask = class(TTntPxMask)
  private
    FOwner: TComponent;
    FControlWndProc: TWndMethod;
    FOnValidChanged: TValidChangedEvent;
    FOldValue: WideString;
    FEnableReset: Boolean;

    function GetText: WideString;
    procedure SetText(const Value: WideString);
    function GetMaxLen: Integer;
    function GetTextLen: Integer;
    function GetEditControl: TWinControl;
    function GetModified : Boolean;
    procedure CheckEditText;
  protected
    procedure PrepareReset(const OldText : WideString); dynamic;
    procedure Reset; dynamic;
    function CanEditModify: Boolean; dynamic;

    {*** Used internally: TCustomEdit --> Result = Owner.Handle
                          TCustomComboBox --> Result = Owner.EditHandle
                          TCustomGrid --> Result = Owner.InplaceEditor.Handle }
    function GetEditHandle: THandle; virtual;
    procedure SetModified(Value: Boolean); virtual;
    procedure ValidChanged; override;
    procedure ValidateError(Incomplete : Boolean); override;
  public
    constructor Create(aOwner : TComponent; EnableReset: Boolean);
    destructor Destroy; override;

    {*** To be called from Owner'S KeyPress}
    procedure HookKeyPress(var Key: WideChar);
    procedure CheckResetKey(var Key : WideChar);
    procedure SetCaret;

    procedure ValidateEdit; dynamic;

    {*** Replaces Owner'S WindowProc}
    procedure HookWndProc(var Message: TMessage); virtual;

    procedure DefaultHandler(var Message); override;
    property EditModified: Boolean read GetModified write SetModified;
    property EditText : WideString read GetText write SetText;
    property EditTextLen: Integer read GetTextLen;
    property EditMaxLen: Integer read GetMaxLen;
    property EditHandle: THandle read GetEditHandle;
    property EditControl: TWinControl read GetEditControl;
    property Owner : TComponent read FOwner;
  published
    property OnValidChanged: TValidChangedEvent read FOnValidChanged
                                                write FOnValidChanged;
  end;

  TTntPxMaskEdit = class(TTntEdit2)
  private
    FPxMask : TControlPxMask;
    function GetOnValidChanged: TValidChangedEvent;
    procedure SetOnValidChanged(Value: TValidChangedEvent);
    procedure SetPxMask(Value : TControlPxMask);
  public
    constructor Create(aOwner : TComponent); override;
    destructor Destroy; override;
    function Valid: Boolean; virtual; {New, used when AutoValidate = False}
  published
    property PxMask : TControlPxMask read FPxMask write SetPxMask;
    property OnValidChanged: TValidChangedEvent read GetOnValidChanged
                                                write SetOnValidChanged
                                                stored False;
  end;

  TTntPxMaskEditR = class(TTntPxMaskEdit) {New}
  private
    FRequired: Boolean;
  public
    constructor Create(AOwner : TComponent); override;
    function Valid: Boolean; override;
    function ValidL (L: TTntLabel): Boolean; virtual;
  published
    property Required: Boolean read FRequired write FRequired default True;
  end;

{*** check syntax of supplied mask, if an exception is raised then ErrorPos > 0;
     removes any trailing blanks and illegal commas: '[a,b,,c]  ' -> '[a,b,c]' }
procedure PxMaskCheckSyntax(var aMask: WideString; var ErrorPos: Integer);

{*** aMask : ParadoxMask (no syntax checks)
     Input : WideString to be Checked
     AutoFill : if True then Input will automatically be completed
     returns position of error in Input, if any: 0 = OK}
function PxMaskCheckInput(const aMask: WideString; var Input: WideString;
  AutoFill: Boolean): Integer;

{*** aMask : ParadoxMask (no syntax checks)
     Input : WideString to be Checked
     returns position of error in Input, or Length(Input), if not complete}
function PxMaskValidate(const aMask: WideString; const Input: WideString): Integer;

{*** returns TRUE if Control has a published property PxMask}
function HasPxMask(Control: TPersistent; var PxMask: TTntPxMask) : Boolean;

implementation

uses
  TntControls, TntSysUtils,
  TypInfo, TntStrUtil0, TntEdiTAmp,
  TntdnUtil;

{--------------------------------------}

{*** copied from VALIDATE.PAS (and partially modified)}
type
  TPxMaskResult = (prComplete, prIncomplete, prEmpty, prError, prSyntax,
    prAmbiguous, prIncompNoFill);

const
  SpecialChars = '#?&!@*{}[],~';
  Digits = '0123456789';

function IsComplete(Rslt: TPxMaskResult): Boolean;
begin
  Result := Rslt in [prComplete, prAmbiguous];
end;

function IsIncomplete(Rslt: TPxMaskResult): Boolean;
begin
  Result := Rslt in [prIncomplete, prIncompNoFill];
end;

{The compiler produces following warnings/hints:
  Hint - Value Assigned to 'Iteration' never used
  Warning - Variable 'Rslt' might not have been initialized.
  Warning - Variable 'IncompJ' might not have been initialized.
  Warning - Variable 'Incomp' might not have been initialized.

  They're produced from Borland'S original code, there wasn't such a smart
  compiler in BP7, that'S why I turn off warnings and hints for function Picture
}

{$WARNINGS Off}
{$HINTS Off}

function Picture(const aMask: WideString; var Input: WideString;
  AutoFill: Boolean; var ErrorPos : Integer): TPxMaskResult;
var
  I, J: Cardinal;
  Rslt: TPxMaskResult;
  Reprocess: Boolean;

  function Process(TermCh: Cardinal): TPxMaskResult;
  var
    Rslt: TPxMaskResult;
    Incomp: Boolean;
    OldI, OldJ, IncompJ, IncompI: Cardinal;

    { Consume input }

    procedure Consume(Ch: WideChar);
    begin
      Input[J] := Ch;
      Inc(J);
      Inc(I);
      Inc(ErrorPos);
    end;

    { Skip a character or a picture group }

    procedure ToGroupEnd(var I: Cardinal);
    var
      BrkLevel, BrcLevel: Integer;
    begin
      BrkLevel := 0;
      BrcLevel := 0;
      repeat
        if I = TermCh then Exit;
        case aMask[I] of
          '[': Inc(BrkLevel);
          ']': Dec(BrkLevel);
          '{': Inc(BrcLevel);
          '}': Dec(BrcLevel);
          ';': Inc(I);
          '*':
            begin
              Inc(I);
              while  Pos (aMask[I], Digits) <> 0  do Inc(I);
              ToGroupEnd(I);
              Continue;
            end;
        end;
        Inc(I);
      until (BrkLevel = 0) and (BrcLevel = 0);
    end;

    { Find the a comma separator }

    function SkipToComma: Boolean;
    begin
      repeat ToGroupEnd(I) until (I = TermCh) or (aMask[I] = ',');
      while (I <= TermCh) and (aMask[I] = ',') do Inc(I);
      Result := I < TermCh;
    end;

    { Calclate the end of a group }

    function CalcTerm: Cardinal;
    begin
      Result := I;
      ToGroupEnd(Result);
    end;

    { The next group is repeated X times }

    function Iteration: TPxMaskResult;
    var
      oldJ, Itr, K, L: Cardinal;
      NewTermCh: Cardinal;
      Rslt: TPxMaskResult;
    begin
      Itr := 0;
      Iteration := prError;

      Inc(I);  { Skip '*' }

      { Retrieve number }

      while  Pos (aMask[I], Digits) <> 0   do
      begin
        Itr := Itr * 10 + Byte(aMask[I]) - Byte('0');
        Inc(I);
      end;

      if I > TermCh then
      begin
        Iteration := prSyntax;
        Exit;
      end;

      K := I;
      NewTermCh := CalcTerm;

      { If Itr is 0 allow any number, otherwise enforce the number }
      if Itr <> 0 then
      begin
        for L := 1 to Itr do
        begin
          I := K;
          Rslt := Process(NewTermCh);
          if not IsComplete(Rslt) then
          begin
            { Empty means incomplete since all are required }
            (**** BugFix: empty means incomplete ONLY if no optional input.
                 e.g. *3[..] means 0 to 3 times, *3{...} means exactly 3 times *)
            if (Rslt = prEmpty) then
            begin
              if (aMask[K] <> '[') then
                Iteration := prIncomplete
              else
                Iteration := prAmbiguous;
            end;
            Exit;
          end;
        end;
      end
      else
      begin
        repeat
          OldJ := J;
          I := K;
          Rslt := Process(NewTermCh);
        until not IsComplete(Rslt) or (OldJ = J);
        if (Rslt = prEmpty) or (Rslt = prError) then
        begin
          Inc(I);
          Rslt := prAmbiguous;
        end;
      end;
      if IsComplete(Rslt) then I := NewTermCh;
      Iteration := Rslt;
    end;

    { Process a picture group }

    function Group: TPxMaskResult;
    var
      TermCh: Cardinal;
    begin
      TermCh := CalcTerm;
      Inc(I);
      Result := Process(TermCh - 1);
      if not IsIncomplete(Result) then I := TermCh;
    end;

    function CheckComplete(Rslt: TPxMaskResult): TPxMaskResult;
    var
      J, Z: Cardinal;
    begin
      J := I;
      if IsIncomplete(Rslt) then
      begin
        { Skip optional pieces }
        while True do
          case aMask[J] of
            '[': ToGroupEnd(J);
            '*':
              if  Pos (aMask[J+1], Digits) = 0  then
              begin
                Inc(J);
                ToGroupEnd(J);
              end
              else
              begin
                Z := J;
                repeat Inc(Z)
                until  Pos (aMask[Z], Digits) = 0;
                if aMask[Z] = '[' then
                begin
                  J := Z;
                  ToGroupEnd(J);
                end
                else
                  Break;
              end;
          else
            Break;
          end;

        if J = TermCh then Rslt := prAmbiguous;
      end;
      Result := Rslt;
    end;

    function Scan: TPxMaskResult;
    var
      Ch: WideChar;
      Rslt: TPxMaskResult;
    begin
      Scan := prError;
      Rslt := prEmpty;
      while (I <> TermCh) and (aMask[I] <> ',') do
      begin
        if J > Length(Input) then
        begin
          Scan := CheckComplete(Rslt);
          Exit;
        end;

        Ch := Input[J];
        case aMask[I] of
          '#': if  Pos (Ch, Digits) = 0 then Exit
               else Consume(Ch);
          '?': if not IsCharAlphaW(Ch) then Exit
               else Consume(Ch);
          '&': if not IsCharAlphaW(Ch) then Exit
               else Consume(WideUpCase(Ch));
          {New special WideChar in DBD32}
          '~': if not IsCharAlphaW(Ch) then Exit
               else Consume(WideLoCase(Ch));
          '!': Consume(WideUpCase(Ch));
          '@': Consume(Ch);
          '*':
            begin
              Rslt := Iteration;
              if not IsComplete(Rslt) then
              begin
                Scan := Rslt;
                Exit;
              end;
              if Rslt = prError then Rslt := prAmbiguous;
            end;
          '{':
            begin
              Rslt := Group;
              if not IsComplete(Rslt) then
              begin
                Scan := Rslt;
                Exit;
              end;
            end;
          '[':
            begin
              Rslt := Group;
              if IsIncomplete(Rslt) then
              begin
                Scan := Rslt;
                Exit;
              end;
              if (Rslt = prError) then Rslt := prAmbiguous;
            end;
        else
          if aMask[I] = ';' then Inc(I);
         if (WideUpCase(aMask[I]) = WideUpCase(Ch)) or (Ch = ' ')
          then Consume(aMask[I])
          else Exit;
        end;

        if Rslt = prAmbiguous then
          Rslt := prIncompNoFill
        else
          Rslt := prIncomplete;
      end;

      if Rslt = prIncompNoFill then
        Scan := prAmbiguous
      else
        Scan := prComplete;
    end;

  begin
    Incomp := False;
    OldI := I;
    OldJ := J;
    repeat
      Rslt := Scan;

      { Only accept completes if they make it farther in the input
        stream from the last incomplete }
      {Bug???Fix??? J < IncompJ --> J <= IncompJ}
      if (Rslt in [prComplete, prAmbiguous]) and Incomp and (J <= IncompJ) then
      begin
        Rslt := prIncomplete;
        J := IncompJ;
      end;

      if (Rslt = prError) or (Rslt = prIncomplete) then
      begin
        Process := Rslt;
        if not Incomp and (Rslt = prIncomplete) then
        begin
          Incomp := True;
          IncompI := I;
          IncompJ := J;
        end;
        I := OldI;
        J := OldJ;
        if not SkipToComma then
        begin
          if Incomp then
          begin
            Process := prIncomplete;
            I := IncompI;
            J := IncompJ;
          end;
          Exit;
        end;
        OldI := I;
      end;
    until (Rslt <> prError) and (Rslt <> prIncomplete);

    if (Rslt = prComplete) and Incomp then
      Process := prAmbiguous
    else
      Process := Rslt;
  end;
begin
  ErrorPos := 1;
  Picture := prComplete;
  if aMask = '' then Exit;

  Picture := prEmpty;
  if Input = '' then Exit;

  J := 1;
  I := 1;

  Rslt := Process(Length(aMask) + 1);
  if (Rslt <> prError) and (Rslt <> prSyntax) and (J <= Length(Input)) then
    Rslt := prError;

  if (Rslt = prIncomplete) and AutoFill then
  begin
    Reprocess := False;
    while (I <= Length(aMask)) and
      (Pos (aMask[I],SpecialChars) = 0) do
    begin
      if aMask[I] = ';' then Inc(I);
      Input := Input + aMask[I];
      Inc(I);
      Reprocess := True;
    end;
    J := 1;
    I := 1;
    if Reprocess then
      Rslt := Process(Length(aMask) + 1)
  end;

  case Rslt of
    prAmbiguous : Picture := prComplete;
    prIncompNoFill : Picture := prIncomplete;
  else
    Picture := Rslt;
  end;
end;

{$WARNINGS On}
{$HINTS On}

function RemoveCommas(const aMask : WideString): WideString;
type
  TStatus = (none, wasLiteral, noComma);
var
  Len, I, J : Integer;
  Status : TStatus;

  procedure PutChar;
  begin
    Inc(J);
    if I <= Length(aMask) then Result[J] := aMask[I];
    Inc(I);
  end;

begin
  Len := Length(aMask);
  Result := aMask;
  setLength(Result, Len);
  J := 0;
  I := 1;
  Status := noComma;
  while I <= Len do
  begin
    case aMask[I] of
      '*':
        begin
          PutChar;
          while  Pos (aMask[I],Digits) <> 0  do PutChar;
          Status := noComma;
          Continue;
        end;
      '[', '{' : Status := noComma;
      ']', '}' :
        begin
          if (Status <> wasLiteral) and (Result[J] = ',') then Dec(J);
          Status := none;
        end;
      ';':
        begin
          PutChar;
          Status := wasLiteral;
        end;
      ',':
        begin
          if Status = noComma then
          begin
            while ((I <= Len) and (aMask[I] = ',')) do Inc(I);
            Status := noComma;
            Continue;
          end;
          Status := noComma;
        end;
      else
        Status := none;
    end;
    PutChar;
  end;
  if (Status <> wasLiteral) and (aMask[Len] = ',') then Dec(J);
  setLength(Result, J);
end;

procedure PxMaskCheckSyntax(var aMask : WideString; var ErrorPos : Integer);
const
  MaxBracketLevel = 32;  { maximale Anzahl der Klammerebenen }
var
  I: Integer;
  fStack : array[0..pred(MaxBracketLevel)] of WideChar; {Klammerstack}
  fBracketLevel : shortInt;
  isLiteral : Boolean;

  procedure RaiseError(const Err : WideString);
  var
    Error : EPxMaskError;
  begin
    ErrorPos := I;
    Error := EPxMaskError.Create(SPxMaskSyntax+ #13#10 + Err);
    Error.ErrorCode := errPxMaskSyntax;
    raise Error;
  end;

  procedure PopBracket;
  begin
    if fBracketLevel > 0 then
    begin
      Dec(fBracketLevel);
      if (fStack[fBracketLevel] <> aMask[I]) then
        RaiseError(Format(SPxMaskBracketError, [fStack[fBracketLevel]]));
    end else
      RaiseError(Format(SPxMaskBracketClose, [aMask[I]]));
  end;

  procedure PushBracket(c : WideChar);
  begin
    if fBracketLevel >= MaxBracketLevel then
      RaiseError(SPxMaskBracketLevel);
    fStack[fBracketLevel] := c;
    Inc(fBracketLevel);
  end;
begin
  ErrorPos := 0;
  aMask := TrimRight(aMask);
  if aMask = '' then Exit;
  fBracketLevel := 0;
  I := 1;
  isLiteral := False;
  while I <= Length(aMask) do
  begin
    case aMask[I] of
      '[': pushBracket(']');
      '{': pushBracket('}');
      ']', '}':
        begin
          popBracket;
          case aMask[i-1] of
            '[', '{':
              if not isLiteral then
                RaiseError(SPxMaskBracketEmpty);
          end;
        end;
      ';':
        begin
          Inc(I, 2);
          isLiteral := True;
          Continue;
        end;
      '*':
        begin
          Inc(I);
          while Pos (aMask[I],Digits) <> 0  do Inc(I);
          if (I > Length(aMask)) or (Pos (aMask[I],']}*') <> 0) then
            RaiseError(SPxMaskIterationExpected);
          Continue;
        end;
    end;
    isLiteral := False;
    Inc(I);
  end;
  if fBracketLevel > 0 then
  begin
    Dec(fBracketLevel);
    RaiseError(Format(SPxMaskBracketMissing, [fStack[fBracketLevel]]));
  end;
  I := Length(aMask);
  if (aMask[I] = ';') and not isLiteral then
    RaiseError(SPxMaskLiteralExpected);
  aMask := RemoveCommas(aMask);
end;

function PxMaskCheckInput(const aMask   : WideString;
                            var Input   : WideString;
                                AutoFill: Boolean): Integer;
begin
  if not (Picture(aMask, Input, AutoFill, Result) in [prError, prSyntax]) then
    Result := 0;
end;

function PxMaskValidate(const aMask : WideString;
                        const Input : WideString): Integer;
var
  Str: WideString;
begin
  Str := Input;
  if Picture(aMask, Str, False, Result) in [prComplete, prEmpty] then
    Result := 0;
end;

function PxMaskValidateText(const aMask : WideString;
                            var Input : WideString): Integer;
begin
  if Picture(aMask, Input, False, Result) in [prComplete, prEmpty] then
    Result := 0;
end;

{**** TTntPxMask ****}
type
  TSelection =
  record
    StartPos, EndPos: Integer;
  end;

constructor TTntPxMask.Create(aOwner : TComponent);
begin
  inherited Create;
  Clear;
end;

procedure TTntPxMask.SetAutoValidate(Value: Boolean);
begin
  if Value <> AutoValidate then
  begin
    FAutoValidate := Value;
    Include(FState, pxStored);
  end;
end;

procedure TTntPxMask.SetAutoFill(Value: Boolean);
begin
  if Value <> AutoFill then
  begin
    FAutoFill := Value;
    Include(FState, pxStored);
  end;
end;

procedure TTntPxMask.SetErrMsg(const Value: WideString);
begin
  if Value <> ErrorMessage then
  begin
    FErrMsg := Value;
    Include(FState, pxStored);
  end;
end;

procedure TTntPxMask.SetMask(const Value : WideString);
var
  S : WideString;
begin
  if Value <> Mask then
  begin
    if Value <> '' then
    begin
      S := Value;
      CheckSyntax(S);
    end;
    if (pxMaskFromParadox in FState) then
      if (csDesigning in Application{Owner}.ComponentState) then
        raise exception.Create(SPxMaskDefinedOnTableLevel)
      else
        Exit;
    Include(FState, pxStored);
    FMask := S;
    if Value <> '' then Include(FState, pxModified);
  end;
end;

procedure TTntPxMask.SetRaiseInputError(Value: Boolean);
begin
  if Value <> RaiseInputError then
  begin
    FRaiseInputError := Value;
    Include(FState, pxStored);
  end;
end;

function TTntPxMask.MaskStored : Boolean;
begin
  Result := FState * [pxMaskFromParadox, pxStored] = [pxStored];
end;

function TTntPxMask.PropsStored: Boolean;
begin
  Result := pxStored in FState;
end;

function TControlPxMask.GetTextLen: Integer;
begin
  if  not Win32PlatformIsUnicode  then  begin
    Result := SendMessageA(EditHandle, WM_GETTEXTLENGTH, 0, 0);
  end
  else  begin
    Result := SendMessageW(EditHandle, WM_GETTEXTLENGTH, 0, 0);
  end;
end;

function TControlPxMask.GetModified : Boolean;
begin
  Result := pxModified in FState;
end;

procedure TTntPxMask.Validate(var Value : WideString);
begin
  if IsMasked then
  begin
    Value := CheckValid(Value);
    if ErrorPos = 0 then Exit;
    ValidateError(ErrorPos > Length(Value));
    Abort;
  end;
end;

function TTntPxMask.CheckValid(S : WideString): WideString;
begin
  FErrPos := PxMaskValidateText(Mask, S);
  if (ErrorPos <> 0) then
  begin
    PxMaskCheckInput(Mask, S, AutoFill);
    FErrPos := PxMaskValidateText(Mask, S);
  end;
  if (ErrorPos <> 0) and not AutoFill then
  begin
    PxMaskCheckInput(Mask, S, True);
    FErrPos := PxMaskValidateText(Mask, S);
  end;
  Result := S;
  ErrorPos := FErrPos;
end;

procedure TControlPxMask.CheckEditText;
begin
  CheckValid(EditText);
  EditModified := True;
end;

procedure TTntPxMask.Assign(Source: TPersistent);
var
  l: LongInt;
begin
  if not (Source is TTntPxMask) then
    if HasPxMask(Source, TTntPxMask(l)) and (TObject(l) is TTntPxMask) then
      Source := TPersistent(l);
  if Source is TTntPxMask then
  with TTntPxMask(Source) do begin
    if not (pxMaskFromParadox in Self.FState) then
      Self.Mask := Mask;
    Self.ErrorMessage := ErrorMessage;
    Self.AutoFill := AutoFill;
    Self.AutoValidate := AutoValidate;
    Self.RaiseInputError := RaiseInputError;
  end
  else
    if source <> nil then inherited Assign(Source);
end;

procedure TTntPxMask.LoadMask;
begin
  if MaskLoaded then Exclude(FState, pxStored) else Include(FState, pxStored);
end;

function TTntPxMask.MaskLoaded : Boolean;
begin
  Result := False;
end;

function TTntPxMask.FormatErrorText(Ident: WideString): WideString;
var
  I : Integer;
const
  cMask = '%MASK%';
  cError = '%ERROR%';
begin
  Result := ErrorMessage;
  if Result <> '' then
  begin
    I := Pos(cError, Result);
    if I > 0 then
      Result := Format('%s%s%s', [Copy(Result, 1, I - 1), Ident,
                                  Copy(Result, I + Length(cError), MaxInt)]);
  end
  else
    Result := Ident;
  I := Pos(cMask, Result);
  if I > 0 then
    Result := Format('%s%s%s', [Copy(Result, 1, I - 1),
                                Mask,
                                Copy(Result, I + Length(cMask), MaxInt)]);
end;

procedure TTntPxMask.Clear;
begin
  ErrorMessage := '';
  Mask := '';
  FAutoFill := AutoFillDefault;
  FAutoValidate := AutoValidateDefault;
  FRaiseInputError := RaiseInputErrorDefault;
end;

function TTntPxMask.Cleared: Boolean;
begin
  Result := (AutoFill = AutoFillDefault) and
            (AutoValidate = AutoValidateDefault) and
            (RaiseInputError = RaiseInputErrorDefault) and
            (Mask = '') and
            (ErrorMessage = '');
end;

procedure TTntPxMask.CheckSyntax(var aMask : WideString);
begin
  PxMaskCheckSyntax(aMask, FErrPos);
end;

function TTntPxMask.isMasked : Boolean;
begin
  Result := Mask <> '';
end;

function TTntPxMask.Valid (const S : WideString): Boolean; {New, used when AutoValidate = False}
begin
  if IsMasked then  begin
    CheckValid(S);
    Valid := ErrorPos=0;
  end
  else
    Valid := True;
end;

procedure TTntPxMask.InputError;
var
  S : WideString;
  Error : EPxMaskError;
begin
  if BeepOnPxError and BeepOnPxInputError then Beep;
  if RaiseInputError then
  begin
    S := FormatErrorText(SPxMaskIllegalChar);
    if S <> '' then
    begin
      Error := EPxMaskError.Create(S);
      Error.ErrorCode := errPxMaskIllegalChar;
      Include(FState, pxReEnter);
      raise Error;
    end;
  end;
end;

procedure TTntPxMask.ValidChanged;
begin
end;

procedure TTntPxMask.ValidateError(Incomplete : Boolean);
var
  S : WideString;
  Error : EPxMaskError;
begin
  if BeepOnPxError then Beep;
  if Incomplete then
    S := FormatErrorText(SPxMaskInputIncomplete)
  else
    S := FormatErrorText(SPxMaskIllegalString);
  if S <> '' then
  begin
    Error := EPxMaskError.Create(S);
    if Incomplete then
      Error.ErrorCode := errPxMaskInputIncomplete
    else
      Error.ErrorCode := errPxMaskIllegalString;
    raise Error;
  end
  else
    Abort;
end;

{*** TControlPxMask ***}
constructor TControlPxMask.Create(aOwner : TComponent; EnableReset: Boolean);
begin
  inherited Create(aOwner);
  FOwner := aOwner;
  FEnableReset := EnableReset;
  FControlWndProc := (aOwner as TControl).WindowProc;
  TControl(aOwner).WindowProc := HookWndProc;
end;

destructor TControlPxMask.Destroy;
begin
  if Assigned(FControlWndProc) then TControl(Owner).WindowProc := FControlWndProc;
  inherited Destroy;
end;

procedure TControlPxMask.SetModified(Value: Boolean);
begin
  if Value then
    Include(FState, pxModified)
  else
    Exclude(FState, pxModified);
end;

function TControlPxMask.GetText: WideString;
var
  Len: Integer;
begin
  Len := GetTextLen;
  SetString(Result, PWideChar(nil), Len);
  if Len <> 0 then
    if  not Win32PlatformIsUnicode  then  begin
      SendMessageA(EditHandle, WM_GETTEXT, Len + 1, Longint(Result));
    end
    else  begin
      SendMessageW(EditHandle, WM_GETTEXT, Len + 1, Longint(Result));
    end;
end;

procedure TControlPxMask.SetText(const Value: WideString);
var
  EH : THandle;
begin
  if GetText <> Value then
  begin
    EH := EditHandle;
    if  not Win32PlatformIsUnicode  then  begin
      SendMessageA(EH, WM_SETTEXT, 0, Integer(PAnsiChar(AnsiString(Value))));
      SendMessageA(EH, CM_TEXTCHANGED, 0, 0);
    end
    else begin
      SendMessageW(EH, WM_SETTEXT, 0, Integer(PWideChar(Value)));
      SendMessageW(EH, CM_TEXTCHANGED, 0, 0);
    end;
    CheckValid(Value);
  end;
end;

procedure TControlPxMask.PrepareReset(const OldText : WideString);
begin
  if FEnableReset and not (csDesigning in FOwner.ComponentState) then
  begin
    if not (pxReEnter in FState) then
      FOldValue := OldText;
  end;
  EditModified := False;
  CheckValid(EditText);
end;

procedure TControlPxMask.HookWndProc(var Message: TMessage);

  function DoKeyPress: Boolean;
  var
    Key: WideChar;
  begin
    Key := GetWideCharFromWMCharMsg (TWMChar(Message));
    CheckResetKey(Key);
    HookKeyPress(Key);
    SetWideCharForWMCharMsg (TWMChar(Message), Key);
    Result := (Key = #0);
  end;

begin
  with Message do begin
    case Msg of
      WM_CHAR: if DoKeyPress then Exit;
      CM_Exit:
        if AutoValidate and IsMasked and not (csDesigning in FOwner.ComponentState) then
          ValidateEdit;
      CM_Enter:
        if IsMasked and not (csDesigning in FOwner.ComponentState) then
        begin
          if not (pxReEnter in FState) then
            PrepareReset(EditText)
          else
          begin
            SetCaret;
            Exclude(FState, pxReEnter);
            Exit;
          end
        end;
    end;

    Dispatch(Message);

    case Msg of
      WM_Command:
        with TWMCommand(Message) do
          if Ctl = EditHandle then
            case NotifyCode of
              EN_SETFOCUS: if (ErrorPos > 0) then SetCaret;
              EN_CHANGE:   CheckEditText;
            end;
      CN_Command:
        with TWMCommand(message) do
        case NotifyCode of
          CBN_SELCHANGE : CheckEditText;
        end;
      WM_KeyUp:
        with TWMKeyUp(message) do
          if ((CharCode = VK_DELETE) and
              ([ssShift, ssCtrl] * KeyDataToShiftState(KeyData) = [])) or
             (CharCode = VK_BACK)
          then
            CheckEditText
          else
            if (CharCode = VK_ESCAPE) then
              ErrorPos := 0;
      WM_Cut,
      WM_Paste:
        if isMasked then CheckEditText;
      CM_TextChanged:
        if not (Owner is TCustomComboBox) then
          PrepareReset(EditText);
      CM_WantSpecialKey:
        if (TWMKey(Message).CharCode = VK_ESCAPE) and IsMasked and EditModified then
          Message.Result := 1;
    end;
  end;
end;

procedure TControlPxMask.SetCaret;
begin
  if ErrorPos = 0 then Exit;
  if  not Win32PlatformIsUnicode  then  begin
    SendMessageA(EditHandle, EM_SETSEL, ErrorPos - 1, ErrorPos - 1);
  end
  else  begin
    SendMessageW(EditHandle, EM_SETSEL, ErrorPos - 1, ErrorPos - 1);
  end;  
end;

procedure TControlPxMask.CheckResetKey(var Key : WideChar);
begin
  if not (pxSending in FState) and (Cardinal(Key) = VK_ESCAPE) then
  begin
    if EditModified then
    begin
      Reset;
      Key := #0;
    end;
    ErrorPos := 0;
  end;
end;

procedure TTntPxMask.SetErrorPos(Value : Integer);
var
  prevValid: Boolean;
begin
  FErrPos := Value;
  prevValid := (pxValid in FState);
  if Value > 0 then Exclude(FState, pxValid) else Include(FState, pxValid);
  if prevValid xor (pxValid in FState) then
    ValidChanged;
end;

function TControlPxMask.GetEditControl: TWinControl;
begin
  Result := FindControl(EditHandle);
  if Result = nil then
    Result := TWinControl(Owner);
end;

function TControlPxMask.GetMaxLen: Integer;
var
  Ctl: TWinControl;
begin
  Result := 0;
  Ctl := EditControl;
  if (Ctl is TCustomEdit) then
    Result := TTntPxMaskEdit(Ctl).MaxLength;
end;

procedure TControlPxMask.HookKeyPress(var Key: WideChar);
var
  S: WideString;
  MaxLen, I, Len, ErrPos, EP: Integer;
  Selection: TSelection;
  EH : THandle;
begin
  if (Word(Key) > 32) and (not (pxSending in FState)) and
     IsMasked and CanEditModify
  then
  begin
    EH := EditHandle;
    if  not Win32PlatformIsUnicode  then  begin
      SendMessageA(EH, EM_GETSEL, Longint(@Selection.StartPos), Longint(@Selection.EndPos));
      SetLength(S, Selection.StartPos + 1);
      SendMessageA(EH, WM_GETTEXT, Succ(Selection.StartPos), longint(S));
    end
    else  begin
      SendMessageW(EH, EM_GETSEL, Longint(@Selection.StartPos), Longint(@Selection.EndPos));
      SetLength(S, Selection.StartPos + 1);
      SendMessageW(EH, WM_GETTEXT, Succ(Selection.StartPos), longint(S));
    end;
    S[Succ(Selection.StartPos)] := Key;
    Key := #0;
    if  not Win32PlatformIsUnicode  then  begin
      EP := SendMessageA(EH, WM_GETTEXTLENGTH, 0, 0);
    end
    else begin
      EP := SendMessageW(EH, WM_GETTEXTLENGTH, 0, 0);
    end;
    if  Selection.EndPos = EP  then
    begin
      ErrPos := PxMaskCheckInput(Mask, S, AutoFill);
      Len := Length(S) - Selection.StartPos;
    end
    else
    begin
      Len := 1;
      ErrPos := PxMaskCheckInput(Mask, S, False);
    end;
    if ErrPos = 0 then
    begin
      MaxLen := 0;
      if (Len > 1) then
      begin
        MaxLen := EditMaxLen;
        if (MaxLen > 0) and (Length(S) > MaxLen) then  begin
          if  not Win32PlatformIsUnicode  then  begin
            SendMessageA(EH, EM_LIMITTEXT, Length(S), 0);
          end
          else begin
            SendMessageW(EH, EM_LIMITTEXT, Length(S), 0);
          end;
        end  
        else
          MaxLen := 0;
      end;
      Include(FState, pxSending);
      try
        for I := Selection.StartPos + 1 to Selection.StartPos + Len do
          if  not Win32PlatformIsUnicode  then  begin
            SendMessageA(EH, WM_CHAR, ord(S[I]), 1);
          end
          else begin
            SendMessageW(EH, WM_CHAR, ord(S[I]), 1);
          end;
      finally
        Exclude(FState, pxSending);
        if MaxLen > 0 then
          if  not Win32PlatformIsUnicode  then  begin
            SendMessageA(EH, EM_LIMITTEXT, MaxLen, 0)
          end
          else begin
            SendMessageW(EH, EM_LIMITTEXT, MaxLen, 0)
          end;
      end;
      CheckEditText;
    end
    else
    begin
      CheckEditText;
      InputError;
      Abort;
    end;
  end
  else
    if (Cardinal(Key) = VK_RETURN) and AutoValidate then
      validateEdit;
end;

procedure TControlPxMask.ValidateEdit;
var
  S: WideString;
begin
  if EditModified then
  begin
    S := EditText;
    Validate(S);
    EditText := S;
  end;
end;

procedure TControlPxMask.ValidateError(Incomplete : Boolean);
begin
  if not (csDesigning in FOwner.ComponentState) and (Owner is TWinControl) then
  begin
    Include(FState, pxReEnter);
    with TWinControl(Owner) do
      if CanFocus then SetFocus;
  end;
  inherited;
end;

procedure TControlPxMask.Reset;
begin
  if FEnableReset and not (csDesigning in FOwner.ComponentState) and
    EditModified then
  begin
    EditText := FOldValue;
    if  not Win32PlatformIsUnicode  then  begin
      SendMessageA(EditHandle, EM_SETSEL, 0, -1);
    end
    else begin
      SendMessageW(EditHandle, EM_SETSEL, 0, -1);
    end;
  end;
  Exclude(FState, pxReEnter);
  EditModified := False;
  CheckValid(EditText);
end;

function TControlPxMask.CanEditModify: Boolean;
begin
  Result := False;
  if (Owner is TCustomEdit) then
    Result := not TTntPxMaskEdit(Owner).ReadOnly;
end;

procedure TControlPxMask.ValidChanged;
begin
  if Assigned(FOnValidChanged) then
    FOnValidChanged(Self, EditControl, ErrorPos = 0);
end;

procedure TControlPxMask.DefaultHandler(var Message);
begin
  FControlWndProc(TMessage(Message));
end;

function TControlPxMask.GetEditHandle: THandle;
begin
  Result := 0;
  if TWinControl(FOwner).HandleAllocated then
  begin
    Result := TWinControl(FOwner).Handle;
  end;
end;

{*** TTntPxMaskEdit ***}
constructor TTntPxMaskEdit.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  FPxMask := TControlPxMask.Create(Self, True); {//MUST be created after inherited}
end;

destructor TTntPxMaskEdit.Destroy;
begin
  if Assigned(FPxMask) then FPxMask.Free;
  inherited Destroy;
end;

function TTntPxMaskEdit.GetOnValidChanged: TValidChangedEvent;
begin
  Result := FPxMask.OnValidChanged;
end;

procedure TTntPxMaskEdit.SetOnValidChanged(Value: TValidChangedEvent);
begin
  FPxMask.OnValidChanged := Value;
end;

procedure TTntPxMaskEdit.SetPxMask(Value : TControlPxMask);
begin
  FPxMask.Assign(Value);
end;

function HasPxMask(Control: TPersistent; var PxMask: TTntPxMask) : Boolean;
begin
  Result := GetOrdProperty(Control, 'PxMask', tkClass, LongInt(PxMask));
end;

{--------------------------------------}
{-PUBLIC-----------}

function TTntPxMaskEdit.Valid: Boolean;
begin
  Result := FPxMask.Valid (Text);
end;

{--------------------------------------}
{-PUBLIC-----------}

constructor TTntPxMaskEditR.Create(AOwner: TComponent);
begin
  inherited;
  FRequired := True;
end;

function TTntPxMaskEditR.Valid: Boolean;
begin
  Result := ((not FRequired) and (Text = '')) or
    (FRequired and TntEditValid (Self) and inherited Valid);
end;

function TTntPxMaskEditR.ValidL (L: TTntLabel): Boolean;
begin
  Result := ((not FRequired) and (Text = '')) or
    (FRequired and TntEditValidL (L, Self) and inherited Valid);
end;

{--------------------------------------}

end.
