unit TntdnPxProp;

(*****************************************************************************
Copyright  1997-99 Dieter Nth
version    : 2.1
date       : 10.08.1999
description:

  Editor for Paradox-style inputmasks

  1.03 Added:
    Example mask handling

  2.00 Added:
    ErrorMessage-Edit
******************************************************************************)

{$I JVCL.inc}

interface

uses
  Windows, SysUtils, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, Mask,
  TntClasses, TntForms, TntDialogs, TntStdCtrls, TntButtons, TntExtCtrls, TntGraphics,
  TntdnGlobal, TntdnPxMask, TntStdCtrls2;

resourcestring
  RS_dnPxProp_OpenDialog_Title = 'Open Sample Paradox Masks';
  RS_dnPxProp_OpenDialog_Filter = 'Paradox sample masks (*.PXD)|*.PXD|All Files (*.*)|*.*';
  RS_dnPxProp_BB_Restore_Hint = 'Restore original mask';
  RS_dnPxProp_BB_LoadMask_Hint = 'Load sample masks from file';
  RS_dnPxProp_BB_Delete_Hint = 'Delete selected mask from drop-down list';
  RS_dnPxProp_BB_Add_Hint = 'Add new mask to drop-down list';
  RS_dnPxProp_BB_Use_Hint = 'Copy to Paradox mask';

type
  TdlgPxMaskEditor = class(TTntForm)
    BB_Add: TTntBitBtn;
    BB_Cancel: TTntBitBtn;
    BB_Delete: TTntBitBtn;
    BB_LoadMask: TTntBitBtn;
    BB_OK: TTntBitBtn;
    BB_Restore: TTntSpeedButton;
    BB_Use: TTntBitBtn;
    CB_AutoFill: TTntCheckBox;
    CB_AutoValidate: TTntCheckBox;
    CB_Masks: TTntComboBox;
    CB_RaiseInputError: TTntCheckBox;
    E_ErrorMsg: TTntEdit;
    E_Mask: TTntEdit;
    E_Test: TTntPxMaskEdit;
    GB_ExampleMasks: TTntGroupBox;
    GB_PdxMask: TTntGroupBox;
    L_ErrorMsg: TTntLabel;
    L_Test: TTntLabel;
    Memo: TTntMemo;
    OpenDialog: TTntOpenDialog;
    P_Memo: TTntPanel;
    procedure BB_AddClick(Sender: TObject);
    procedure BB_DeleteClick(Sender: TObject);
    procedure BB_LoadMaskClick(Sender: TObject);
    procedure BB_RestoreClick(Sender: TObject);
    procedure BB_UseClick(Sender: TObject);
    procedure CB_MasksChange(Sender: TObject);
    procedure CB_MasksDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure CB_MasksKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure CB_AutoFillClick(Sender: TObject);
    procedure CB_AutoValidateClick(Sender: TObject);
    procedure CB_RaiseInputErrorClick(Sender: TObject);
    procedure E_ErrorMsgChange(Sender: TObject);
    procedure E_MaskChange(Sender: TObject);
    procedure E_TestChange(Sender: TObject);
    procedure E_TestValidChanged(Sender: TControlPxMask;
      EditControl: TWinControl; TextValid: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure GB_ExampleMasksEnter(Sender: TObject);
    procedure GB_ExampleMasksExit(Sender: TObject);
  private
    FPxMask: TTntPxMask;
    FOldHandler: TExceptionEvent;
    OldBeepOnPxError,
      OldBeepOnPxInputError,
      ListModified: Boolean;
    procedure SetPxMask(Value: TTntPxMask);
    function GetPxMask: TTntPxMask;
    function GetListMaskValue(Index: Integer): WideString;
    function GetMaskValue(Index: Integer): WideString;
    procedure LoadMaskList(const FileName: WideString);
    procedure AppEx(Sender: TObject; E: Exception);
  protected
    procedure Loaded; override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    property PxMask: TTntPxMask read GetPxMask write SetPxMask;
  end;

var
  dlgPxMaskEditor: TdlgPxMaskEditor;

function NameOf(const Param: WideString): WideString;
function ValueOf(const Param: WideString): WideString;

implementation

uses
  Registry,
  TntdnUtil, TntdnPxCnst;

{$R *.DFM}

const
{$IFDEF DELPHI3}
  RegKey = 'Software\Borland\Delphi\3.0\';
{$ENDIF}
{$IFDEF DELPHI4}
  RegKey = 'Software\Borland\Delphi\4.0\';
{$ENDIF}
{$IFDEF DELPHI5}
  RegKey = 'Software\Borland\Delphi\5.0\';
{$ENDIF}
{$IFDEF DELPHI6}
  RegKey = 'Software\Borland\Delphi\6.0\';
{$ENDIF}
{$IFDEF DELPHI7}
  RegKey = 'Software\Borland\Delphi\7.0\';
{$ENDIF}

function NameOf(const Param: WideString): WideString;
var
  P: Integer;
begin
  P := Pos('=', Param);
  if P > 1 then
    Result := Copy(Param, 1, P - 1)
  else
    Result := '';
end;

function ValueOf(const Param: WideString): WideString;
var
  P: Integer;
begin
  P := Pos('=', Param);
  if P < Length(Param) then
    Result := Copy(Param, P + 1, MaxInt)
  else
    Result := '';
end;

{--------------------------------------}

procedure TdlgPxMaskEditor.E_TestChange(Sender: TObject);
var
  S: WideString;
  I: Integer;
begin
  S := '';
  with E_Test do
    if Length(Text) <> 0 then
    begin
      I := SelStart;
      S := Text;
      try
        PxMask.Validate(S);
        S := SPxPropEdInputOK;
      except
        on E: Exception do
        begin
          SelStart := I;
          S := Text;
          if SelStart < PxMaskCheckInput(PxMask.Mask, S, CB_AutoFill.Checked)
            then
            S := SPxPropEdInputOkToCursor
          else
            S := E.Message;
        end;
      end;
    end;
  Memo.Text := S;
end;

procedure TdlgPxMaskEditor.E_MaskChange(Sender: TObject);
var
  AnyMask: Boolean;
begin
  BB_Restore.Enabled := True;      
  E_Test.Enabled := False;
  E_Test.Color := clBtnFace;
  try
    E_Test.PxMask.Mask := E_Mask.Text;
    AnyMask := Length(E_Mask.Text) > 0;
    if AnyMask then
    begin
      if E_Test.PxMask.Mask <> TrimRight(E_Mask.Text) then
        Memo.Text := WideFormat(SPxPropEdCommasRemoved,
          [E_Test.PxMask.Mask])
      else
        Memo.Text := SPxPropEdSyntaxOK;
      E_Test.Enabled := True;
      E_Test.Color := clWindow;
    end
    else
    begin
      Memo.Text := '';
    end;
    BB_Add.Enabled := AnyMask;
    BB_OK.Enabled := AnyMask;
  except
    on E: EPxMaskError do
    begin
      Memo.Text := E.Message;
      BB_OK.Enabled := False;
      BB_Add.Enabled := False;
    end;
  end;
end;

procedure TdlgPxMaskEditor.BB_RestoreClick(Sender: TObject);
begin
  E_Mask.Text := FPxMask.Mask;
  BB_Restore.Enabled := False;
end;

function ReadMaskFileName: WideString;
var
  AppIniFile: TRegIniFile;
begin
  AppIniFile := TRegIniFile.Create(RegKey);
  try
    Result := AppIniFile.ReadString(SIniEditorsName, SIniEditMaskName, '');
  finally
    AppIniFile.Free;
  end;
end;

procedure TdlgPxMaskEditor.BB_LoadMaskClick(Sender: TObject);
var
  AppIniFile: TRegIniFile;
begin
  with OpenDialog do
    if Execute then
    begin
      LoadMaskList(FileName);
      AppIniFile := TRegIniFile.Create(RegKey);
      try
        AppIniFile.WriteString(SIniEditorsName, SIniEditMaskName, FileName);
      finally
        AppIniFile.Free;
      end;
    end;
end;

procedure TdlgPxMaskEditor.Loaded;
begin
  inherited Loaded;
  LoadMaskList(ReadMaskFileName);
end;

procedure TdlgPxMaskEditor.LoadMaskList(const FileName: WideString);
var
  I: Integer;
  Items: TTntStringList;
begin
  if (Length(FileName) > 0) and FileExists(FileName) then
  begin
    Items := TTntStringList.Create;
    try
      Items.LoadFromFile(FileName);
      I := 0;
      while I < (Items.Count - 1) do
      begin
        if (Length(Items[I]) = 0) then
          Items.Delete(I)
        else if (Pos('=', Items[I]) = 0) then
          raise EPxMaskError.Create(SInvalidSampleMask)
        else
          Inc(I);
        if I >= Items.Count then Break;
      end;
      CB_Masks.Items := Items;
      CB_Masks.ItemIndex := 0;
      CB_MasksChange(CB_Masks);
    finally
      Items.Free;
    end;
  end;
end;

procedure TdlgPxMaskEditor.CB_MasksDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  with CB_Masks do
  begin
    Canvas.FillRect(Rect);
    WideCanvasTextOut(Canvas, Rect.Left + 2, Rect.Top, GetMaskValue(Index));
  end;
end;

procedure TdlgPxMaskEditor.CB_MasksChange(Sender: TObject);
begin
  Memo.Text := GetListMaskValue(CB_Masks.ItemIndex);
  BB_Delete.Enabled := (ActiveControl = CB_Masks) and (CB_Masks.ItemIndex >= 0);
  BB_Use.Enabled := GetMaskValue(CB_Masks.ItemIndex) <> '';
end;

function TdlgPxMaskEditor.GetListMaskValue(Index: Integer): WideString;
begin
  if Index >= 0 then
    Result := NameOf(CB_Masks.Items[Index])
  else
    Result := '?';
end;

function TdlgPxMaskEditor.GetMaskValue(Index: Integer): WideString;
begin
  Result := ValueOf(CB_Masks.Items[Index]);
end;

procedure TdlgPxMaskEditor.BB_UseClick(Sender: TObject);
begin
  E_Mask.Text := GetMaskValue(CB_Masks.ItemIndex);
  E_Mask.SetFocus;
  E_Test.Text := '';
  BB_Add.Enabled := False;
end;

procedure TdlgPxMaskEditor.CB_MasksKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Word(Key) of
    VK_RETURN:
      begin
        if CB_Masks.DroppedDown then BB_UseClick(BB_Use);
        CB_Masks.DroppedDown := False;
      end;
    VK_ESCAPE:
      CB_Masks.DroppedDown := False;
  end;
end;

procedure TdlgPxMaskEditor.BB_DeleteClick(Sender: TObject);
var
  I: Integer;
begin
  with CB_Masks do
    if (ItemIndex > 0) and
      (WideMessageDlg(WideFormat(SPxPropEdConfirmDelete,
      [GetListMaskValue(CB_Masks.ItemIndex)]),
      mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
    begin
      I := ItemIndex;
      Items.Delete(ItemIndex);
      if I >= Items.Count then I := Items.Count - 1;
      ItemIndex := I;
      CB_MasksChange(Self);
    end;
end;

procedure TdlgPxMaskEditor.BB_AddClick(Sender: TObject);
var
  S: WideString;

  procedure SaveMask;
  begin
    CB_Masks.Items.Values[S] := E_Mask.Text;
    ListModified := True;
  end;
begin
  S := '';
  while WideInputQuery(WideFormat(SPxPropEdStore, [E_Mask.Text]), '', S) do
    if (S = '') then
    begin
      if (WideMessageDlg(SPxPropEdNeedDesc,
        mtError, [mbOK, mbCancel], 0) <> mrOK) then Break;
    end
    else
    begin
      if (CB_Masks.Items.Values[S] <> '') then
        case WideMessageDlg(WideFormat(SPxPropEdOverwrite,
          [S]),
          mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
          mrYes: ;
          mrNo: Continue;
        else
          Break;
        end;
      CB_Masks.Items.Values[S] := E_Test.PxMask.Mask;
      ListModified := True;
      Break;
    end;
end;

procedure TdlgPxMaskEditor.CB_AutoFillClick(Sender: TObject);
begin
  E_Test.PxMask.AutoFill := CB_AutoFill.Checked;
  FPxMask.AutoFill := CB_AutoFill.Checked;
end;

procedure TdlgPxMaskEditor.CB_AutoValidateClick(Sender: TObject);
begin
  FPxMask.AutoValidate := CB_AutoValidate.Checked;
end;

procedure TdlgPxMaskEditor.CB_RaiseInputErrorClick(Sender: TObject);
begin
  FPxMask.RaiseInputError := CB_RaiseInputError.Checked;
end;

procedure TdlgPxMaskEditor.E_ErrorMsgChange(Sender: TObject);
begin
  E_Test.PxMask.ErrorMessage := E_ErrorMsg.Text;
end;

procedure TdlgPxMaskEditor.E_TestValidChanged(Sender: TControlPxMask;
  EditControl: TWinControl; TextValid: Boolean);
begin
  if TextValid then
    TdlgPxMaskEditor(EditControl).Font.Color := clBlack
  else
    TdlgPxMaskEditor(EditControl).Font.Color := clRed;
end;

procedure TdlgPxMaskEditor.FormActivate(Sender: TObject);
begin
  OldBeepOnPxError := BeepOnPxError;
  OldBeepOnPxInputError := BeepOnPxInputError;
  BeepOnPxError := False;
  BeepOnPxInputError := False;
  E_Test.PxMask.AutoValidate := False;
  E_Test.PxMask.RaiseInputError := True;
  FOldHandler := Application.OnException;
  Application.OnException := AppEx;
end {FormActivate};

procedure TdlgPxMaskEditor.FormCreate(Sender: TObject);
begin
  OpenDialog.Title := RS_dnPxProp_OpenDialog_Title;
  OpenDialog.Filter := RS_dnPxProp_OpenDialog_Filter;
  BB_Restore.Hint := RS_dnPxProp_BB_Restore_Hint;
  BB_LoadMask.Hint := RS_dnPxProp_BB_LoadMask_Hint;
  BB_Delete.Hint := RS_dnPxProp_BB_Delete_Hint;
  BB_Add.Hint := RS_dnPxProp_BB_Add_Hint;
  BB_Use.Hint := RS_dnPxProp_BB_Use_Hint;
end {FormCreate};

procedure TdlgPxMaskEditor.FormDeactivate(Sender: TObject);
begin
  BeepOnPxError := OldBeepOnPxError;
  BeepOnPxInputError := OldBeepOnPxInputError;
  Application.OnException := FOldHandler;
end;

procedure TdlgPxMaskEditor.FormDestroy(Sender: TObject);
begin
  if ListModified then
    CB_Masks.Items.SaveToFile(ReadMaskFileName);
end;

procedure TdlgPxMaskEditor.GB_ExampleMasksEnter(Sender: TObject);
begin
  BB_Delete.Enabled := (CB_Masks.ItemIndex >= 0);
end;

procedure TdlgPxMaskEditor.GB_ExampleMasksExit(Sender: TObject);
begin
  BB_Delete.Enabled := False;
end;

{-PRIVATE----------}

procedure TdlgPxMaskEditor.AppEx(Sender: TObject; E: Exception);
begin
  if E is EPxMaskError then
    Memo.Text := E.Message
  else
    Application.ShowException(E);
end;

function TdlgPxMaskEditor.GetPxMask: TTntPxMask;
begin
  FPxMask.Mask := E_Test.PxMask.Mask;
  FPxMask.AutoFill := CB_AutoFill.Checked;
  FPxMask.AutoValidate := CB_AutoValidate.Checked;
  FPxMask.RaiseInputError := CB_RaiseInputError.Checked;
  FPxMask.ErrorMessage := E_Test.PxMask.ErrorMessage;
  Result := FPxMask;
end;

procedure TdlgPxMaskEditor.SetPxMask(Value: TTntPxMask);
begin
  FPxMask.assign(Value);
  FPxMask.AutoValidate := False;
  FPxMask.RaiseInputError := False;

  E_Test.PxMask.Mask := Value.Mask;
  E_Test.PxMask.ErrorMessage := Value.ErrorMessage;

  E_Mask.Text := Value.Mask;
  E_ErrorMsg.Text := Value.ErrorMessage;
  E_Test.Text := '';

  BB_Restore.Enabled := False;
  CB_AutoFill.Checked := Value.AutoFill;
  CB_AutoValidate.Checked := Value.AutoValidate;
  CB_RaiseInputError.Checked := Value.RaiseInputError;
end;

{-PUBLIC-----------}

constructor TdlgPxMaskEditor.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FPxMask := TTntPxMask.Create(Self);
end;

destructor TdlgPxMaskEditor.Destroy;
begin
  FPxMask.Free;
  inherited Destroy;
end;

{--------------------------------------}

end.

