unit TntdnUtil;
(*****************************************************************************
some internal Utility routines
******************************************************************************)

{$I JVCL.inc}

interface
uses
  TypInfo, {$IFDEF DELPHI6_UP} Variants, {$ENDIF} classes;

const
  StringProps = [tkString, tkLString{$IFDEF DELPHI3_UP}, tkWString{$ENDIF}];
  OrdProps = [tkInteger, tkChar, tkEnumeration, tkSet, tkClass];
  ClassProp = [tkClass];
  BoolProp = [tkEnumeration];
  FloatProps = [tkFloat];
  AnyProps = [low(TTypeKind)..High(TTypeKind)] - [low(TTypeKind)];
{ AnyProps = [Succ(low(TTypeKind))..High(TTypeKind)]; <-- this caused some strange compiler error}

function HasProperty( Instance : TObject;
                const PropName : WideString;
                      PropKind : TTypeKinds;
                  var PropInfo : PPropInfo) : Boolean;

function GetBoolProperty( Instance : TPersistent;
                    const PropName : WideString;
                      var Value    : Boolean) : Boolean;

function GetOrdProperty( Instance : TPersistent;
                   const PropName : WideString;
                         PropKind : TTypeKind;
                     var Value    : Longint) : Boolean;

function GetStringProperty( Instance : TPersistent;
                      const PropName : WideString;
                        var Value    : WideString) : Boolean;

procedure FakeInheritance(Instance: TObject; FakeClassParent: TClass);

implementation

uses
  SysUtils;

function HasProperty( Instance : TObject;
                const PropName : WideString;
                      PropKind : TTypeKinds;
                  var PropInfo : PPropInfo) : Boolean;
begin
  Result := False;
  if Assigned(Instance) then
  begin
    PropInfo := GetPropInfo(Instance.ClassInfo, PropName);
    Result := (PropInfo <> nil) and (PropInfo^.PropType^.Kind in PropKind);
  end
  else
    PropInfo := nil;
end;

function GetBoolProperty( Instance : TPersistent;
                    const PropName : WideString;
                      var Value    : Boolean) : Boolean;
var
  PropInfo: PPropInfo;
begin
  Result := False;
  Value := False;
  if not HasProperty(Instance, PropName, BoolProp, PropInfo) then Exit;
  Value := Boolean(GetOrdProp(Instance, PropInfo));
  Result := True;
end;

function GetStringProperty( Instance : TPersistent;
                      const PropName : WideString;
                        var Value    : WideString) : Boolean;
var
  PropInfo: PPropInfo;
begin
  Result := False;
  Value := '';
  if not HasProperty(Instance, PropName, StringProps, PropInfo) then Exit;
  Value := (GetStrProp(Instance, PropInfo));
  Result := True;
end;


function GetOrdProperty( Instance : TPersistent;
                   const PropName : WideString;
                         PropKind : TTypeKind;
                     var Value    : Longint) : Boolean;
var
  PropInfo: PPropInfo;
begin
  Result := False;
  Value := 0;
  if not HasProperty(Instance, PropName, [PropKind], PropInfo) then Exit;
  Value := GetOrdProp(Instance, PropInfo);
  Result := True;
end;

function GetProperty( Instance : TPersistent;
                const PropName : WideString;
                      PropKinds: TTypeKinds;
                  var Value    : variant) : Boolean;
var
  PropInfo : PPropInfo;
begin
  VarClear(Value);
  if Assigned(Instance) then
  try
    PropInfo := GetPropInfo(Instance.ClassInfo, PropName);
    if (PropInfo <> nil) and (PropInfo^.PropType^.Kind in PropKinds) then
    case PropInfo^.PropType^.Kind of
      tkInteger, tkChar, tkEnumeration, tkSet, tkClass:
        Value := GetOrdProp(Instance, PropInfo);
      tkFloat:
        Value := GetFloatProp(Instance, PropInfo);
      tkString, tkLString{$IFDEF DELPHI3_UP}, tkWString{$ENDIF}:
        Value := GetStrProp(Instance, PropInfo);
      tkVariant:
        Value := GetVariantProp(Instance, PropInfo);
    end;
  except
  end;
  Result := not VarIsEmpty(Value);
end;

function SetProperty( Instance : TPersistent;
                const PropName : WideString;
                      PropKinds: TTypeKinds;
                      Value    : variant) : Boolean;
var
  PropInfo : PPropInfo;
  S: WideString;
begin
  Result := False;
  if Assigned(Instance) then
  begin
    PropInfo := GetPropInfo(Instance.ClassInfo, PropName);
    if (PropInfo <> nil) and (PropInfo^.PropType^.Kind in PropKinds) then
    case PropInfo^.PropType^.Kind of
      tkInteger, tkChar, tkEnumeration, tkSet, tkClass:
        SetOrdProp(Instance, PropInfo, Value);
      tkFloat:
        SetFloatProp(Instance, PropInfo, Value);
      tkString, tkLString{$IFDEF DELPHI3_UP}, tkWString{$ENDIF}: begin
        S := Value; {New for Delphi5}
        SetStrProp(Instance, PropInfo, S);
        end;
      tkVariant:
        SetVariantProp(Instance, PropInfo, Value);
    else
      Exit;
    end;
    Result := True;
  end;
end;

procedure FakeInheritance(Instance: TObject; FakeClassParent: TClass);
begin
  if Instance.ClassType <> FakeClassParent.ClassParent then
      Exception.createFmt('%s is not the ClassParent of %s',
                          [FakeClassParent.ClassName, Instance.ClassName]);
  asm
    MOV     ECX, FakeClassParent
    MOV     EAX, Instance
    MOV     [EAX], ECX            //Replace Instance'S VMT-pointer
  end;
end;


end.
