{---------------------------------------------------------------------

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 05/19/2006 version

----------------------------------------------------------------------}

unit TntRZNE0;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TntStdCtrls, TntStdCtrls2,
  GrTypFun;

resourcestring
  RS_RZNE0_NoTextClipboard = 'No text in clipboard';
  RS_RZNE0_Value = 'Value';
  RS_RZNE0_MustBe = '%s must be ';
  RS_RZNE0_ValueMustBeInRange = 'in range from %s to %s';
  RS_RZNE0_ValueMustBeLess = 'lesser or equal than %s';
  RS_RZNE0_ValueMustBeMore = 'bigger or equal than %s';
  RS_RZNE0_InvalidInteger = '''%s'' is not an integer number';
  RS_RZNE0_InvalidFloat = '''%s'' is not a real number';
  RS_MB_Edit = '&Edit';
  RS_MB_Continue = '&Continue';
  RS_MB_Validate = '&Validate';

var
  VRealNul: TVReal;
  VLongIntNul: TVLongInt;

procedure CheckValueLongint (var NewValue: Longint;
  FMinValue, FMaxValue: TVLongint);
procedure CheckValueReal (var NewValue: Real;
  FMinValue, FMaxValue: TVReal);
function LongintValueValid (Value: Longint;
  MinValue, MaxValue: TVLongint): Boolean;
function RealValueValid (Value: Real;
  MinValue, MaxValue: TVReal): Boolean;
function SetOfRealChars: WideString;
function SetOfIntegerChars (FDecimalsAccept: Boolean): WideString;
function SetOfCardinalChars (FDecimalsAccept: Boolean): WideString;
function StrToInt0 (S: string;  out Res: LongInt): Boolean;
function StrToFloat0 (S: string;  out Res: Real): Boolean;
function StrToFloatLim (S: string;  out Res: Real): Boolean;
function StrToVValue (const Text: string;  DecimalsAccept: Boolean): TVLongint;

type
  TKeyChar = (NumberChar, BackspaceChar, UnknownChar);
  TTntRZNEdit0 = class(TTntCustomEditA)
  private
    FAlignment: TAlignment;
    FFontColorNormal,
    FFontColorWarning: TColor;
    FPromptOnExit: Boolean;
    procedure SetAlignment (const AValue : TAlignment);
    procedure SetFontColorNormal(const Value: TColor);
    procedure WhenRangeError;
  protected
    FEditName: WideString;
    FFormatString, VText: string;
    FValidate: Boolean;
    procedure CMEnter (var message: TCMGotFocus); message CM_ENTER;
    procedure CMExit (var message: TCMExit); message CM_EXIT;
    procedure CreateParams (var Params: TCreateParams); override;
    procedure FormatValue; virtual; abstract;
    function  GetCMaxValue: string; virtual; abstract;
    function  GetCMinValue: string; virtual; abstract;
    function  IsValidChar (const Key: Char): Boolean; virtual;
    procedure KeyDown (var Key: Word; Shift: TShiftState); override;
    procedure KeyPress (var Key: Char); override;
    procedure RestoreNormalColors; virtual;
    function  MaxValueAssig: Boolean; virtual; abstract;
    function  MinValueAssig: Boolean; virtual; abstract;
    procedure MessageErrorAndPSetFocus (const S: WideString);
    procedure WhenError; virtual;
    function  WhenRangeErrorStr: WideString;
    procedure SetFontColorValid (Valid: Boolean);
    procedure SetMaxValue (const V: string); virtual; abstract;
    procedure SetMinValue (const V: string); virtual; abstract;
    procedure SetValidate (B: Boolean); virtual; abstract;
    procedure SetValueValid; virtual; abstract;
    function  SyntaxOk: Boolean; virtual; abstract;
    function  Valid: Boolean; virtual;
    function  ValidNoMess: Boolean; virtual;
    function  ValidChars: WideString;  virtual; abstract;
  public
    constructor Create (AOwner: TComponent); override;
    procedure ClearMaxValue;
    procedure ClearMinValue;
    procedure EditNameFromLabel (L: TTntLabel);
    function  SetStrValueDirect (const S: string): Boolean;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment default taRightJustify;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property Ctl3D;
    property Color;
    property Cursor;
    property DragCursor;
    property DragMode;
    property EditName: WideString read FEditName write FEditName;
    property Enabled;
    property Font;
    property FontColorNormal: TColor read FFontColorNormal write SetFontColorNormal default clWindowText;
    property FontColorWarning: TColor read FFontColorWarning write FFontColorWarning default clRed;
    property Height;
    property HelpContext;
    property Hint;
    property Left;
    property MaxLength;
    property MaxValue: string read GetCMaxValue write SetMaxValue;
    property MinValue: string read GetCMinValue write SetMinValue;
    property Name;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property PromptOnExit: Boolean read FPromptOnExit write FPromptOnExit default False;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Tag;
    property Validate: Boolean read FValidate write SetValidate default False;
    property Visible;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  TTntRZNEdit = class(TTntRZNEdit0)
  protected
    FFormatString: string;
    procedure SetFormatString (const A: string); virtual; abstract;
  published
    property FormatString: string read  FFormatString write SetFormatString;
  end;

  TTntRZNNumber = class (TTntRZNEdit0)
  private
    FNotFormat: Boolean;
    procedure SetWithThousandSeparator(const Value: Boolean);
  protected
    FWithThousandSeparator: Boolean;
    procedure SetFormatString; virtual; abstract;
    function FFormatString: string; virtual; abstract;
  public
    constructor Create (AOwner: TComponent); override;
  published
    property NotFormat: Boolean read FNotFormat write FNotFormat default False;
    property WithThousandSeparator: Boolean read FWithThousandSeparator write SetWithThousandSeparator default True;
  end;

implementation

uses
  WinProcs, Math,
  TntClipbrd,
  JvConsts,
  TntJvJCLUtils,
  TntStrUtil0, StrNumU, TntMessUsual, TntMessOther;

const
  DigitChars = '0123456789';

{--------------------------------------}

procedure CheckValueLongint (var NewValue: Longint;
  FMinValue, FMaxValue: TVLongint);
begin
  if  FMinValue.Valid
  then  NewValue := Max (NewValue, FMinValue.Num);
  if  FMaxValue.Valid
  then  NewValue := Min (NewValue, FMaxValue.Num);
end {CheckValueLongint};

procedure CheckValueReal (var NewValue: Real;
  FMinValue, FMaxValue: TVReal);
begin
  if  FMinValue.Valid
  then  NewValue := Max (NewValue, FMinValue.Num);
  if  FMaxValue.Valid
  then  NewValue := Min (NewValue, FMaxValue.Num);
end {CheckValueReal};

function LongintValueValid (Value: Longint;
  MinValue, MaxValue: TVLongint): Boolean;
begin
  Result := True;
  if  MinValue.Valid  then  begin
    if  Value < MinValue.Num
    then  Result := False;
  end;
  if  MaxValue.Valid  then begin
    if  Value > MaxValue.Num
    then  Result := False;
  end;
end {LongintValueValid};

function RealValueValid (Value: Real;
  MinValue, MaxValue: TVReal): Boolean;
begin
  Result := True;
  if  MinValue.Valid  then  begin
    if  Value < MinValue.Num
    then  Result := False;
  end;
  if  MaxValue.Valid  then begin
    if  Value > MaxValue.Num
    then  Result := False;
  end;
end {RealValueValid};

function SetOfRealChars: WideString;
begin
  Result := DigitChars + '+-Ee' + DecimalSeparator;
end {SetOfRealChars};

function SetOfIntegerChars (FDecimalsAccept: Boolean): WideString;
begin
  Result := DigitChars + '+-';
  if  FDecimalsAccept
  then  Result := Result + DecimalSeparator;
end {SetOfIntegerChars};

function SetOfCardinalChars (FDecimalsAccept: Boolean): WideString;
begin
  Result := DigitChars;
  if  FDecimalsAccept
  then  Result := Result + DecimalSeparator;
end {SetOfCardinalChars};

function StrToInt0 (S: string;  out Res: LongInt): Boolean;
begin
  Res := 0;
  Result := False;
  if  S = ''
  then  Exit;
  try
    Res := StrToInt (WideDelChars(S, WideChar(ThousandSeparator)));
    Result := True;
  except
    on EConvertError do begin
      WideMessageBoxWarning (WideFormat (RS_RZNE0_InvalidInteger, [S]));
      Result := False;
    end;
  end;
end {StrToInt0};

function StrToFloat0 (S: string;  out Res: Real): Boolean;
begin
  Res := 0;
  Result := False;
  if  S = ''
  then  Exit;
  try
    Res := StrToFloat (WideDelChars(S, WideChar(ThousandSeparator)));
    Result := True;
  except
    on EConvertError do begin
      WideMessageBoxWarning (WideFormat (RS_RZNE0_InvalidFloat, [S]));
      Result := False;
    end;
  end;
end {StrToFloat0};

function StrToFloatLim (S: string;  out Res: Real): Boolean;
var
  Posic: Byte;
begin
  Res := 0;
  Result := False;
  if  S = ''
  then  Exit;
  try
    Posic := Pos (ThousandSeparator, S);
    if  Posic <> 0
    then  S[Posic] := DecimalSeparator;
    Res := StrToFloat (S);
    Result := True;
  except
    on EConvertError do begin
      WideMessageBoxWarning (WideFormat (RS_RZNE0_InvalidFloat, [S]));
      Result := False;
    end;
  end;
end {StrToFloatLim};

function StrToVValue (const Text: string;  DecimalsAccept: Boolean): TVLongint;
var
  R: Real;
begin
  Result.Valid := False;
  if  Length(Text) = 0
  then
    Result.Num := 0
  else  begin
    if  DecimalsAccept and (Pos(DecimalSeparator,Text)<>0)  then  begin
      Result.Valid := StrToFloat0 (Text, R);
      Result.Num := Round (R);
    end
    else
      Result.Valid := StrToInt0 (Text, Result.Num);
  end;
end {StrToVValue};

{--------------------------------------}
{-PRIVATE----------}

procedure TTntRZNEdit0.SetAlignment(const AValue: TAlignment);
begin
  if  FAlignment <> AValue  then begin
    FAlignment := AValue;
    RecreateWnd;
  end;
end {SetAlignment};

procedure TTntRZNEdit0.SetFontColorNormal (const Value: TColor);
begin
  FFontColorNormal := Value;
  Font.Color := Value;
end {SetFontColorNormal};

procedure TTntRZNEdit0.WhenRangeError;
begin
  if FValidate and not FPromptOnExit
  then  SetValueValid
  else if FPromptOnExit
  then  MessageErrorAndPSetFocus (WhenRangeErrorStr)
end {WhenRangeError};

{-PROTECTED--------}

procedure TTntRZNEdit0.CMEnter (var message: TCMGotFocus);
begin
  Text := WideDelChars (Text, WideChar(ThousandSeparator));
  if AutoSelect and not (csLButtonDown in ControlState) then SelectAll;
  RestoreNormalColors;
  inherited;
end {CMEnter};

procedure TTntRZNEdit0.CMExit (var message: TCMExit);
begin
  FormatValue;
  if  not ValidNoMess
  then  WhenError;
  inherited;
end {CMExit};

procedure TTntRZNEdit0.CreateParams (var Params: TCreateParams);
const
  Alignments: array[TAlignment] of Word = (ES_LEFT, ES_RIGHT, ES_CENTER);
begin
  inherited;
  Params.Style := Params.Style or ES_MULTILINE or Alignments[Alignment];
end {CreateParams};

function TTntRZNEdit0.IsValidChar (const Key: Char): Boolean;
begin
  if  Pos (Key, ValidChars) <> 0  then  begin
    VText := Copy (Text, 1, SelStart) + Key +
             Copy (Text, SelStart+1+SelLength, 255);
    Result := SyntaxOk;
  end
  else if  Key = Backspace  then  begin
    if  SelLength = 0  then
      VText := Copy (Text, 1, SelStart-1) +
               Copy (Text, SelStart+1, 255)
    else
      VText := Copy (Text, 1, SelStart) +
               Copy (Text, SelStart+1+SelLength, 255);
      Result := SyntaxOk;
  end
  else
    Result := False;
end {IsValidChar};

procedure TTntRZNEdit0.KeyDown (var Key: Word; Shift: TShiftState);
begin
  if ssCtrl in Shift then begin
    case Key of
    88: { Ctrl X }
       if SelLength <> 0 then begin
         VText := Copy (Text, 1, SelStart) +
                  Copy (Text, SelStart+1+SelLength, 255);
         if SyntaxOk then begin
           TntClipboard.AsText := SelText;
           Key := VK_DELETE;
           KeyDown (Key, []);
         end
         else Key := 0;
       end;

    67: { Ctrl C }
       TntClipboard.AsText := SelText;

    86: { Ctrl V }
       if TntClipboard.HasFormat(CF_TEXT) then begin
         VText := Copy (Text, 1, SelStart) +
                  TntClipboard.AsText +
                  Copy (Text, SelStart+1, 255);
         if SyntaxOk then begin
           SelText := TntClipboard.AsText;
           Key := 0;
         end;
       end
       else
         WideMessageBoxInfo (RS_RZNE0_NoTextClipboard);

    else end;
  end
  else
    if Key = VK_DELETE then begin
      if SelLength = 0 then
        VText := Copy (Text, 1, SelStart) +
                 Copy (Text, SelStart+2, 255)
      else
        VText := Copy (Text, 1, SelStart) +
                 Copy (Text, SelStart+1+SelLength, 255);
      if not SyntaxOk then Key := 0;
    end;
  inherited KeyDown (Key, Shift);
end {KeyDown};

procedure TTntRZNEdit0.KeyPress (var Key: Char);
begin
  if  IsValidChar (Key)
  then  inherited KeyPress (Key)
  else  Key := #0;
end {KeyPress};

procedure TTntRZNEdit0.MessageErrorAndPSetFocus (const S: WideString);
var
  Res: Integer;
begin
  if FValidate then  begin
    Res := WideMessageXDlg (S, mtWarning,
     [Concat ('Yes ', RS_MB_Edit),
      Concat ('No ', RS_MB_Continue),
      Concat ('Cancel ', RS_MB_Validate)], 0);
    case Res of
    mrYes: SetFocus;
    mrCancel: SetValueValid;
    end;
  end
  else if  WideMessageXDlg (S, mtWarning,
   [Concat ('Yes ', RS_MB_Edit),
    Concat ('No ', RS_MB_Continue)], 0) = mrYes
  then  SetFocus;
end {MessageErrorAndPSetFocus};

procedure TTntRZNEdit0.RestoreNormalColors;
begin
  Font.Color := FFontColorNormal;
end {RestoreNormalColors};

procedure TTntRZNEdit0.SetFontColorValid (Valid: Boolean);
begin
  if Valid
  then  Font.Color := FFontColorNormal
  else  Font.Color := FFontColorWarning;
end {SetFontColorValid};

procedure TTntRZNEdit0.WhenError;
begin
  WhenRangeError;
end {WhenError};

function TTntRZNEdit0.WhenRangeErrorStr: WideString;
begin
  if  FEditName <> ''
  then  Result := FEditName
  else  Result := RS_RZNE0_Value;
  Result := WideFormat (RS_RZNE0_MustBe, [Result]);
  if MaxValueAssig and MinValueAssig
  then  Result := WideFormat (Concat (Result,RS_RZNE0_ValueMustBeInRange),
    [GetCMinValue, GetCMaxValue])
  else if  MaxValueAssig
  then  Result := WideFormat (Concat (Result,RS_RZNE0_ValueMustBeLess),
    [GetCMaxValue])
  else  Result := WideFormat (Concat (Result,RS_RZNE0_ValueMustBeMore),
    [GetCMinValue]);
end {WhenRangeErrorStr};

function TTntRZNEdit0.Valid: Boolean;
begin
  Result := True;
end {Valid};

function TTntRZNEdit0.ValidNoMess: Boolean;
begin
  Result := True;
end {ValidNoMess};

{-PUBLIC-----------}

procedure TTntRZNEdit0.ClearMaxValue;
begin
  SetMaxValue ('');
end {ClearMaxValue};

procedure TTntRZNEdit0.ClearMinValue;
begin
  SetMinValue ('');
end {ClearMinValue};

constructor TTntRZNEdit0.Create (AOwner: TComponent);
begin
  inherited;
  FFontColorNormal := clWindowText;
  FFontColorWarning := clRed;
  FAlignment := taRightJustify;
end {Create};

procedure TTntRZNEdit0.EditNameFromLabel (L: TTntLabel);
begin
  EditName := DelAmpersandW (L.Caption);
end {EditNameFromLabel};

function TTntRZNEdit0.SetStrValueDirect (const S: string): Boolean;
var
  IsValid: Boolean;
  OldText: string;
begin
  OldText := Text;
  Text := S;
  IsValid := Valid;
  if  not IsValid
  then  Text := OldText;
  SetStrValueDirect := IsValid;
end {SetStrValueDirect};

{--------------------------------------}
{-PRIVATE----------}

procedure TTntRZNNumber.SetWithThousandSeparator(const Value: Boolean);
begin
  FWithThousandSeparator := Value;
  SetFormatString;
end {SetWithThousandSeparator};

{-PUBLIC-----------}

constructor TTntRZNNumber.Create (AOwner: TComponent);
begin
  inherited;
  WithThousandSeparator := True;
  Width := 60;
end {Create};

{--------------------------------------}

begin
  FillChar (VRealNul, SizeOf(VRealNul), #0);
  FillChar (VLongIntNul, SizeOf(VLongIntNul), #0);
end.
