{---------------------------------------------------------------------

  TntRZNEdit2, a component with edit controls for
    real numbers      -> TTntEditVReal
    integers          -> TTntEditVInteger
    positive integers -> TTntEditVCardinal

  This is my version of a numeric input component. It comes with three
  types, each handling its special input range ( real, integer and
  cardinal ).
  All input is watched, a TTntEditVInteger accepts only characters in
  the set [-,0,1,..,9] for instance.
  The clipboard is supported.
  An empty input is NOT interpreted as 0.
  For TTntEditVReal and TTntEditVInteger an input containing only a plus-sign
  or a minus-sign is also regarded as 0.
  Ancestor class is the TTntCustomEdit, new properties are:
    FormatString: is used in the FormatFloat-routine from Delphi
    MaxValue \
             : determine the input boundaries, if Validate is True  
    MinValue /
    Validate : determines if input is checked against boundaries
    Value    : the result 
  modified from
  Robert Kratz                                  kratz@labs.polycnrs-gre.fr

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es

----------------------------------------------------------------------}

unit TntRZNEdit2;

interface

uses
  WinTypes, Classes, StdCtrls, ExtCtrls, Controls, SysUtils, Graphics,
  GrTypFun, TntRZNE0;

resourcestring
  RS_RZNEdit2_CurrentFocused = 'Field/input where there is cursor';
  RS_RZNEdit2_EmptyValue = '%s must have any value';

type
  TTntEditV = class (TTntRZNNumber)
  private
    FColorNormal,
    FColorWarning: TColor;
    FRequired: Boolean;
    procedure SetColorNormal(const Value: TColor);
    procedure SetRequired(const Value: Boolean);
  protected
    procedure RestoreNormalColors; override;
    function  WhenEmptyErrorStr: WideString;
    procedure WhenError; override;
  public
    constructor Create (AOwner: TComponent); override;
    function TextStored: Boolean; virtual;
    function Valid: Boolean; override;
    function ValidNoMess: Boolean; override;
  published
    property ColorNormal: TColor read FColorNormal write SetColorNormal default clWindow;
    property ColorWarning: TColor read FColorWarning write FColorWarning default $00AEBFF8 {Clear Red};
    property Required: Boolean read FRequired write SetRequired default False;
  end;

  TTntEditVReal = class(TTntEditV)
  private
    FDecimalPlaces: Byte;
    FMinValue, FMaxValue: TVReal;
    function  CheckValidValue (NewValue: TVReal): Boolean;
    function  CheckValue (NewValue: Real): Real;
    procedure SetDecimalPlaces(const Value: Byte);
    function  EmptyOrSign: Boolean;
    function  GetNMax: Real;
    procedure SetNMax (R: Real);
    function  GetNMin: Real;
    procedure SetNMin (R: Real);
    function  GetValue: Real;
    procedure SetValue (NewValue: Real);
    function  GetVValue: TVReal;
    procedure SetVValue (NewValue: TVReal);
  protected {for no compiler Hint message}
    procedure FormatValue; override;
    function  FFormatString: string; override;
    function  GetCMaxValue: string; override;
    function  GetCMinValue: string; override;
    procedure SetFormatString; override;
    procedure SetMaxValue (const V: string); override;
    procedure SetMinValue (const V: string); override;
    function  MaxValueAssig: Boolean; override;
    function  MinValueAssig: Boolean; override;
    procedure SetValidate (B: Boolean); override;
    procedure SetValueValid; override;
    function  SyntaxOk: Boolean; override;
    function  ValidChars: WideString; override;
  protected
    function  GetLimValue (Lim: TVReal): string;
    procedure SetLimValue (const V: string;  var Lim: TVReal);
  public
    constructor Create (AOwner: TComponent); override;
    function  ValidNoMess: Boolean; override;
    property  NMax: Real read GetNMax write SetNMax;
    property  NMin: Real read GetNMin write SetNMin;
    property  NVMax: TVReal read FMaxValue write FMaxValue;
    property  NVMin: TVReal read FMinValue write FMinValue;
    property  Value: Real read GetValue write SetValue;
    property  VValue: TVReal read GetVValue write SetVValue;
  published
    property DecimalPlaces: Byte read FDecimalPlaces write SetDecimalPlaces;
  end;

  TTntEditVInteger = class(TTntEditV)
  private
    FDecimalsAccept: Boolean;
    FMinValue, FMaxValue: TVLongint;
    function  CheckValidValue (NewValue: TVLongint): Boolean;
    function  CheckValue (NewValue: Longint): Longint;
    function  EmptyOrSign: Boolean;
    function  GetNMax: Longint;
    procedure SetNMax (R: Longint);
    function  GetNMin: Longint;
    procedure SetNMin (R: Longint);
    function  GetValue: Longint;
    procedure SetValue (NewValue: Longint);
    function  GetVValue: TVLongint;
    procedure SetVValue (NewValue: TVLongint);
  protected {for no compiler Hint message}
    procedure FormatValue; override;
    function  FFormatString: string; override;
    function  GetCMaxValue: string; override;
    function  GetCMinValue: string; override;
    procedure SetFormatString; override;
    procedure SetMaxValue (const V: string); override;
    procedure SetMinValue (const V: string); override;
    function  MaxValueAssig: Boolean; override;
    function  MinValueAssig: Boolean; override;
    function  SyntaxOk: Boolean; override;
    procedure SetValidate (B:  Boolean); override;
    procedure SetValueValid; override;
    function  ValidChars: WideString; override;
  protected
    function  GetLimValue (Lim: TVLongint): string;
    procedure SetLimValue (const V: string;  var Lim: TVLongint);
  public
    constructor Create (AOwner: TComponent); override;
    function  ValidNoMess: Boolean; override;
    property  NMax: Longint read GetNMax write SetNMax;
    property  NMin: Longint read GetNMin write SetNMin;
    property  NVMax: TVLongint read FMaxValue write FMaxValue;
    property  NVMin: TVLongint read FMinValue write FMinValue;
    property  Value: Longint read GetValue write SetValue;
    property  VValue: TVLongint read GetVValue write SetVValue;
  published
    property DecimalsAccept: Boolean read FDecimalsAccept write FDecimalsAccept default False;
  end;

  TTntEditVCardinal = class(TTntEditVInteger)
  protected
    procedure SetLimValue (const V: string;  var Lim: TVLongint);
  protected {for no compiler Hint message}
    procedure SetMaxValue (const V: string); override;
    procedure SetMinValue (const V: string); override;
    function  ValidChars: WideString; override;
  end;

implementation

uses
  Math,
  StrNumU, TntMessUsual;

{--------------------------------------}
{-PRIVATE----------}

procedure TTntEditV.SetColorNormal (const Value: TColor);
begin
  FColorNormal := Value;
  Color := Value;
end {SetColorNormal};

procedure TTntEditV.SetRequired (const Value: Boolean);
begin
  FRequired := Value;
  if FRequired and (not TextStored)
  then  Color := FColorWarning
  else  Color := FColorNormal;
end {SetRequired};

{-PROTECTED--------}

procedure TTntEditV.RestoreNormalColors;
begin
  inherited;
  Color := FColorNormal;
end {RestoreNormalColors};

function TTntEditV.WhenEmptyErrorStr: WideString;
begin
  if  FEditName <> ''
  then  Result := EditName
  else  Result := RS_RZNEdit2_CurrentFocused;
  Result := WideFormat (RS_RZNEdit2_EmptyValue, [Result]);
end {WhenEmptyErrorStr};

procedure TTntEditV.WhenError;
begin
  if FValidate and not PromptOnExit
  then  SetValueValid
  else if PromptOnExit then  begin
    if FRequired and (not TextStored)
    then  MessageErrorAndPSetFocus (WhenEmptyErrorStr)
    else  MessageErrorAndPSetFocus (WhenRangeErrorStr);
  end;
end {WhenError};

{-PUBLIC-----------}

constructor TTntEditV.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle - [csSetCaption];
  FColorNormal := clWindow;
  FColorWarning := $00AEBFF8 {Clear Red};
end {Create};

function TTntEditV.TextStored: Boolean;
begin
  Result := Text <> '';
end {TextStored};

function TTntEditV.Valid: Boolean;
begin
  Result := ValidNoMess;
  if  not Result  then  begin
    if FRequired and (not TextStored)
    then  WideMessageBoxWarning (WhenEmptyErrorStr)
    else  WideMessageBoxWarning (WhenRangeErrorStr);
    SetFocus;
  end;
end {Valid};

function TTntEditV.ValidNoMess: Boolean;
begin
  Result := True;
  if FRequired and (not TextStored)  then  begin
    Result := False;
    Color := FColorWarning;
  end
  else
    Color := FColorNormal;
end {ValidNoMess};

{--------------------------------------}

function TTntEditVReal.CheckValidValue (NewValue: TVReal): Boolean;
begin
  if  NewValue.Valid
  then  Result := RealValueValid (NewValue.Num, FMinValue, FMaxValue)
  else  Result := False;
end {CheckValidValue};

function TTntEditVReal.CheckValue (NewValue: Real): Real;
begin
  if FValidate
  then  CheckValueReal (NewValue, FMinValue, FMaxValue);
  Result := NewValue;
end {CheckValue};

procedure TTntEditVReal.SetDecimalPlaces (const Value: Byte);
begin
  if  Value <> FDecimalPlaces  then  begin
    FDecimalPlaces := Value;
    FormatValue;
  end;
end {SetDecimalPlaces};

function TTntEditVReal.EmptyOrSign: Boolean;
begin
  Result := (VText='') or (VText='-') or (VText='+');
end {EmptyOrSign};

function TTntEditVReal.GetNMax: Real;
begin
  Result := GetFromVReal (FMaxValue);
end {GetNMax};

procedure TTntEditVReal.SetNMax (R: Real);
begin
  SetToVReal (FMaxValue, R);
  ValidNoMess;
end {SetNMax};

function TTntEditVReal.GetNMin: Real;
begin
  Result := GetFromVReal (FMinValue);
end {GetNMin};

procedure TTntEditVReal.SetNMin (R: Real);
begin
  SetToVReal (FMinValue, R);
  ValidNoMess;
end {SetNMin};

function TTntEditVReal.GetValue: Real;
begin
  Result := GetFromVReal (GetVValue);
end {GetValue};

procedure TTntEditVReal.SetValue (NewValue: Real);
var
  NV: TVReal;
begin
  SetToVReal (NV, NewValue);
  SetVValue (NV);
end {SetValue};

function TTntEditVReal.GetVValue: TVReal;
begin
  Result.Valid := StrToFloat0 (Text, Result.Num);
end {GetVValue};

procedure TTntEditVReal.SetVValue (NewValue: TVReal);
begin
  if  not NewValue.Valid and FRequired and FValidate
  then  NewValue.Valid := True;
  if  NewValue.Valid
  then  NewValue.Num := CheckValue (NewValue.Num);
  Text := GetLimValue (NewValue);
  ValidNoMess;
end {SetVValue};

{-PROTECTED--------}

procedure TTntEditVReal.FormatValue;
begin
  if  VValue.Valid  then  begin
    if  not NotFormat
    then  Text := FormatFloat (FFormatString, VValue.Num)
  end
  else
    Text := '';
end {FormatValue};

function TTntEditVReal.FFormatString: string;
begin
  Result := FloatFormat (FWithThousandSeparator, FDecimalPlaces);
end {FFormatString};

function TTntEditVReal.GetCMaxValue: string;
begin
  Result := GetLimValue (FMaxValue);
end {GetCMaxValue};

function TTntEditVReal.GetCMinValue: string;
begin
  Result := GetLimValue (FMinValue);
end {GetCMinValue};

procedure TTntEditVReal.SetFormatString;
begin
  SetVValue (VValue);
end {SetFormatString};

procedure TTntEditVReal.SetMaxValue (const V: string);
begin
  SetLimValue (V, FMaxValue);
end {SetMaxValue};

procedure TTntEditVReal.SetMinValue (const V: string);
begin
  SetLimValue (V, FMinValue);
end {SetMinValue};

function TTntEditVReal.MaxValueAssig: Boolean;
begin
  MaxValueAssig := FMaxValue.Valid;
end {MaxValueAssig};

function TTntEditVReal.MinValueAssig: Boolean;
begin
  MinValueAssig := FMinValue.Valid;
end {MinValueAssig};

procedure TTntEditVReal.SetValidate (B:  Boolean);
begin
  FValidate := B;
  SetVValue (VValue);
end {SetValidate};

procedure TTntEditVReal.SetValueValid;
begin
  SetVValue (VValue);
end {SetValueValid};

function TTntEditVReal.SyntaxOk: Boolean;
begin
  try
    {-----------------------------------------------------------
      Syntax correct if transformation causes no exception.
      if the input is empty or a sign ( VText = '', '+', '-' ),
      it is interpreted as 0
     -----------------------------------------------------------}
    if not EmptyOrSign then StrToFloat (VText);
    Result := True;
  except
    on EConvertError do Result := False;
  end;
end {SyntaxOk};

function TTntEditVReal.ValidChars: WideString;
begin
  ValidChars := SetOfRealChars;
end {ValidChars};

{-PROTECTED--------}

function TTntEditVReal.GetLimValue (Lim: TVReal): string;
begin
  if  Lim.Valid
  then  Result := FloatToStr (Lim.Num)
  else  Result := '';
end {GetLimValue};

procedure TTntEditVReal.SetLimValue (const V: string;  var Lim: TVReal);
begin
  Lim.Valid := StrToFloatLim (V, Lim.Num);
  ValidNoMess;
end {SetLimValue};

{-PUBLIC-----------}

constructor TTntEditVReal.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  CharCase := ecUpperCase;
  FMinValue := VRealNul;
  FMaxValue := VRealNul;
  SetVValue (VRealNul);
end;

function TTntEditVReal.ValidNoMess: Boolean;
var
  IsValid: Boolean;
begin
  IsValid := inherited ValidNoMess;
  if IsValid and TextStored then  begin
    IsValid := CheckValidValue(VValue);
    SetFontColorValid (IsValid);
  end;
  Result := IsValid;
end {ValidNoMess};

{--------------------------------------}

function TTntEditVInteger.CheckValidValue (NewValue: TVLongint): Boolean;
begin
  if  NewValue.Valid
  then  Result := LongintValueValid (Value, FMinValue, FMaxValue)
  else  Result := False;
end {CheckValidValue};

function TTntEditVInteger.CheckValue (NewValue: Longint): Longint;
begin
  if FValidate
  then  CheckValueLongint (NewValue, FMinValue, FMaxValue);
  Result := NewValue;
end {CheckValue};

function TTntEditVInteger.EmptyOrSign: Boolean;
begin
  Result := (VText='') or (VText='-') or (VText='+');
end {EmptyOrSign};

function TTntEditVInteger.GetNMax: Longint;
begin
  Result := GetFromVLongint (FMaxValue);
end {GetNMax};

procedure TTntEditVInteger.SetNMax (R: Longint);
begin
  SetToVLongint (FMaxValue, R);
  ValidNoMess;
end {SetNMax};

function TTntEditVInteger.GetNMin: Longint;
begin
  Result := GetFromVLongint (FMinValue);
end {GetNMin};

procedure TTntEditVInteger.SetNMin (R: Longint);
begin
  SetToVLongint (FMinValue, R);
  ValidNoMess;
end {SetNMin};

function TTntEditVInteger.GetValue: Integer;
begin
  Result := GetFromVLongint (GetVValue);
end {GetValue};

procedure TTntEditVInteger.SetValue (NewValue: Integer);
var
  NV: TVLongInt;
begin
  SetToVLongint (NV, NewValue);
  SetVValue (NV);
end {SetValue};

function TTntEditVInteger.GetVValue: TVLongint;
begin
  Result := StrToVValue (Text, FDecimalsAccept);
end {GetVValue};

procedure TTntEditVInteger.SetVValue (NewValue: TVLongint);
begin
  if  not NewValue.Valid and FRequired and FValidate
  then  NewValue.Valid := True;
  if  NewValue.Valid
  then  NewValue.Num := CheckValue (NewValue.Num);
  Text := GetLimValue (NewValue);
  ValidNoMess;
end {SetVValue};

{-PROTECTED--------}

procedure TTntEditVInteger.FormatValue;
begin
  if  VValue.Valid  then  begin
    if  not NotFormat
    then  Text := FormatFloat (FFormatString, VValue.Num)
  end
  else
    Text := '';
end {FormatValue};

function TTntEditVInteger.FFormatString: string;
begin
  Result := IntegerFormat (FWithThousandSeparator);
end {FFormatString};

function TTntEditVInteger.GetCMaxValue: string;
begin
  Result := GetLimValue (FMaxValue);
end {GetCMaxValue};

function TTntEditVInteger.GetCMinValue: string;
begin
  Result := GetLimValue (FMinValue);
end {GetCMinValue};

procedure TTntEditVInteger.SetFormatString;
begin
  SetVValue (VValue);
end {SetFormatString};

procedure TTntEditVInteger.SetMaxValue (const V: string);
begin
  SetLimValue (V, FMaxValue);
end {SetMaxValue};

procedure TTntEditVInteger.SetMinValue (const V: string);
begin
  SetLimValue (V, FMinValue);
end {SetMinValue};

function TTntEditVInteger.MaxValueAssig: Boolean;
begin
  MaxValueAssig := FMaxValue.Valid;
end {MaxValueAssig};

function TTntEditVInteger.MinValueAssig: Boolean;
begin
  MinValueAssig := FMinValue.Valid;
end {MinValueAssig};

function TTntEditVInteger.SyntaxOk: Boolean;
begin
  try
    {-----------------------------------------------------------
      Syntax correct if transformation causes no exception.
      if the input is empty or a sign ( VText = '', '+', '-' ),
      it is interpreted as 0
     -----------------------------------------------------------}
    if not EmptyOrSign then begin
      if  FDecimalsAccept and (Pos(DecimalSeparator,VText)<>0)
      then  StrToFloat(VText)
      else  StrToInt(VText);
    end;
    Result := True;
  except
    on EConvertError do Result := False;
  end;
end {SyntaxOk};

procedure TTntEditVInteger.SetValidate (B:  Boolean);
begin
  FValidate := B;
  SetVValue (VValue);
end {SetValidate};

procedure TTntEditVInteger.SetValueValid;
begin
  SetVValue (VValue);
end {SetValueValid};

function TTntEditVInteger.ValidChars: WideString;
begin
  ValidChars := SetOfIntegerChars (FDecimalsAccept);
end {ValidChars};

{-PROTECTED--------}

function TTntEditVInteger.GetLimValue (Lim: TVLongInt): string;
begin
  if  Lim.Valid
  then  Result := IntToStr (Lim.Num)
  else  Result := '';
end {GetLimValue};

procedure TTntEditVInteger.SetLimValue (const V: string;  var Lim: TVLongInt);
begin
  Lim.Valid := StrToInt0 (V, Lim.Num);
  ValidNoMess;
end {SetLimValue};

{-PUBLIC-----------}

constructor TTntEditVInteger.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FMinValue := VLongintNul;
  FMaxValue := VLongintNul;
  SetVValue (VLongintNul);
end {Create};

function TTntEditVInteger.ValidNoMess: Boolean;
var
  IsValid: Boolean;
begin
  IsValid := inherited ValidNoMess;
  if IsValid and TextStored then  begin
    IsValid := CheckValidValue(VValue);
    SetFontColorValid (IsValid);
  end;
  Result := IsValid;
end {ValidNoMess};

{--------------------------------------}
{-PRIVATE----------}

procedure TTntEditVCardinal.SetLimValue (const V: string;  var Lim: TVLongint);
begin
  Lim.Valid := StrToInt0 (V, Lim.Num);
  if  Lim.Valid
  then  Lim.Num := Max (Lim.Num, 0);
  ValidNoMess;
end {SetLimValue};

{-PROTECTED--------}

procedure TTntEditVCardinal.SetMaxValue (const V: string);
begin
  SetLimValue (V, FMaxValue);
end {SetMaxValue};

procedure TTntEditVCardinal.SetMinValue (const V: string);
begin
  SetLimValue (V, FMinValue);
end {SetMinValue};

function TTntEditVCardinal.ValidChars: WideString;
begin
  ValidChars := SetOfCardinalChars (FDecimalsAccept);
end {ValidChars};

{--------------------------------------}

end.

