{---------------------------------------------------------------------

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/01/2006 version
 05/27/2006 version: Add TTntResizeBtnArray
 08/16/2006 version: SetGBColor moved to VCtrls

----------------------------------------------------------------------}

unit TntSeveral;     

interface

uses
  Classes, Controls, StdCtrls, Buttons, Forms, Graphics, IniFiles,
  TntClasses, TntStdCtrls, TntMenus, TntButtons;

resourcestring
  RS_FontNotFound = 'Font %s not found, to use in %s';
  RS_Several_PromptSaveFile = '%s'#13#10'is modified, save it?';
  RS_Several_PromptSaveFiles = '%s'#13#10'are modified, save them?';
  RS_Several_Save = 'Save';
  RS_Several_SaveAs = 'Save as...';

procedure BasicGetFont (const Section: string;  Font: TFont;
  IniFile: TCustomIniFile;  SLFont: TTntStringList);
procedure BasicSetFont (const Section: string;  Font: TFont;
  IniFile: TCustomIniFile);
procedure GetSLFont (var SLFont: TTntStringList);
procedure LoadTrueTypeFonts (TS: TTntStrings);

function GetFontSizeFromHWFC (InitialSize, HeightC, WidthC: Integer;
  F: TFont;  const S: WideString): Integer;
procedure GetCaptionHeightWidth (F: TFont;  const S: WideString;
  var CaptHeight, CaptWidth: Integer);
function GetCaptionHeight (F: TFont;  const S: WideString): Integer;
function GetCaptionWidth (F: TFont;  const S: WideString): Integer;
function GetMainMenuWidth  (F: TFont;  MainMenu: TTntMainMenu): Integer;
function GetFontHeight (F: TFont): Integer;
function GetFontHeightMod (F: TFont;  const S: WideString): Integer;
function GetFontWidth (F: TFont): Integer;
function GetEditHeight (F: TFont): Integer;

function FilterFromExtW (const TitleFile, Ext: WideString): WideString;

type
  TTntResizeBtnArray = class (TObject)
  private
    FAlignment: TLeftRight;
    FBBSpacing: Word;
    FFitToMaxWidth: Boolean;
    FMinWidth: Word;
    FirstR: Word;
    procedure Resize (B: array of TButton);
  public
    constructor Create;
    procedure HoritzResize (B: array of TButton);
    procedure HoritzResizeB (B: array of TButton;  LtoR: Boolean);
    procedure HoritzResizeL (B: array of TButton);
    procedure HoritzResizeR (B: array of TButton);
    procedure HoritzResizeSpac (B: array of TButton;  Spacing: Word);
    procedure VertResize (B: array of TButton);
    procedure VertResizeB (B: array of TButton;  LtoR: Boolean);
    procedure VertResizeL (B: array of TButton);
    procedure VertResizeR (B: array of TButton);
    function  HoritzSize (B: array of TButton): Word;
  published
    property Alignment: TLeftRight read FAlignment write FAlignment default taLeftJustify;
    property BBSpacing: Word read FBBSpacing write FBBSpacing default 8;
    property FitToMaxWidth: Boolean read FFitToMaxWidth write FFitToMaxWidth default False;
    property MinWidth: Word read FMinWidth write FMinWidth default 25;
  end;

implementation

uses
  SysUtils, Windows, Math,
  Several0, VCtrls, TntMessUsual;

procedure BasicGetFont (const Section: string;  Font: TFont;
  IniFile: TCustomIniFile;  SLFont: TTntStringList);
var
  FN: TFontName;
begin
  Font.Color := IniFile.ReadInteger
   (Section, IdentFontColor, clWindowText);
  FN := IniFile.ReadString
   (Section, IdentFontName, 'MS Sans Serif');
  if  SLFont.IndexOf(WideUpperCase(FN)) < 0
  then  WideMessageBoxWarning (WideFormat (RS_FontNotFound, [FN, Section]))
  else  Font.Name := FN;
  Font.Size := IniFile.ReadInteger
   (Section, IdentFontSize, 8);
  Font.Style := [];
  if IniFile.ReadBool
   (Section, IdentfsBold, False)
  then  Font.Style := [fsBold];
  if IniFile.ReadBool
   (Section, IdentfsItalic, False)
  then  Font.Style := Font.Style + [fsItalic];
end {BasicGetFontGetFont};

procedure BasicSetFont (const Section: string;  Font: TFont;
  IniFile: TCustomIniFile);
begin
  IniFile.WriteInteger
   (Section, IdentFontColor, Font.Color);
  IniFile.WriteString
   (Section, IdentFontName, Font.Name);
  IniFile.WriteInteger
   (Section, IdentFontSize, Font.Size);
  IniFile.WriteBool
   (Section, IdentfsBold, fsBold in Font.Style);
  IniFile.WriteBool
   (Section, IdentfsItalic, fsItalic in Font.Style);
end {BasicSetFont};

procedure GetSLFont (var SLFont: TTntStringList);

function EnumFontsProc(var LogFont: TLogFontW; var TextMetric: TTextMetric;
  FontType: Integer; Data: Pointer): Integer; stdcall;
var
  S: TTntStrings;
  Temp: WideString;
begin
  S := TTntStrings(Data);
  Temp := LogFont.lfFaceName;
  if (S.Count = 0) or (WideCompareText(S[S.Count-1], Temp) <> 0) then
    S.Add(Temp);
  Result := 1;
end {EnumFontsProc};

var
  DC: HDC;
begin
  SLFont := TTntStringList.Create;
  DC := GetDC(0);
  try
    EnumFontFamiliesW(DC, nil, @EnumFontsProc, Longint(SLFont));
  finally
    ReleaseDC(0, DC);
  end;
end {GetSLFont};

procedure LoadTrueTypeFonts (TS: TTntStrings);

function EnumFontsProc(var LogFont: TLogFontW; var TextMetric: TTextMetric;
  FontType: Integer; Data: Pointer): Integer; stdcall;
var
  S: TTntStrings;
  Temp: WideString;
begin
  S := TTntStrings(Data);
  Temp := LogFont.lfFaceName;
  if ((S.Count = 0) or (WideCompareText(S[S.Count-1], Temp) <> 0)) and (FontType and TRUETYPE_FONTTYPE > 0) then
    S.Add(Temp);
  Result := 1;
end;

var
  DC: HDC;
  LFont: TLogFontW;
  FFonts: TTntStringList;
begin
  FFonts := TTntStringList.Create;
  DC := GetDC(0);
  try
    FillChar(LFont, sizeof(LFont), 0);
    LFont.lfCharset := DEFAULT_CHARSET;
    EnumFontFamiliesExW(DC, LFont, @EnumFontsProc, LongInt(FFonts), 0);
    TTntStringList(FFonts).Sorted := TRUE;
    TS.Clear;
    TS.AddStrings(FFonts);
  finally
    ReleaseDC(0, DC);
  end;
  FFonts.Free;
end {LoadTrueTypeFonts};


function GetFontSizeFromHWFC (InitialSize, HeightC, WidthC: Integer;
  F: TFont;  const S: WideString): Integer;
var
  T: TTntLabel;
begin
  if  (S = '') or (InitialSize < 0) or (HeightC < 0) or (WidthC < 0)
  then  Result := F.Size
  else  begin
    T := TTntLabel.Create (Application);
    with T do begin
      Font.Assign (F);
      Font.Size := InitialSize;
      Caption := S;
      while  Font.Size > 0  do  begin
        if  (T.Height <= HeightC) and (T.Width <= WidthC)
        then  Break
        else  Font.Size := Font.Size -1;
      end;
      Result := Font.Size;
      Free;
    end;
  end;
end {GetFontSizeFromHWFC};

procedure GetCaptionHeightWidth (F: TFont;  const S: WideString;
  var CaptHeight, CaptWidth: Integer);
var
  T: TTntLabel;
begin
  if  S = ''  then  begin
    CaptHeight := 0;
    CaptWidth := 0;
  end
  else  begin
    T := TTntLabel.Create (Application);
    with T do begin
      Font.Assign (F);
      Caption := S;
      CaptHeight := Height;
      CaptWidth := Width;
      Free;
    end;
  end;
end {GetCaptionHeightWidth};

function GetCaptionHeight (F: TFont;  const S: WideString): Integer;
var
  T: TTntLabel;
begin
  if  S = ''
  then  Result := 0
  else  begin
    T := TTntLabel.Create (Application);
    with T do begin
      Font.Assign (F);
      Caption := S;
      Result := Height;
      Free;
    end;
  end;
end {GetCaptionHeight};

function GetCaptionWidth (F: TFont;  const S: WideString): Integer;
var
  T: TTntLabel;
begin
  if  S = ''
  then  Result := 0
  else  begin
    T := TTntLabel.Create (Application);
    with T do begin
      Font.Assign (F);
      Caption := S;
      Result := Width;
      Free;
    end;
  end;
end {GetCaptionWidth};

function GetMainMenuWidth  (F: TFont;  MainMenu: TTntMainMenu): Integer;
var
  S: WideString;
  I: Integer;
begin
  S := 'b';
  for I := 0 to MainMenu.Items.Count-1  do
    S := Concat (S, MainMenu.Items[I].Caption, 'bb');
  Result := GetCaptionWidth (F, S) + 4;
end {GetMainMenuWidth};

function GetFontHeight (F: TFont): Integer;
var
  T: TLabel;
begin
  T := TLabel.Create (Application);
  with T do begin
    Font.Assign (F);
    Caption := 'Aj';
    Result := Height;
    Free;
  end;
end {GetFontHeight};

function GetFontHeightMod (F: TFont;  const S: WideString): Integer;
var
  I, J: Integer;
  LetterDown: Boolean;
begin
  I := GetFontHeight (F);
  LetterDown := False;
  for  J := 1 to length(S)  do
    if  Pos (S[J], 'jgqyp') <> 0  then  begin
      LetterDown := True;
      Break;
    end;
  if  not LetterDown
  then  I := Round (I * 0.86);
  Result := I;
end {GetFontHeightMod};

function GetFontWidth (F: TFont): Integer;
var
  T: TLabel;
begin
  T := TLabel.Create (Application);
  with T do begin
    Font.Assign (F);
    Caption := '00000000';
    Result := Round (Width/8);
    Free;
  end;
end {GetFontWidth};

function GetEditHeight (F: TFont): Integer;
var
  E: TEdit;
begin
  E := TEdit.Create (Application);
  with E do begin
    Font.Assign (F);
    Result := Height;
    Free;
  end;
end {GetEditHeight};

function FilterFromExtW (const TitleFile, Ext: WideString): WideString;
begin
  Result := Concat (TitleFile, ' (*.', Ext, ')|*.', Ext);
end;

{--------------------------------------}
{-PRIVATE----------}

procedure TTntResizeBtnArray.Resize (B: array of TButton);
var
  I: Integer;
  GlyphW: Word;
  TotalCaptW, W: Word;
begin
  if  FAlignment = taRightJustify
  then  FirstR := ControlRight (B[0])
  else  FirstR := 0;
  for  I := 0 to High(B)  do  begin
    if  B[I] is TTntBitBtn  then  begin
      with B[I] as TTntBitBtn do  begin
        GlyphW := Glyph.Width div NumGlyphs;
        TotalCaptW := GetCaptionWidth (Font, Caption);
        if  Layout in [blGlyphLeft, blGlyphRight]  then  begin
          if  GlyphW > 0
          then  W := TotalCaptW + Spacing + GlyphW
          else  W := TotalCaptW;
        end
        else
          W := Max (TotalCaptW, GlyphW);
        Width := Max (W + (Spacing * 4), FMinWidth);
      end;
    end
    else if  B[I] is TTntButton  then  begin
      with B[I] as TTntButton do  begin
        TotalCaptW := GetCaptionWidth (Font, Caption);
        Width := Max (TotalCaptW + 16, FMinWidth);
      end;
    end;
  end;
end {Resize};

{-PUBLIC-----------}

constructor TTntResizeBtnArray.Create;
begin
  FBBSpacing := 8;
  FMinWidth := 25;
end;

procedure TTntResizeBtnArray.HoritzResize (B: array of TButton);
begin
  HoritzResizeSpac (B, FBBSpacing);
end;

procedure TTntResizeBtnArray.HoritzResizeB (B: array of TButton;  LtoR: Boolean);
var
  PrevAlignment: TAlignment;
begin
  PrevAlignment := FAlignment;
  if LtoR
  then  FAlignment := taLeftJustify
  else  FAlignment := taRightJustify;
  HoritzResize (B);
  FAlignment := PrevAlignment;
end {HoritzResizeB};

procedure TTntResizeBtnArray.HoritzResizeL (B: array of TButton);
var
  PrevAlignment: TAlignment;
begin
  PrevAlignment := FAlignment;
  FAlignment := taLeftJustify;
  HoritzResize (B);
  FAlignment := PrevAlignment;
end {HoritzResizeL};

procedure TTntResizeBtnArray.HoritzResizeR (B: array of TButton);
var
  PrevAlignment: TAlignment;
begin
  PrevAlignment := FAlignment;
  FAlignment := taRightJustify;
  HoritzResize (B);
  FAlignment := PrevAlignment;
end {HoritzResizeR};

procedure TTntResizeBtnArray.HoritzResizeSpac (B: array of TButton;
  Spacing: Word);
var
  I: Integer;
begin
  Resize (B);
  for  I := 0 to High(B)  do  begin
    if  FAlignment = taLeftJustify  then  begin
      if  I > 0
      then  B[I].Left := ControlRight (B[I-1]) + Spacing;
    end
    else begin
      if  I = 0
      then  B[0].Left := FirstR - B[0].Width
      else  B[I].Left := B[I-1].Left - (B[I].Width + Spacing);
    end;
  end;
end;

function TTntResizeBtnArray.HoritzSize (B: array of TButton): Word;
var
  I: Integer;
  GlyphW: Word;
  TotalCaptW, W: Word;
begin
  Result := 0;
  for  I := 0 to High(B)  do  begin
    if  B[I] is TTntBitBtn  then  begin
      with B[I] as TTntBitBtn do  begin
        GlyphW := Glyph.Width div NumGlyphs;
        TotalCaptW := GetCaptionWidth (Font, Caption);
        if  Layout in [blGlyphLeft, blGlyphRight]  then  begin
          if  GlyphW > 0
          then  W := TotalCaptW + Spacing + GlyphW
          else  W := TotalCaptW;
        end
        else
          W := Max (TotalCaptW, GlyphW);
        Inc (Result, Max (W + (Spacing * 4), FMinWidth));
      end;
    end
    else if  B[I] is TTntButton  then  begin
      with B[I] as TTntButton do  begin
        TotalCaptW := GetCaptionWidth (Font, Caption);
        Inc (Result, Max (TotalCaptW + 16, FMinWidth));
      end;
    end;
  end;
  Inc (Result, High(B) * FBBSpacing);
end {HoritzSize};

procedure TTntResizeBtnArray.VertResize (B: array of TButton);
var
  I: Integer;
  MaxWidth: Word;
begin
  Resize (B);
  if FFitToMaxWidth then  begin
    MaxWidth := 0;
    for  I := 0 to High(B)  do  begin
      MaxWidth := Max (MaxWidth, B[I].Width);
    end;
    for  I := 0 to High(B)  do  begin
      B[I].Width := MaxWidth;
    end;
  end;
  if  FAlignment = taRightJustify  then  begin
    for  I := 0 to High(B)  do  begin
      B[I].Left := FirstR - B[I].Width;
    end;
  end;
end {VertResize};

procedure TTntResizeBtnArray.VertResizeB (B: array of TButton;  LtoR: Boolean);
var
  PrevAlignment: TAlignment;
begin
  PrevAlignment := FAlignment;
  if LtoR
  then  FAlignment := taLeftJustify
  else  FAlignment := taRightJustify;
  VertResize (B);
  FAlignment := PrevAlignment;
end {VertResizeB};

procedure TTntResizeBtnArray.VertResizeL (B: array of TButton);
var
  PrevAlignment: TAlignment;
begin
  PrevAlignment := FAlignment;
  FAlignment := taLeftJustify;
  VertResize (B);
  FAlignment := PrevAlignment;
end {VertResizeL};

procedure TTntResizeBtnArray.VertResizeR (B: array of TButton);
var
  PrevAlignment: TAlignment;
begin
  PrevAlignment := FAlignment;
  FAlignment := taRightJustify;
  VertResize (B);
  FAlignment := PrevAlignment;
end {VertResizeR};

{--------------------------------------}

end.

