{---------------------------------------------------------------------

 (c) Copyleft or copyright under terms of GNU General Public License
 2003-2006  Jordi March i Nogu   jmarch@comg.es
 03/01/2006 version

----------------------------------------------------------------------}

unit TntStrAnal;

interface

uses
  Classes;

function PosChForwardW (const S: WideString;  Delim: WideChar): Integer;
function PosChBackwardW (const S: WideString;  Delim: WideChar): Integer;
function StrUntilDelimiterW (var InStr: WideString;
  const Delim: WideString): WideString;
function StrBkUntilDelimiterW (var InStr: WideString;
  Delim: WideChar): WideString;
function StrBkUntilSDelimiterW (var InStr: WideString;
  const Delim: WideString): WideString;
function StrUntilDelimitersW (var InStr: WideString;
  const DelStr: WideString): WideString;
function StrUntilDelimitersAndDelimiterW (var InStr: WideString;
  const DelStr: WideString;  out Delimiter: WideChar): WideString;
function StrUntilNoDelimitersW (var InStr: WideString;
  const DelStr: WideString): WideString;
function StrDelimitersXAtBeginW (const InStr, DelStr: WideString): Boolean;
function StrUntilDelimitersXW (var InStr: WideString;
  const DelStr: WideString): WideString;
function StrUntilDelimitersXAndDelimiterW (var InStr: WideString;
  const DelStr: WideString;  out Delimiter: WideChar): WideString;
function StrReplaceSimpleW (InStr: WideString;
  const SearchStr, ReplaceStr: WideString): WideString;
function StrReplacedSimpleW (var S: WideString;
  const SearchStr, ReplaceStr: WideString): Boolean;
function StrReplaceTextW (InStr: WideString;
  const SearchStr, ReplaceStr: WideString;
  var Changed: Boolean;  CaseSensitive: Boolean): WideString;
function StrReplaceBetweenDelimW (InStr: WideString;
  const SearchStr, ReplaceStr: WideString;
  const DelStr: WideString): WideString;
function StrIsAbbreviatedW (const S: WideString): Boolean;
function NotFoundCharsInParentW (const Test, Parent: WideString): WideString;
function ContainsAnyCharW (const Substr, S: WideString): Boolean;
function OnlySubstrCharW (const Substr, S: WideString): Boolean;
function StrSearchW (const Substr, S: WideString;
  const Index: Integer): Integer;
function StrCommonW (const Str1, Str2: WideString): WideString;

implementation

uses
  SysUtils, Math;

function StrFromPosToYesOrNoDelimiterW
 (const InStr, DelStr: WideString;  Span: Boolean;
  var PosB: Integer;  out PosE: Integer): WideString;
{ Returns string with all characters from InStr PosB(egin) position
  until anyone char equal (Span=False) or different (Span=True)
     to anyone char from DelStr }
var
  Done: Boolean;
begin
  Result := '';
  PosE := 0;
  if  PosB > 0  then  begin
    Done := False;
    while  (PosB <= Length(InStr)) and not Done  do  begin
      PosE := Pos (InStr[PosB], DelStr);
      if  Span xor (PosE = 0)  then  begin
        Result := Result + InStr[PosB];
        Inc (PosB);
      end
      else
        Done := True;
    end;
    if  (Span xor (PosE = 0)) or (PosB > Length(InStr))  then  PosB := 0;
  end;
end {StrFromPosToYesOrNoDelimiterW};

function PosChForwardW (const S: WideString;  Delim: WideChar): Integer;
var
  I, N: Integer;
begin
  N := 0;
  for  I := 1 to Length (S) do
    if  S[I] = Delim
    then  Inc (N);
  Result := N;
end {PosChForwardW};

function PosChBackwardW (const S: WideString;  Delim: WideChar): Integer;
var
  I, J: Integer;
begin
  J := Length (S);
  Result := 0;
  for  I := J downto 1  do
    if  S[I] = Delim  then  begin
      Result := I;
      Exit;
    end;
end {PosChBackwardW};

function StrUntilDelimiterW (var InStr: WideString;
  const Delim: WideString): WideString;
{ Returns string with all characters from InStr beginning
  until anyone char equal to anyone char from Delim }
var
  I: Integer;
begin
  I := Pos (Delim, InStr);
  if  I = 0  then  begin
    Result := InStr;
    InStr := '';
  end
  else  begin
    Result := Copy (InStr, 1, I-1);
    Delete (InStr, 1, Length(Delim)+I-1);
  end;
end {StrUntilDelimiterW};

function StrBkUntilDelimiterW (var InStr: WideString;
  Delim: WideChar): WideString;
{ Returns string with all characters from InStr ending
  until anyone char equal to anyone char from Delim }
var
  J, Pos: Integer;
begin
  J := Length (InStr);
  Pos := PosChBackwardW (InStr, Delim);
  if  Pos = 0  then  begin
    Result := InStr;
    InStr := '';
  end
  else  begin
    Result := Copy (InStr, Pos+1, J-Pos);
    Delete (InStr, Pos, J-Pos+1);
  end;
end {StrBkUntilDelimiter};

function StrBkUntilSDelimiterW (var InStr: WideString;
  const Delim: WideString): WideString;
var
  I: Integer;
  TransStr, ResStr: WideString;
  Count: Integer;
begin
  I := Pos (Delim, InStr);
  if  I = 0  then  begin
    Result := InStr;
    InStr := '';
  end
  else  begin
    TransStr := InStr;
    Count := 0;
    repeat
      ResStr := StrUntilDelimiterW (TransStr, Delim);
      Inc (Count, Length (ResStr)+Length(Delim));
    until  Pos (Delim, TransStr) = 0;
    Result := TransStr;
    InStr := Copy (InStr, 1, Count-Length(Delim));
  end;
end {StrBkUntilSDelimiter};

function StrUntilDelimitersW (var InStr: WideString;
  const DelStr: WideString): WideString;
{ Returns string with all characters from InStr beginning
  until anyone char equal to anyone char from Delim }
var
  Cp, Dp: Integer;
begin
  Cp := 1;
  Result := StrFromPosToYesOrNoDelimiterW (InStr, DelStr, False, Cp, Dp);
  if  Cp = 0  then
    InStr := ''
  else
    Delete (InStr, 1, Cp-1);
end {StrUntilDelimitersW};

function StrUntilDelimitersAndDelimiterW (var InStr: WideString;
  const DelStr: WideString;  out Delimiter: WideChar): WideString;
{ Returns string with all characters from InStr beginning
  until anyone char equal to anyone char from Delim }
var
  Cp, Dp: Integer;
begin
  Cp := 1;
  Delimiter := #0;
  Result := StrFromPosToYesOrNoDelimiterW (InStr, DelStr, False, Cp, Dp);
  if  Cp = 0  then
    InStr := ''
  else  begin
    Delimiter := InStr[Cp];
    Delete (InStr, 1, Cp-1);
  end;
end {StrUntilDelimitersAndDelimiterW};

function StrUntilNoDelimitersW (var InStr: WideString;
  const DelStr: WideString): WideString;
{ Returns string with all characters from InStr beginning
  until anyone char different to anyone char from DelStr }
var
  Cp, Dp: Integer;
begin
  Cp := 1;
  Result := StrFromPosToYesOrNoDelimiterW (InStr, DelStr, True, Cp, Dp);
  if  Cp = 0  then
    InStr := ''
  else
    Delete (InStr, 1, Cp-1);
end {StrUntilNoDelimitersW};

function StrDelimitersXAtBeginW (const InStr, DelStr: WideString): Boolean;
var
  J: Word;
begin
  Result := False;
  if  InStr = ''
  then  Exit;
  for  J := 1 to Length (DelStr)  do
    if  DelStr[J] = InStr[1]  then  begin
      Result := True;
      Exit;
    end;
end {StrDelimitersXAtBeginW};

function StrUntilDelimitersXW (var InStr: WideString;
  const DelStr: WideString): WideString;
{ Returns WideString with all characters from InStr beginning
 (or past anyone char equal to anyone char from Delim)
  until past anyone char equal to anyone char from Delim }
begin
  Result := '';
  if  InStr = ''
  then  Exit;
  if  StrDelimitersXAtBeginW (InStr, DelStr)
  then  StrUntilNoDelimitersW (InStr, DelStr);
  Result := StrUntilDelimitersW (InStr, DelStr);
  StrUntilNoDelimitersW (InStr, DelStr);
end {StrUntilDelimitersXW};

function StrUntilDelimitersXAndDelimiterW (var InStr: WideString;
  const DelStr: WideString;  out Delimiter: WideChar): WideString;
{ Returns WideString with all characters from InStr beginning
 (or past anyone char equal to anyone char from Delim)
  until past anyone char equal to anyone char from Delim }
begin
  Result := '';
  Delimiter := #0;
  if  InStr = ''
  then  Exit;
  if  StrDelimitersXAtBeginW (InStr, DelStr)
  then  StrUntilNoDelimitersW (InStr, DelStr);
  Result := StrUntilDelimitersAndDelimiterW (InStr, DelStr, Delimiter);
  StrUntilNoDelimitersW (InStr, DelStr);
end {StrUntilDelimitersXAndDelimiterW};

function StrReplaceSimpleW (InStr: WideString;
  const SearchStr, ReplaceStr: WideString): WideString;
var
  I: Integer;
begin
  if  SearchStr = ''  then  begin
    Result := InStr;
    Exit;
  end
  else
    Result := '';
  while  InStr <> ''  do  begin
    I := Pos (SearchStr, InStr);
    if  I > 0  then  begin
      Result := Concat (Result, Copy (InStr, 1, I-1), ReplaceStr);
      InStr := Copy (InStr, I+Length(SearchStr), MaxInt);
    end
    else  begin
      Result := Concat (Result, InStr);
      InStr := '';
    end;
  end;
end {StrReplaceSimpleW};

function StrReplacedSimpleW (var S: WideString;
  const SearchStr, ReplaceStr: WideString): Boolean;
var
  SRes: WideString;
begin
  SRes := StrReplaceSimpleW (S, SearchStr, ReplaceStr);
  Result := SRes <> S;
  S := SRes;
end {StrReplacedSimpleW};

function StrReplaceTextW (InStr: WideString;
  const SearchStr, ReplaceStr: WideString;
  var Changed: Boolean;  CaseSensitive: Boolean): WideString;
var
  I: Integer;
begin
  Changed := False;
  if  SearchStr = ''  then  begin
    Result := InStr;
    Exit;
  end
  else  begin
    Result := '';
    while  InStr <> ''  do  begin
      if CaseSensitive
      then  I := Pos (SearchStr, InStr)
      else  I := Pos (WideUpperCase(SearchStr), WideUpperCase(InStr));
      if  I > 0  then  begin
        Result := Concat (Result, Copy (InStr, 1, I-1), ReplaceStr);
        Changed := True;
        InStr := Copy (InStr, I+Length(SearchStr), MaxInt);
      end
      else  begin
        Result := Concat (Result, InStr);
        InStr := '';
      end;
    end;
  end;
end {StrReplaceTextW};

function StrReplaceBetweenDelimW (InStr: WideString;
  const SearchStr, ReplaceStr: WideString;
  const DelStr: WideString): WideString;
var
  C, ResultStr: WideString;
begin
  if  SearchStr = ''  then  begin
    Result := InStr;
    Exit;
  end
  else
    ResultStr := '';
  while  InStr <> ''  do  begin
    C := StrUntilNoDelimitersW (InStr, DelStr);
    ResultStr := Concat (ResultStr, C);
    C := StrUntilDelimitersW (InStr, DelStr);
    if  C = SearchStr  then  C := ReplaceStr;
    ResultStr := Concat (ResultStr, C);
  end;
  Result := ResultStr;
end {StrReplaceBetweenDelimW};

function StrIsAbbreviatedW (const S: WideString): Boolean;
var
  S1, S2: WideString;
begin
  S1 := S;
  Result := False;
  while  S1 <> ''  do  begin
    S2 := StrUntilDelimiterW (S1, ' ');
    if  (Length (S2) = 1) or
        ((Length (S2) = 2) and (S2[2]='.'))  then  begin
      Result := True;
      Exit;
    end;
  end;
end {StrIsAbbreviatedW};

function NotFoundCharsInParentW (const Test, Parent: WideString): WideString;
var
  I: Integer;
begin
  Result := '';
  for  I := 1 to Length(Test) do
    if  Pos (Test[I], Parent) = 0
    then  Result := Concat (Result, Test[I]);
end {NotFoundCharsInParentW};

function ContainsAnyCharW (const Substr, S: WideString): Boolean;
var
  I: Integer;
begin
  Result := False;
  if  (Substr = '') or (S = '')
  then  Exit;
  for  I := 1 to Length(Substr) do
    if  Pos (Substr[I], S) <> 0  then begin
      Result := True;
      Exit;
    end;
end {ContainsAnyCharW};

function OnlySubstrCharW (const Substr, S: WideString): Boolean;
var
  I: Integer;
begin
  Result := True;
  if  (Substr = '') or (S = '')
  then  Exit;
  for  I := 1 to Length(S) do
    if  Pos (S[I], Substr) = 0  then begin
      Result := False;
      Exit;
    end;
end {OnlySubstrCharW};

function StrSearchW (const Substr, S: WideString;
  const Index: Integer): Integer;
var
  Res: Integer;
begin
  Res := Pos (Substr, Copy (S, Index, MaxInt));
  if  Res <> 0
  then  Res := Res + (Index-1);
  Result := Res;
end {StrSearchW};

function StrCommonW (const Str1, Str2: WideString): WideString;   
var
  MinL, I: Integer;
begin
  Result := '';
  if  (Str1 = '') or (Str2 = '')
  then  Exit;
  MinL := Min (Length (Str1), Length (Str2));
  for  I := 1 to MinL  do  begin
    if  Str1[I] = Str2[I]
    then  Result := Concat (Result, Str1[I])
    else  Exit;
  end;
end {StrCommonW};

end.
